@@ -1,4 +1,6 @@
 /*
+ * $Id: alive.h,v 1.4 2003/01/01 00:14:34 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  * Alive status checking ping/pongs.
@@ -23,8 +25,8 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __alive_h__
-#define __alive_h__
+#ifndef _alive_h_
+#define _alive_h_
 
 #include <glib.h>
 
@@ -38,6 +40,7 @@ gpointer alive_make(struct gnutella_node *n, gint max);
 void alive_free(gpointer obj);
 gboolean alive_send_ping(gpointer obj);
 gboolean alive_ack_ping(gpointer obj, guchar *muid);
+void alive_get_roundtrip_ms(gpointer obj, guint32 *avg, guint32 *last);
 
-#endif /* __alive_h__ */
+#endif /* _alive_h_ */
 
