@@ -1,4 +1,6 @@
 /*
+ * $Id: ban.h,v 1.8 2003/04/27 20:49:16 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  * Banning control.
@@ -23,8 +25,8 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __ban_h__
-#define __ban_h__
+#ifndef _ban_h_
+#define _ban_h_
 
 #include <glib.h>
 
@@ -35,6 +37,9 @@ void ban_close(void);
 gint ban_allow(guint32 ip);
 void ban_force(struct gnutella_socket *s);
 gint ban_delay(guint32 ip);
+void ban_max_recompute(void);
+
+const gchar *ban_vendor(const gchar *vendor);
 
 /*
  * Return codes for ban_allow().
@@ -44,7 +49,7 @@ gint ban_delay(guint32 ip);
 #define BAN_FIRST	1		/* Initial banning, send polite denial */
 #define BAN_FORCE	2		/* Force banning, don't send back anything */
 
-#endif	/* __ban_h__ */
+#endif	/* _ban_h_ */
 
 /* vi: set ts=4: */
 
