@@ -1,4 +1,6 @@
 /*
+ * $Id: filter.c,v 1.63 2003/05/03 21:17:51 cbiere Exp $
+ *
  * Copyright (c) 2001-2002, Raphael Manfredi, Richard Eckart
  *
  * GUI filtering functions.
@@ -29,14 +31,16 @@
 
 #include "filter.h"
 #include "filter_gui.h"
-#include "misc.h"
-#include "interface.h"
-#include "search.h"
+#include "search_gui.h"
 #include "gtk-missing.h"
 
-#define BIT_TO_BOOL(m) ((m == 0) ? FALSE : TRUE)
-
+#ifdef USE_GTK2
+#include "interface-glade2.h"
+#else
+#include "interface-glade1.h"
+#endif
 
+RCSID("$Id: filter.c,v 1.63 2003/05/03 21:17:51 cbiere Exp $");
 
 /*
  * If FILTER_HIDE_ON_CLOSE is defined, the filter dialog is only hidden
@@ -66,6 +70,11 @@ static shadow_t *shadow_find(filter_t *s);
 static int filter_apply(filter_t *, record_t *, filter_result_t *);
 static void filter_remove_rule(filter_t *f, rule_t *r);
 static void filter_free(filter_t *filter);
+static void filter_refresh_display(GList *filter_list);
+
+void dump_ruleset(GList *ruleset);
+void dump_filter(filter_t *filter);
+void dump_shadow(shadow_t *shadow);
 
 /*
  * Public variables
@@ -180,11 +189,11 @@ static shadow_t *shadow_find(filter_t *f)
         (shadow_filters, f, (GCompareFunc) shadow_filter_eq);
 
     if (l != NULL) {
-        if (dbg >= 6)
+        if (gui_debug >= 6)
             printf("shadow found for: %s\n", f->name);
         return l->data;
     } else {
-        if (dbg >= 6)
+        if (gui_debug >= 6)
             printf("no shadow found for: %s\n", f->name);
         return NULL;
     }
@@ -205,7 +214,7 @@ static shadow_t *shadow_new(filter_t *f)
     g_assert(f != NULL);
     g_assert(f->name != NULL);
 
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("creating shadow for: %s\n", f->name);
 
     shadow = g_new0(shadow_t, 1);
@@ -241,7 +250,7 @@ static void shadow_cancel(shadow_t *shadow)
     g_assert(shadow != NULL);
     g_assert(shadow->filter != NULL);
 
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("cancel shadow for filter: %s\n", shadow->filter->name);
 
     for (r = shadow->added; r != NULL; r = r->next)
@@ -279,7 +288,7 @@ static void shadow_commit(shadow_t *shadow)
 
     realf = shadow->filter;
 
-    if (dbg >= 6) {
+    if (gui_debug >= 6) {
         printf("committing shadow for filter:\n");
         dump_shadow(shadow);
     }
@@ -332,7 +341,7 @@ static void shadow_commit(shadow_t *shadow)
     shadow_filters = g_list_remove(shadow_filters, shadow); 
     g_free(shadow);
 
-    if (dbg >= 6) {
+    if (gui_debug >= 6) {
         printf("after commit filter looks like this\n");
         dump_filter(realf);
     }
@@ -345,13 +354,13 @@ static void shadow_commit(shadow_t *shadow)
  *
  * Regenerates the filter tree and rules display from after a apply/revert.
  */
-void filter_refresh_display(GList *filters)
+static void filter_refresh_display(GList *filter_list)
 {
     GList *l;
 
     filter_gui_freeze_filters();
     filter_gui_filter_clear_list();
-    for (l = filters; l != NULL; l = l->next) {
+    for (l = filter_list; l != NULL; l = l->next) {
         filter_t *filter = (filter_t *)l->data;
         shadow_t *shadow;
         GList *ruleset;
@@ -376,9 +385,13 @@ void filter_refresh_display(GList *filters)
  *
  * Open and initialize the filter dialog.
  */
-void filter_open_dialog() {
+void filter_open_dialog(void) {
+    search_t *current_search;
+
+    current_search = search_gui_get_current_search();
+
     if (filter_dialog == NULL) {
-        filter_dialog = create_dlg_filters();
+        filter_dialog = filter_gui_create_dlg_filters();
         g_assert(filter_dialog != NULL);
    
         filter_gui_init();
@@ -391,8 +404,6 @@ void filter_open_dialog() {
         filter_set(NULL);
     }
 
-    filter_gui_set_default_policy(filter_default_policy);
-   
     filter_gui_show_dialog();
 }
 
@@ -408,19 +419,22 @@ void filter_close_dialog(gboolean commit)
 {
     if (commit) {
         filter_apply_changes();
-        filter_default_policy = (gint) option_menu_get_selected_data
-            (optionmenu_filter_default_policy);
     } else
         filter_revert_changes();
 
     if (filter_dialog != NULL) {
+        guint32 coord[4] = {0, 0, 0, 0};
+
         gdk_window_get_root_origin
-            (filter_dialog->window, &flt_dlg_x, &flt_dlg_y);
-        gdk_window_get_size
-            (filter_dialog->window, &flt_dlg_w, &flt_dlg_h);
+            (filter_dialog->window, &coord[0], &coord[1]);
+        gdk_drawable_get_size
+            (filter_dialog->window, &coord[2], &coord[3]);
+
+        gui_prop_set_guint32(PROP_FILTER_DLG_COORDS, coord, 0, 4);
         
         filter_main_divider_pos =
-            gtk_paned_get_position(GTK_PANED(hpaned_filter_main));
+            gtk_paned_get_position
+                (GTK_PANED(lookup_widget(filter_dialog, "hpaned_filter_main")));
 
 #ifdef FILTER_HIDE_ON_CLOSE        
         gtk_widget_hide(filter_dialog);
@@ -492,6 +506,7 @@ rule_t *filter_new_text_rule(gchar *match, gint type,
     r->u.text.case_sensitive = case_sensitive;
     r->u.text.type           = type;
     r->u.text.match          = g_strdup(match);
+    r->u.text.matchlen       = strlen(match);
     set_flags(r->flags, RULE_FLAG_VALID);
 
     if (!r->u.text.case_sensitive)
@@ -517,8 +532,8 @@ rule_t *filter_new_text_rule(gchar *match, gint type,
 			REG_EXTENDED|REG_NOSUB|(r->u.text.case_sensitive ? 0 : REG_ICASE));
 
 		if (err) {
-			char buf[1000];
-			regerror(err, re, buf, 1000);
+			gchar regbuf[1000];
+			regerror(err, re, regbuf, sizeof(regbuf));
 
 			g_warning(
                 "problem in regular expression: %s"
@@ -742,7 +757,7 @@ void filter_close_search(search_t *s)
     g_assert(s != NULL);
     g_assert(s->filter != NULL);
 
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("closing search (freeing filter): %s\n", s->query);
 
     shadow = shadow_find(s->filter);
@@ -784,7 +799,7 @@ void filter_close_search(search_t *s)
  * Then shadow->current will be set as the new filter for that
  * search.
  */
-void filter_apply_changes() 
+void filter_apply_changes(void) 
 {
     GList *s;
 
@@ -827,12 +842,12 @@ void filter_apply_changes()
  * Free the ressources for all added filters and forget all shadows.
  * A running session will not be ended by this.
  */
-void filter_revert_changes()
+void filter_revert_changes(void)
 {
     GList *s;
     gint n;
 
-    if (dbg >= 5)
+    if (gui_debug >= 5)
         printf("Canceling all changes to filters/rules\n");
 
     filter_gui_freeze_filters();
@@ -912,42 +927,42 @@ gchar *filter_rule_condition_to_gchar(const rule_t *r)
     case RULE_TEXT:
         switch (r->u.text.type) {
         case RULE_TEXT_PREFIX:
-           	g_snprintf(
+           	gm_snprintf(
                 tmp, sizeof(tmp), 
                 "If filename begins with \"%s\" %s",
                 r->u.text.match,
                 r->u.text.case_sensitive ? "(case sensitive)" : "");
             break;
         case RULE_TEXT_WORDS:
-           	g_snprintf(
+           	gm_snprintf(
                 tmp, sizeof(tmp), 
                 "If filename contains the words \"%s\" %s",
                 r->u.text.match,
                 r->u.text.case_sensitive ? "(case sensitive)" : "");
             break;
         case RULE_TEXT_SUFFIX:
-          	g_snprintf(
+          	gm_snprintf(
                 tmp, sizeof(tmp), 
                 "If filename ends with \"%s\" %s",
                 r->u.text.match,
                 r->u.text.case_sensitive ? "(case sensitive)" : "");
             break;
         case RULE_TEXT_SUBSTR:
-           	g_snprintf(
+           	gm_snprintf(
                 tmp, sizeof(tmp), 
                 "If filename contains the substring \"%s\" %s",
                 r->u.text.match,
                 r->u.text.case_sensitive ? "(case sensitive)" : "");
             break;
         case RULE_TEXT_REGEXP:
-           	g_snprintf(
+           	gm_snprintf(
                 tmp, sizeof(tmp), 
                 "If filename matches the regex \"%s\" %s",
                 r->u.text.match,
                 r->u.text.case_sensitive ? "(case sensitive)" : "");
             break;
         case RULE_TEXT_EXACT:
-           	g_snprintf(
+           	gm_snprintf(
                 tmp, sizeof(tmp), 
                 "If filename is \"%s\" %s",
                 r->u.text.match,
@@ -966,7 +981,7 @@ gchar *filter_rule_condition_to_gchar(const rule_t *r)
             mask = g_strdup(ip_to_gchar(r->u.ip.mask));
             addr = g_strdup(ip_to_gchar(r->u.ip.addr));
 
-            g_snprintf(tmp, sizeof(tmp), 
+            gm_snprintf(tmp, sizeof(tmp), 
                 "If IP address matches %s/%s", addr, mask);
 
             g_free(addr);
@@ -975,13 +990,13 @@ gchar *filter_rule_condition_to_gchar(const rule_t *r)
         break;
     case RULE_SIZE:
 		if (r->u.size.lower == 0)
-			g_snprintf(tmp, sizeof(tmp),
+			gm_snprintf(tmp, sizeof(tmp),
 				"If filesize is smaller than %d (%s)",
-				r->u.size.upper, short_size(r->u.size.upper));
+				(gint) r->u.size.upper, short_size(r->u.size.upper));
 		else if (r->u.size.upper == r->u.size.lower)
-			g_snprintf(tmp, sizeof(tmp),
+			gm_snprintf(tmp, sizeof(tmp),
 				"If filesize is exactly %d (%s)",
-				r->u.size.upper, short_size(r->u.size.upper));
+				(gint) r->u.size.upper, short_size(r->u.size.upper));
         else {
             gchar *s1;
             gchar *s2;
@@ -989,9 +1004,9 @@ gchar *filter_rule_condition_to_gchar(const rule_t *r)
             s1 = g_strdup(short_size(r->u.size.lower));
             s2 = g_strdup(short_size(r->u.size.upper));
     
-			g_snprintf(tmp, sizeof(tmp),
+			gm_snprintf(tmp, sizeof(tmp),
 				"If filesize is between %d and %d (%s - %s)",
-				r->u.size.lower, r->u.size.upper, s1, s2);
+				(gint) r->u.size.lower, (int)r->u.size.upper, s1, s2);
 
             g_free(s1);
             g_free(s2);
@@ -999,13 +1014,13 @@ gchar *filter_rule_condition_to_gchar(const rule_t *r)
         break;
     case RULE_SHA1:
         if (r->u.sha1.hash != NULL) {
-            g_snprintf(tmp, sizeof(tmp), "If urn:sha1 is same as for \"%s\"",
+            gm_snprintf(tmp, sizeof(tmp), "If urn:sha1 is same as for \"%s\"",
                 r->u.sha1.filename);
         } else 
-            g_snprintf(tmp, sizeof(tmp), "If urn:sha1 is not available");
+            gm_snprintf(tmp, sizeof(tmp), "If urn:sha1 is not available");
         break;
     case RULE_JUMP:
-       	g_snprintf(
+       	gm_snprintf(
             tmp, sizeof(tmp), 
             "Always");
         break;
@@ -1062,12 +1077,12 @@ gchar *filter_rule_condition_to_gchar(const rule_t *r)
             }
 
             if (b)
-                g_snprintf(
+                gm_snprintf(
                     tmp, sizeof(tmp),
                     "If flag %s%s%s%s%s", 
                     busy_str, s1, push_str, s2, stable_str);
             else
-                 g_snprintf(
+                 gm_snprintf(
                     tmp, sizeof(tmp),
                     "Always (all flags ignored)");
         }
@@ -1121,12 +1136,12 @@ gchar *filter_rule_condition_to_gchar(const rule_t *r)
             }
      
             if (b)
-                g_snprintf(
+                gm_snprintf(
                     tmp, sizeof(tmp),
                     "If flag %s%s%s", 
                     display_str, s1, download_str);
             else
-                 g_snprintf(
+                 gm_snprintf(
                     tmp, sizeof(tmp),
                     "Always (all states ignored)");
         }
@@ -1155,7 +1170,7 @@ gchar *filter_rule_to_gchar(rule_t *r)
 
     cond = g_strdup(filter_rule_condition_to_gchar(r));
 
-	g_snprintf(f_tmp, sizeof(f_tmp), "%s%s %s jump to \"%s\"", 
+	gm_snprintf(f_tmp, sizeof(f_tmp), "%s%s %s jump to \"%s\"", 
         RULE_IS_NEGATED(r) ? "(Negated) " : "",
         RULE_IS_ACTIVE(r) ? "" : "(deactivated)",
         cond,
@@ -1351,7 +1366,7 @@ void filter_free_rule(rule_t *r)
 {
     g_assert(r != NULL);
 
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("freeing rule: %s\n", filter_rule_to_gchar(r));
 
     switch (r->type) {
@@ -1429,7 +1444,7 @@ void filter_append_rule(filter_t *f, rule_t * const r)
      */
     f->ruleset = g_list_append(f->ruleset, r);
     r->target->refcount ++;
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("increased refcount on \"%s\" to %d\n",
             r->target->name, r->target->refcount);
 
@@ -1445,7 +1460,7 @@ void filter_append_rule(filter_t *f, rule_t * const r)
     if (target_shadow != NULL) {
         target_shadow->refcount ++;
 
-        if (dbg >= 6)
+        if (gui_debug >= 6)
             printf("increased refcount on shadow of \"%s\" to %d\n",
                 target_shadow->filter->name, target_shadow->refcount);
     }
@@ -1482,7 +1497,7 @@ void filter_append_rule_to_session(filter_t *f, rule_t * const r)
     g_assert(f != NULL);
     g_assert(r->target != NULL);
 
-    if (dbg >= 4)
+    if (gui_debug >= 4)
         printf("appending rule to filter: %s <- %s (%p)\n",
             f->name, filter_rule_to_gchar(r), r->target);
 
@@ -1520,7 +1535,7 @@ void filter_append_rule_to_session(filter_t *f, rule_t * const r)
         target_shadow = shadow_new(r->target);
 
     target_shadow->refcount ++;
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("increased refcount on shadow of \"%s\" to %d\n",
             target_shadow->filter->name, target_shadow->refcount);
 
@@ -1644,7 +1659,7 @@ void filter_remove_rule(filter_t *f, rule_t *r)
     if (in_filter) {
         r->target->refcount --;
 
-        if (dbg >= 6)
+        if (gui_debug >= 6)
             printf("decreased refcount on \"%s\" to %d\n",
                 r->target->name, r->target->refcount);
     }
@@ -1653,7 +1668,7 @@ void filter_remove_rule(filter_t *f, rule_t *r)
         if (target_shadow != NULL) {
             target_shadow->refcount --;
         
-            if (dbg >= 6)
+            if (gui_debug >= 6)
                 printf("decreased refcount on shadow of \"%s\" to %d\n",
                     target_shadow->filter->name, target_shadow->refcount);
         }
@@ -1698,7 +1713,7 @@ void filter_remove_rule_from_session(filter_t *f, rule_t * const r)
     g_assert(r != NULL);
     g_assert(f != NULL);
 
-    if (dbg >= 4)
+    if (gui_debug >= 4)
         printf("removing rule in filter: %s -> %s\n", 
             f->name, filter_rule_to_gchar(r));
 
@@ -1723,7 +1738,7 @@ void filter_remove_rule_from_session(filter_t *f, rule_t * const r)
         target_shadow = shadow_new(r->target);
 
     target_shadow->refcount --;
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("decreased refcount on shadow of \"%s\" to %d\n",
             target_shadow->filter->name, target_shadow->refcount);
 
@@ -1734,7 +1749,7 @@ void filter_remove_rule_from_session(filter_t *f, rule_t * const r)
          * not committed. We removed it from the added list
          * and free the ressources.
          */
-        if (dbg >= 4)
+        if (gui_debug >= 4)
             printf("while removing from %s: removing from added: %s\n",
                 f->name, filter_rule_to_gchar(r));
         shadow->added = g_list_remove(shadow->added, r);
@@ -1746,7 +1761,7 @@ void filter_remove_rule_from_session(filter_t *f, rule_t * const r)
          */
         g_assert(g_list_find(shadow->removed, r) == NULL);
 
-        if (dbg >= 4)
+        if (gui_debug >= 4)
             printf("while removing from %s: adding to removed: %s\n",
                 f->name, filter_rule_to_gchar(r));
       
@@ -1799,7 +1814,7 @@ void filter_replace_rule_in_session(filter_t *f,
     filter = g_list_find(shadow->current, old_rule);
     g_assert(filter != NULL);
 
-    if (dbg >= 4) {
+    if (gui_debug >= 4) {
         gchar * f1 = g_strdup(filter_rule_to_gchar(old_rule));
         gchar * f2 = g_strdup(filter_rule_to_gchar(new_rule));
 
@@ -1818,7 +1833,7 @@ void filter_replace_rule_in_session(filter_t *f,
         target_shadow = shadow_new(old_rule->target);
 
     target_shadow->refcount --;
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("decreased refcount on shadow of \"%s\" to %d\n",
             target_shadow->filter->name, target_shadow->refcount);
 
@@ -1858,7 +1873,7 @@ void filter_replace_rule_in_session(filter_t *f,
         target_shadow = shadow_new(new_rule->target);
 
     target_shadow->refcount ++;
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("increased refcount on shadow of \"%s\" to %d\n",
             target_shadow->filter->name, target_shadow->refcount);
         
@@ -1892,10 +1907,13 @@ void filter_adapt_order(void)
     GList *neworder = NULL;
     gint row;
     shadow_t *shadow;
+    GtkCList *clist;
 
     if (!work_filter || filter_dialog == NULL)
         return;
    
+    clist = GTK_CLIST(lookup_widget(filter_dialog, "clist_filter_rules"));
+
     /*
      * Create a new shadow if necessary.
      */
@@ -1911,10 +1929,10 @@ void filter_adapt_order(void)
      */
     g_list_free(shadow->current);
 
-    for (row = 0; row < GTK_CLIST(clist_filter_rules)->rows; row ++) {
+    for (row = 0; row < clist->rows; row ++) {
         filter_t *f;
 
-        f = gtk_clist_get_row_data(GTK_CLIST(clist_filter_rules), row);
+        f = gtk_clist_get_row_data(clist, row);
         g_assert(f != NULL);
         
         neworder = g_list_append(neworder, f);
@@ -1930,7 +1948,7 @@ void filter_adapt_order(void)
     (prop_count) ++;                                              \
     (r)->target->match_count ++;                                  \
                                                                   \
-    if (dbg >= 10)                                                 \
+    if (gui_debug >= 10)                                                 \
         printf("matched rule: %s\n", filter_rule_to_gchar((r)));
    
 /*
@@ -1947,7 +1965,7 @@ static int filter_apply
 	char *l_name;
     GList *list;
     gint prop_count = 0;
-    gboolean abort = FALSE;
+    gboolean do_abort = FALSE;
 
     g_assert(filter != NULL);
     g_assert(rec != NULL);
@@ -1969,14 +1987,14 @@ static int filter_apply
 	strlower(l_name, rec->name);
 
 	list = g_list_first(list);
-	while ((list != NULL) && (res->props_set < MAX_FILTER_PROP) && !abort) {
+	while ((list != NULL) && (res->props_set < MAX_FILTER_PROP) && !do_abort) {
 		size_t n;
 		int i;
 		rule_t *r; 
         gboolean match = FALSE;
 
         r = (rule_t *)list->data;
-        if (dbg >= 10)
+        if (gui_debug >= 10)
             printf("trying to match against: %s\n", filter_rule_to_gchar(r));
 
         if (RULE_IS_ACTIVE(r)) {
@@ -1993,7 +2011,7 @@ static int filter_apply
                     break;
                 case RULE_TEXT_PREFIX:
                     if (strncmp(r->u.text.case_sensitive ? rec->name : l_name,
-                            r->u.text.match, strlen(r->u.text.match)) == 0)
+                            r->u.text.match, r->u.text.matchlen) == 0)
                         match = TRUE;
                     break;
                 case RULE_TEXT_WORDS:	/* Contains ALL the words */
@@ -2018,7 +2036,7 @@ static int filter_apply
                     }
                     break;
                 case RULE_TEXT_SUFFIX:
-                    n = strlen(r->u.text.match);
+                    n = r->u.text.matchlen;
                     if (namelen > n
                         && strcmp((r->u.text.case_sensitive
                                ? rec->name : l_name) + namelen
@@ -2127,7 +2145,7 @@ static int filter_apply
          */
         if (match) {
             if (r->target == filter_return) {
-                abort = TRUE;
+                do_abort = TRUE;
                 r->match_count ++;
                 r->target->match_count ++;
             } else
@@ -2146,7 +2164,7 @@ static int filter_apply
                     res->props[FILTER_PROP_DISPLAY].state =
                         FILTER_PROP_STATE_DONT;
                     res->props[FILTER_PROP_DISPLAY].user_data =
-                        (gpointer) (RULE_IS_SOFT(r) ? 1 : 0);
+                        GINT_TO_POINTER(RULE_IS_SOFT(r) ? 1 : 0);
 
                     MATCH_RULE(filter, r, res);
                 }
@@ -2237,7 +2255,7 @@ filter_result_t *filter_record(search_t *sch, record_t *rec)
 		filter_apply(filter_global_post, rec, result);
     
     /* FIXME: this does no longer give useful output
-    if (dbg >= 5) {
+    if (gui_debug >= 5) {
         printf("result %d for search \"%s\" matching \"%s\" (%s)\n",
             r, sch->query, rec->name, 
             filtered ? "filtered" : "unfiltered");
@@ -2252,7 +2270,7 @@ filter_result_t *filter_record(search_t *sch, record_t *rec)
         case FILTER_PROP_DISPLAY:
             if (!result->props[i].state) {
                 result->props[i].state = 
-                    filter_default_policy;
+                    FILTER_PROP_STATE_DO;
                 result->props_set ++;
             }
             break;
@@ -2280,12 +2298,7 @@ void filter_shutdown(void)
 {
     GList *f;
 
-    /*
-     * Discard all changes and close the dialog.
-     */
-    filter_close_dialog(FALSE);
-
-    if (dbg >= 5)
+    if (gui_debug >= 5)
         printf("shutting down filters\n");
 
     /*
@@ -2353,7 +2366,7 @@ void filter_init(void)
 
     filters_current = g_list_copy(filters);
 
-    create_popup_filter_rule();
+    popup_filter_rule = create_popup_filter_rule();
 }
 
 
@@ -2555,3 +2568,85 @@ inline filter_t *filter_get_global_post(void)
 {
     return filter_global_post;
 }
+
+/*
+ * Adds a drop SHA1 rule to specified filter.
+ */
+void filter_add_drop_sha1_rule(record_t *rec, filter_t *filter)
+{
+    rule_t *rule;
+
+    g_assert(rec != NULL);
+    g_assert(filter != NULL);
+
+    rule = filter_new_sha1_rule(rec->sha1, rec->name,
+        filter_get_drop_target(), RULE_FLAG_ACTIVE);
+
+    filter_append_rule(filter, rule);
+}
+
+/*
+ * Adds a drop filename rule to specified filter.
+ */
+void filter_add_drop_name_rule(record_t *rec, filter_t *filter)
+{
+    rule_t *rule;
+
+    g_assert(rec != NULL);
+    g_assert(filter != NULL);
+
+    rule = filter_new_text_rule(rec->name, RULE_TEXT_EXACT, TRUE,
+        filter_get_drop_target(), RULE_FLAG_ACTIVE);
+
+    filter_append_rule(filter, rule);
+}
+
+/*
+ * Adds a drop host rule to specified filter.
+ */
+void filter_add_drop_host_rule(record_t *rec, filter_t *filter)
+{
+    rule_t *rule;
+
+    g_assert(rec != NULL);
+    g_assert(filter != NULL);
+
+    rule = filter_new_ip_rule(rec->results_set->ip, 0xFFFFFFFF,
+        filter_get_drop_target(), RULE_FLAG_ACTIVE);
+
+    filter_append_rule(filter, rule);
+}
+
+/*
+ * Adds a download SHA1 rule to specified filter.
+ */
+void filter_add_download_sha1_rule(record_t *rec, filter_t *filter)
+{
+    g_assert(rec != NULL);
+    g_assert(filter != NULL);
+
+    if (rec->sha1) {
+        rule_t *rule;
+
+        rule = filter_new_sha1_rule(rec->sha1, rec->name,
+            filter_get_download_target(), RULE_FLAG_ACTIVE);
+
+        filter_append_rule(filter, rule);
+    }
+}
+
+/*
+ * Adds a download filename rule to specified filter.
+ */
+void filter_add_download_name_rule(record_t *rec, filter_t *filter)
+{
+    rule_t *rule;
+
+    g_assert(rec != NULL);
+    g_assert(filter != NULL);
+
+    rule = filter_new_text_rule(rec->name, RULE_TEXT_EXACT, TRUE,
+        filter_get_download_target(), RULE_FLAG_ACTIVE);
+
+    filter_append_rule(filter, rule);
+}
