@@ -27,6 +27,8 @@
 #line 1 "getdate.y"
 
 /*
+ * $Id: getdate.y,v 1.7 2002/12/05 10:44:44 cbiere Exp $
+ *
  * Date parsing.
  *
  *----------------------------------------------------------------------
@@ -63,7 +65,7 @@
 **  This code is in the public domain and has no copyright.
 */
 
-#include "config.h"
+#include "common.h"
 
 #ifdef HAVE_CONFIG_H
 # ifdef FORCE_ALLOCA_H
@@ -211,7 +213,7 @@ typedef enum _MERIDIAN {
 **  yacc had the %union construct.)  Maybe someday; right now we only use
 **  the %union very rarely.
 */
-static const char	*yyInput;
+static const unsigned char	*yyInput;
 static int	yyDayOrdinal;
 static int	yyDayNumber;
 static int	yyHaveDate;
@@ -235,7 +237,7 @@ static int	yyRelSeconds;
 static int	yyRelYear;
 
 
-#line 212 "getdate.y"
+#line 214 "getdate.y"
 typedef union {
     int			Number;
     enum _MERIDIAN	Meridian;
@@ -322,12 +324,12 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   230,   231,   234,   237,   240,   243,   246,   249,   252,   258,
-   264,   273,   279,   285,   291,   298,   310,   313,   316,   322,
-   331,   335,   339,   345,   349,   367,   373,   379,   385,   389,
-   394,   398,   405,   413,   416,   419,   422,   425,   428,   431,
-   434,   437,   440,   443,   446,   449,   452,   455,   458,   461,
-   464,   467,   472,   505,   509
+   232,   233,   236,   239,   242,   245,   248,   251,   254,   260,
+   266,   275,   281,   287,   293,   300,   312,   315,   318,   324,
+   333,   337,   341,   347,   351,   369,   375,   381,   387,   391,
+   396,   400,   407,   415,   418,   421,   424,   427,   430,   433,
+   436,   439,   442,   445,   448,   451,   454,   457,   460,   463,
+   466,   469,   474,   507,   511
 };
 #endif
 
@@ -910,37 +912,37 @@ yyreduce:
   switch (yyn) {
 
 case 3:
-#line 234 "getdate.y"
+#line 236 "getdate.y"
 {
 	    yyHaveTime++;
 	;
     break;}
 case 4:
-#line 237 "getdate.y"
+#line 239 "getdate.y"
 {
 	    yyHaveZone++;
 	;
     break;}
 case 5:
-#line 240 "getdate.y"
+#line 242 "getdate.y"
 {
 	    yyHaveDate++;
 	;
     break;}
 case 6:
-#line 243 "getdate.y"
+#line 245 "getdate.y"
 {
 	    yyHaveDay++;
 	;
     break;}
 case 7:
-#line 246 "getdate.y"
+#line 248 "getdate.y"
 {
 	    yyHaveRel++;
 	;
     break;}
 case 9:
-#line 252 "getdate.y"
+#line 254 "getdate.y"
 {
 	    yyHour = yyvsp[-1].Number;
 	    yyMinutes = 0;
@@ -949,7 +951,7 @@ case 9:
 	;
     break;}
 case 10:
-#line 258 "getdate.y"
+#line 260 "getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -958,7 +960,7 @@ case 10:
 	;
     break;}
 case 11:
-#line 264 "getdate.y"
+#line 266 "getdate.y"
 {
 	    yyHour = yyvsp[-3].Number;
 	    yyMinutes = yyvsp[-1].Number;
@@ -970,7 +972,7 @@ case 11:
 	;
     break;}
 case 12:
-#line 273 "getdate.y"
+#line 275 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -979,7 +981,7 @@ case 12:
 	;
     break;}
 case 13:
-#line 279 "getdate.y"
+#line 281 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -988,7 +990,7 @@ case 13:
 	;
     break;}
 case 14:
-#line 285 "getdate.y"
+#line 287 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -997,7 +999,7 @@ case 14:
 	;
     break;}
 case 15:
-#line 291 "getdate.y"
+#line 293 "getdate.y"
 {
 	    yyHour = yyvsp[-6].Number;
 	    yyMinutes = yyvsp[-4].Number;
@@ -1007,7 +1009,7 @@ case 15:
 	;
     break;}
 case 16:
-#line 298 "getdate.y"
+#line 300 "getdate.y"
 {
 	    yyHour = yyvsp[-5].Number;
 	    yyMinutes = yyvsp[-3].Number;
@@ -1020,25 +1022,25 @@ case 16:
 	;
     break;}
 case 17:
-#line 310 "getdate.y"
+#line 312 "getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number;
 	;
     break;}
 case 18:
-#line 313 "getdate.y"
+#line 315 "getdate.y"
 {
 	    yyTimezone = yyvsp[0].Number - 60;
 	;
     break;}
 case 19:
-#line 317 "getdate.y"
+#line 319 "getdate.y"
 {
 	    yyTimezone = yyvsp[-1].Number - 60;
 	;
     break;}
 case 20:
-#line 322 "getdate.y"
+#line 324 "getdate.y"
 {
 	    /* ISO 8601 format.  +02:00 -- RAM */
 
@@ -1048,35 +1050,35 @@ case 20:
 	;
     break;}
 case 21:
-#line 331 "getdate.y"
+#line 333 "getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 22:
-#line 335 "getdate.y"
+#line 337 "getdate.y"
 {
 	    yyDayOrdinal = 1;
 	    yyDayNumber = yyvsp[-1].Number;
 	;
     break;}
 case 23:
-#line 339 "getdate.y"
+#line 341 "getdate.y"
 {
 	    yyDayOrdinal = yyvsp[-1].Number;
 	    yyDayNumber = yyvsp[0].Number;
 	;
     break;}
 case 24:
-#line 345 "getdate.y"
+#line 347 "getdate.y"
 {
 	    yyMonth = yyvsp[-2].Number;
 	    yyDay = yyvsp[0].Number;
 	;
     break;}
 case 25:
-#line 349 "getdate.y"
+#line 351 "getdate.y"
 {
 	  /* Interpret as YYYY/MM/DD if $1 >= 1000, otherwise as DD/MM/YYYY.
 	     The goal in recognizing YYYY/MM/DD is solely to support legacy
@@ -1097,7 +1099,7 @@ case 25:
 	;
     break;}
 case 26:
-#line 367 "getdate.y"
+#line 369 "getdate.y"
 {
 	    /* ISO 8601 format.  yyyy-mm-dd.  */
 	    yyYear = yyvsp[-2].Number;
@@ -1106,7 +1108,7 @@ case 26:
 	;
     break;}
 case 27:
-#line 373 "getdate.y"
+#line 375 "getdate.y"
 {
 	    /* ISO 8601 format.  yyyy-mm-ddT -- RAM */
 	    yyYear = yyvsp[-2].Number;
@@ -1115,7 +1117,7 @@ case 27:
 	;
     break;}
 case 28:
-#line 379 "getdate.y"
+#line 381 "getdate.y"
 {
 	    /* e.g. 17-JUN-1992.  */
 	    yyDay = yyvsp[-2].Number;
@@ -1124,14 +1126,14 @@ case 28:
 	;
     break;}
 case 29:
-#line 385 "getdate.y"
+#line 387 "getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[0].Number;
 	;
     break;}
 case 30:
-#line 389 "getdate.y"
+#line 391 "getdate.y"
 {
 	    yyMonth = yyvsp[-3].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1139,14 +1141,14 @@ case 30:
 	;
     break;}
 case 31:
-#line 394 "getdate.y"
+#line 396 "getdate.y"
 {
 	    yyMonth = yyvsp[0].Number;
 	    yyDay = yyvsp[-1].Number;
 	;
     break;}
 case 32:
-#line 398 "getdate.y"
+#line 400 "getdate.y"
 {
 	    yyMonth = yyvsp[-1].Number;
 	    yyDay = yyvsp[-2].Number;
@@ -1154,7 +1156,7 @@ case 32:
 	;
     break;}
 case 33:
-#line 405 "getdate.y"
+#line 407 "getdate.y"
 {
 	    yyRelSeconds = -yyRelSeconds;
 	    yyRelMinutes = -yyRelMinutes;
@@ -1165,115 +1167,115 @@ case 33:
 	;
     break;}
 case 35:
-#line 416 "getdate.y"
+#line 418 "getdate.y"
 {
 	    yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 36:
-#line 419 "getdate.y"
+#line 421 "getdate.y"
 {
 	    yyRelYear += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 37:
-#line 422 "getdate.y"
+#line 424 "getdate.y"
 {
 	    yyRelYear++;
 	;
     break;}
 case 38:
-#line 425 "getdate.y"
+#line 427 "getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 39:
-#line 428 "getdate.y"
+#line 430 "getdate.y"
 {
 	    yyRelMonth += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 40:
-#line 431 "getdate.y"
+#line 433 "getdate.y"
 {
 	    yyRelMonth++;
 	;
     break;}
 case 41:
-#line 434 "getdate.y"
+#line 436 "getdate.y"
 {
 	    yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 42:
-#line 437 "getdate.y"
+#line 439 "getdate.y"
 {
 	    yyRelDay += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 43:
-#line 440 "getdate.y"
+#line 442 "getdate.y"
 {
 	    yyRelDay++;
 	;
     break;}
 case 44:
-#line 443 "getdate.y"
+#line 445 "getdate.y"
 {
 	    yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 45:
-#line 446 "getdate.y"
+#line 448 "getdate.y"
 {
 	    yyRelHour += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 46:
-#line 449 "getdate.y"
+#line 451 "getdate.y"
 {
 	    yyRelHour++;
 	;
     break;}
 case 47:
-#line 452 "getdate.y"
+#line 454 "getdate.y"
 {
 	    yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 48:
-#line 455 "getdate.y"
+#line 457 "getdate.y"
 {
 	    yyRelMinutes += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 49:
-#line 458 "getdate.y"
+#line 460 "getdate.y"
 {
 	    yyRelMinutes++;
 	;
     break;}
 case 50:
-#line 461 "getdate.y"
+#line 463 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 51:
-#line 464 "getdate.y"
+#line 466 "getdate.y"
 {
 	    yyRelSeconds += yyvsp[-1].Number * yyvsp[0].Number;
 	;
     break;}
 case 52:
-#line 467 "getdate.y"
+#line 469 "getdate.y"
 {
 	    yyRelSeconds++;
 	;
     break;}
 case 53:
-#line 473 "getdate.y"
+#line 475 "getdate.y"
 {
 	    if (yyHaveTime && yyHaveDate && !yyHaveRel)
 	      yyYear = yyvsp[0].Number;
@@ -1306,13 +1308,13 @@ case 53:
 	  ;
     break;}
 case 54:
-#line 506 "getdate.y"
+#line 508 "getdate.y"
 {
 	    yyval.Meridian = MER24;
 	  ;
     break;}
 case 55:
-#line 510 "getdate.y"
+#line 512 "getdate.y"
 {
 	    yyval.Meridian = yyvsp[0].Meridian;
 	  ;
@@ -1515,7 +1517,7 @@ yyerrhandle:
   yystate = yyn;
   goto yynewstate;
 }
-#line 515 "getdate.y"
+#line 517 "getdate.y"
 
 
 /* Month and day table. */
@@ -1753,8 +1755,8 @@ static int ToYear(int Year)
 
 static int LookupWord(char *buff)
 {
-    register char *p;
-    register char *q;
+    register unsigned char *p;
+    register unsigned char *q;
     register const TABLE *tp;
     int i;
     int abbrev;
@@ -1828,7 +1830,7 @@ static int LookupWord(char *buff)
 	}
 
     /* Military timezones. */
-    if (buff[1] == '\0' && ISALPHA(*buff)) {
+    if (buff[1] == '\0' && ISALPHA((unsigned char) *buff)) {
 	for (tp = MilitaryTable; tp->name; tp++)
 	    if (strcmp(buff, tp->name) == 0) {
 		yylval.Number = tp->value;
@@ -1855,9 +1857,9 @@ static int LookupWord(char *buff)
 
 static int yylex(void)
 {
-    register char c;
-    register char *p;
-    char buff[20];
+    register unsigned char c;
+    register unsigned char *p;
+    unsigned char buff[20];
     int Count;
     int sign;
 
