@@ -1,4 +1,6 @@
 /*
+ * $Id: matching.c,v 1.24 2003/03/18 00:28:29 cbiere Exp $
+ *
  * Copyright (c) 2001, Raphael Manfredi
  *
  * Search bins are Copyright (c) 2001, Kenn Brooks Hamm & Raphael Manfredi
@@ -23,10 +25,66 @@
  *----------------------------------------------------------------------
  */
 
+#include "common.h"
+#include "qrp.h"			/* For qhvec_add() */
+
 #include <ctype.h>
-#include "gnutella.h"
-#include "matching.h"
-#include "search_stats.h"
+
+RCSID("$Id: matching.c,v 1.24 2003/03/18 00:28:29 cbiere Exp $");
+
+/* FIXME: remove this dependency */
+extern guint32 dbg;
+
+#define WOVEC_DFLT	10				/* Default size of word-vectors */
+
+/*
+ * Masks for mask_hash().
+ */
+
+#define MASK_LETTER(x)		(1 << (x))		/* bits 0 to 25 */
+#define MASK_DIGIT			0x80000000
+
+static zone_t *pat_zone = NULL;		/* Compiled patterns */
+static zone_t *wovec_zone = NULL;	/* Word-vectors of WOVEC_DFLT entries */
+
+
+
+/*
+ * matching_init
+ *
+ * Initialize matching data structures.
+ */
+void matching_init(void)
+{
+	/*
+	 * We don't expect much word vectors to be created.  They are normally
+	 * created and destroyed in the same routine, without any threading
+	 * taking place.
+	 *
+	 * We only allocate word vectors of WOVEC_DFLT entries in the zone.
+	 * If we need to expand that, it will be done through regular malloc().
+	 */
+
+	wovec_zone = zget(WOVEC_DFLT * sizeof(word_vec_t), 2);
+
+	/*
+	 * Patterns are not only used for query matching but also for filters,
+	 * therefore we can expect quite a few to be created at the same time.
+	 */
+
+	pat_zone = zget(sizeof(cpattern_t), 64);
+}
+
+/*
+ * matching_close
+ *
+ * Terminate matching data structures.
+ */
+void matching_close(void)
+{
+	zdestroy(wovec_zone);
+	zdestroy(pat_zone);
+}
 
 /*
  * Search query word splitting.
@@ -40,6 +98,23 @@
  * but also their wanted frequency.
  */
 
+/*
+ * word_vec_zrealloc
+ *
+ * Reallocate a word-vector from the zone into heap memory, to hold `ncount'.
+ */
+static word_vec_t *word_vec_zrealloc(word_vec_t *wv, gint ncount)
+{
+	word_vec_t *nwv = g_malloc(ncount * sizeof(word_vec_t));
+
+	g_assert(ncount > WOVEC_DFLT);
+
+	memcpy(nwv, wv, WOVEC_DFLT * sizeof(word_vec_t));
+	zfree(wovec_zone, wv);
+
+	return nwv;
+}
+
 /*
  * query_make_word_vec
  *
@@ -55,10 +130,10 @@ guint query_make_word_vec(guchar *query, word_vec_t **wovec)
 {
 	guint n = 0;
 	GHashTable *seen_word = g_hash_table_new(g_str_hash, g_str_equal);
-	guint nv = 10;
-	word_vec_t *wv = g_malloc(nv * sizeof(word_vec_t));
+	guint nv = WOVEC_DFLT;
+	word_vec_t *wv = zalloc(wovec_zone);
 	guchar c;
-	gchar *start = NULL;
+	guchar *start = NULL;
 
 	g_assert(wovec != NULL);
 
@@ -76,18 +151,27 @@ guint query_make_word_vec(guchar *query, word_vec_t **wovec)
 			 * If word already seen in query, it's in the seen_word table.
 			 * The associated value is the index in the vector plus 1.
 			 */
-			np1 = (guint) g_hash_table_lookup(seen_word, (gconstpointer) start);
-			if (np1) wv[np1-1].amount++;
-			else {
+			np1 = GPOINTER_TO_UINT(
+				g_hash_table_lookup(seen_word, (gconstpointer) start));
+			if (np1--) {
+				wv[np1].amount++;
+				wv[np1].len = query - start;
+			} else {
 				word_vec_t *entry;
 				if (n == nv) {				/* Filled all the slots */
 					nv *= 2;
-					wv = g_realloc(wv, nv * sizeof(word_vec_t));
+					if (n > WOVEC_DFLT)
+						wv = g_realloc(wv, nv * sizeof(word_vec_t));
+					else
+						wv = word_vec_zrealloc(wv, nv);
 				}
 				entry = &wv[n++];
-				entry->word = g_strdup(start);
+				entry->len = query - start;
+				entry->word = walloc(entry->len + 1);	/* For trailing NUL */
+				memcpy(entry->word, start, entry->len + 1);	/* Include NUL */
 				entry->amount = 1;
-				g_hash_table_insert(seen_word, entry->word, (gpointer) n);
+				g_hash_table_insert(seen_word, entry->word,
+					GUINT_TO_POINTER(n));
 			}
 			*query = c;
 			start = NULL;
@@ -99,7 +183,7 @@ guint query_make_word_vec(guchar *query, word_vec_t **wovec)
 	if (n)
 		*wovec = wv;
 	else
-		g_free(wv);
+		zfree(wovec_zone, wv);
 
 	return n;
 }
@@ -114,9 +198,12 @@ void query_word_vec_free(word_vec_t *wovec, guint n)
 	guint i;
 
 	for (i = 0; i < n; i++)
-		g_free(wovec[i].word);
+		wfree(wovec[i].word, wovec[i].len + 1);
 
-	g_free(wovec);
+	if (n > WOVEC_DFLT)
+		g_free(wovec);
+	else
+		zfree(wovec_zone, wovec);
 }
 
 /*
@@ -138,7 +225,7 @@ void query_word_vec_free(word_vec_t *wovec, guint n)
  */
 cpattern_t *pattern_compile(guchar *pattern)
 {
-	cpattern_t *p = (cpattern_t *) g_malloc0(sizeof(cpattern_t));
+	cpattern_t *p = (cpattern_t *) zalloc(pat_zone);
 	guint32 plen = strlen(pattern);
 	guint32 *pd = p->delta;
 	gint i;
@@ -168,7 +255,7 @@ cpattern_t *pattern_compile(guchar *pattern)
 void pattern_free(cpattern_t *cpat)
 {
 	g_free(cpat->pattern);
-	g_free(cpat);
+	zfree(pat_zone, cpat);
 }
 
 /*
@@ -273,49 +360,9 @@ gchar *pattern_qsearch(
 
 #define ST_MIN_BIN_SIZE		4
 
-/*
- * Masks for mask_hash().
- */
-
-#define MASK_LETTER(x)		(1 << (x))		/* bits 0 to 25 */
-#define MASK_DIGIT			0x80000000
-
-/*
- * mask_hash
- *
- * Compute character mask "hash", using one bit per letter of the alphabet,
- * plus one for any digit.
- */
-static guint32 mask_hash(guchar *str) {
-	guchar *s = str;
-	guchar c;
-	guint32 mask = 0;
-
-	while ((c = *s++)) {
-		if (isspace(c))
-			continue;
-		else if (isdigit(c))
-			mask |= MASK_DIGIT;
-		else {
-			gint idx = tolower(c) - 'a';
-			if (idx >= 0 && idx < 26)
-				mask |= MASK_LETTER(idx);
-		}
-	}
-
-	return mask;
-}
-
-/* get key of two-char pair */
-__inline__ static gint st_key(search_table_t *table, guchar k[2])
-{
-	return table->index_map[k[0]] * table->nchars +
-		table->index_map[k[1]];
-}
 
 static void destroy_entry(struct st_entry *entry)
 {
-	/* FIXME: make sure this frees everything */
 	g_free(entry->string);
 }
 
@@ -460,6 +507,39 @@ void st_destroy(search_table_t *table)
 	}
 }
 
+/*
+ * mask_hash
+ *
+ * Compute character mask "hash", using one bit per letter of the alphabet,
+ * plus one for any digit.
+ */
+static guint32 mask_hash(guchar *str) {
+	guchar *s = str;
+	guchar c;
+	guint32 mask = 0;
+
+	while ((c = *s++)) {
+		if (isspace(c))
+			continue;
+		else if (isdigit(c))
+			mask |= MASK_DIGIT;
+		else {
+			gint idx = tolower(c) - 'a';
+			if (idx >= 0 && idx < 26)
+				mask |= MASK_LETTER(idx);
+		}
+	}
+
+	return mask;
+}
+
+/* get key of two-char pair */
+G_INLINE_FUNC gint st_key(search_table_t *table, guchar k[2])
+{
+	return table->index_map[k[0]] * table->nchars +
+		table->index_map[k[1]];
+}
+
 /* insert an item into the search_table
  * one-char strings are silently ignored */
 void st_insert_item(search_table_t *table, guchar *string, void *data)
@@ -488,10 +568,11 @@ void st_insert_item(search_table_t *table, guchar *string, void *data)
 		gint key = st_key(table, string + i);
 
 		/* don't insert item into same bin twice */
-		if (g_hash_table_lookup(seen_keys, (gconstpointer) key))
+		if (g_hash_table_lookup(seen_keys, (gconstpointer)GINT_TO_POINTER(key)))
 			continue;
 
-		g_hash_table_insert(seen_keys, (gpointer) key, (gpointer) 1);
+		g_hash_table_insert(seen_keys, GINT_TO_POINTER(key),
+			GINT_TO_POINTER(1));
 
 		g_assert(key < table->nbins);
 		if (table->bins[key] == NULL)
@@ -505,8 +586,56 @@ void st_insert_item(search_table_t *table, guchar *string, void *data)
 	g_hash_table_destroy(seen_keys);
 }
 
+/* minimize space consumption */
+void st_compact(search_table_t *table)
+{
+	gint i;
+
+	if (!table->all_entries.nvals)
+		return;			/* Nothing in table */
+
+	bin_compact(&table->all_entries);
+	for (i = 0; i < table->nbins; i++)
+		if (table->bins[i])
+			bin_compact(table->bins[i]);
+}
+
 /*
+ * pattern_compile_fast
+ *
+ * Same as pattern_compile(), but the pattern string is NOT duplicated,
+ * and its length is known upon entry.
+ *
+ * NB: there is no pattern_free_fast(), just call zfree() on the result.
+ */
+static cpattern_t *pattern_compile_fast(guchar *pattern, guint32 plen)
+{
+	cpattern_t *p = (cpattern_t *) zalloc(pat_zone);
+	guint32 *pd = p->delta;
+	gint i;
+	guchar *c;
+
+	p->pattern = pattern;
+	p->len = plen;
+
+	plen++;			/* Avoid increasing within the memset() inlined macro */
+
+	for (i = 0; i < ALPHA_SIZE; i++)
+		*pd++ = plen;
+
+	plen--;			/* Restore original pattern length */
+
+	for (pd = p->delta, c = pattern, i = 0; i < plen; c++, i++)
+		pd[(guint) *c] = plen - i;
+
+	return p;
+}
+
+/*
+ * entry_match
+ *
  * Apply pattern matching on text, matching at the *beginning* of words.
+ * Patterns are lazily compiled as needed, using pattern_compile_fast().
  */
 static gboolean entry_match(
 	gchar *text, gint tlen,
@@ -518,6 +647,10 @@ static gboolean entry_match(
 		gint amount = wovec[i].amount;
 		gint j;
 		guint32 offset = 0;
+
+		if (pw[i] == NULL)
+			pw[i] = pattern_compile_fast(wovec[i].word, wovec[i].len);
+
 		for (j = 0; j < amount; j++) {
 			char *pos =
 				pattern_qsearch(pw[i], text, tlen, offset, qs_begin);
@@ -537,8 +670,9 @@ static gboolean entry_match(
 gint st_search(
 	search_table_t *table,
 	guchar *search,
-	gboolean (*callback)(struct shared_file *),
-	gint max_res)
+	gboolean (*callback)(shared_file_t *),
+	gint max_res,
+	query_hashvec_t *qhv)
 {
 	gint i, key, nres = 0;
 	guint len;
@@ -551,6 +685,7 @@ gint st_search(
 	gint vcnt;
 	gint scanned = 0;		/* measure search mask efficiency */
 	guint32 search_mask;
+	gint minlen;
 
 	len = match_map_string(table->fold_map, search);
 
@@ -586,30 +721,43 @@ gint st_search(
 	 *		--RAM, 06/10/2001
 	 */
 
-	if (best_bin == NULL)
-		return 0;
+	if (best_bin == NULL) {
+		/*
+		 * If we have a `qhv', we need to compute the word vector anway,
+		 * for query routing...
+		 */
 
-	g_assert(best_bin_size > 0);	/* Allocated bin, it must hold something */
+		if (qhv == NULL)
+			return 0;
+	}
 
 	/*
 	 * Prepare matching patterns
 	 */
 
 	wocnt = query_make_word_vec(search, &wovec);
-	if (wocnt == 0)
-		return 0;
 
 	/*
-	 * If search statistics are being gathered, count each word
+	 * Compute the query hashing information for query routing, if needed.
 	 */
-	if (search_stats_enabled)
-	    for (i = 0; i < wocnt; i++)
-		search_stats_tally(&wovec[i]);
 
-	pattern = (cpattern_t **) g_malloc(wocnt * sizeof(cpattern_t *));
+	if (qhv != NULL) {
+		for (i = 0; i < wocnt; i++) {
+			if (wovec[i].len >= QRP_MIN_WORD_LENGTH)
+				qhvec_add(qhv, wovec[i].word, QUERY_H_WORD);
+		}
+	}
 
-	for (i = 0; i < wocnt; i++)
-		pattern[i] = pattern_compile(wovec[i].word);
+	if (wocnt == 0 || best_bin == NULL) {
+		if (wocnt > 0)
+			query_word_vec_free(wovec, wocnt);
+		return 0;
+	}
+
+	g_assert(best_bin_size > 0);	/* Allocated bin, it must hold something */
+
+
+	pattern = (cpattern_t **) g_malloc0(wocnt * sizeof(cpattern_t *));
 
 	/*
 	 * Prepare matching optimization, an idea from Mike Green.
@@ -628,6 +776,18 @@ gint st_search(
 
 	search_mask = mask_hash(search);
 
+	/*
+	 * Prepare second matching optimization: since all words in the query
+	 * must match the exact amount of time, we can compute the minimum length
+	 * the searched file must have.  We add one character after each word
+	 * but the last, to account for space between words.
+	 *		--RAM, 11/07/2002
+	 */
+
+	for (minlen = 0, i = 0; i < wocnt; i++)
+		minlen += wovec[i].len + 1;
+	minlen--;
+
 	/*
 	 * Search through the smallest bin
 	 */
@@ -642,6 +802,9 @@ gint st_search(
 		if ((e->mask & search_mask) != search_mask)
 			continue;		/* Can't match */
 
+		if (sf->file_name_len < minlen)
+			continue;		/* Can't match */
+
 		scanned++;
 
 		if (entry_match(e->string, sf->file_name_len, pattern, wovec, wocnt)) {
@@ -661,27 +824,14 @@ gint st_search(
 			scanned, best_bin_size, nres);
 
 	for (i = 0; i < wocnt; i++)
-		pattern_free(pattern[i]);
+		if (pattern[i])					/* Lazily compiled by entry_match() */
+			zfree(pat_zone, pattern[i]);
+
 	g_free(pattern);
 	query_word_vec_free(wovec, wocnt);
 
 	return nres;
 }
 
-/* minimize space consumption */
-void st_compact(search_table_t *table)
-{
-	gint i;
-
-	if (!table->all_entries.nvals)
-		return;			/* Nothing in table */
-
-	bin_compact(&table->all_entries);
-	for (i = 0; i < table->nbins; i++)
-		if (table->bins[i])
-			bin_compact(table->bins[i]);
-}
-
-
 /* vi: set ts=4: */
 
