@@ -1,4 +1,6 @@
 /*
+ * $Id: pcache.c,v 1.34 2003/05/09 05:36:24 rmanfredi Exp $
+ *
  * Copyright (c) 2001-2002, Raphael Manfredi
  *
  * Pong caching (LimeWire's ping/pong reducing scheme).
@@ -33,23 +35,22 @@
 #include <netinet/in.h>
 #include <arpa/inet.h>
 
-#include "interface.h"
-#include "gui.h"
-#include "misc.h"
 #include "sockets.h"
 #include "hosts.h"
+#include "hcache.h"
 #include "pcache.h"
 #include "nodes.h"
 #include "share.h" /* For files_scanned and kbytes_scanned. */
 #include "routing.h"
 #include "gmsg.h"
 #include "alive.h"
+#include "inet.h"
+#include "gnet_stats.h"
+#include "hostiles.h"
 
-#define FW_STATUS_GRACE		300		/* Startup period where we send pongs */
-
-static GList *last_returned_pong = NULL;	/* Last returned from list */
-static GList *pcache_recent_pongs = NULL;	/* Recent pongs we got */
+#include "settings.h"
 
+RCSID("$Id: pcache.c,v 1.34 2003/05/09 05:36:24 rmanfredi Exp $");
 
 /***
  *** Messages
@@ -64,7 +65,7 @@ static void send_ping(struct gnutella_node *n, guint8 ttl)
 {
 	struct gnutella_msg_init m;
 
-	message_set_muid(&m.header, TRUE);
+	message_set_muid(&m.header, GTA_MSG_INIT);
 
 	m.header.function = GTA_MSG_INIT;
 	m.header.ttl = ttl;
@@ -115,7 +116,7 @@ void send_alive_ping(struct gnutella_node *n, guchar *muid)
 	g_assert(NODE_IS_WRITABLE(n));
 	g_assert(muid);
 
-	message_set_muid(&m.header, TRUE);
+	message_set_muid(&m.header, GTA_MSG_INIT);
 	memcpy(muid, &m.header, 16);
 
 	m.header.function = GTA_MSG_INIT;
@@ -165,6 +166,8 @@ static void send_pong(struct gnutella_node *n, gboolean control,
 {
 	struct gnutella_msg_init_response *r;
 
+	g_assert(ttl >= 1);
+
 	if (!NODE_IS_WRITABLE(n))
 		return;
 
@@ -188,11 +191,26 @@ static void send_pong(struct gnutella_node *n, gboolean control,
  */
 static void send_personal_info(struct gnutella_node *n, gboolean control)
 {
+	guint32 kbytes;
+
 	g_assert(n->header.function == GTA_MSG_INIT);	/* Replying to a ping */
 
 	if (!force_local_ip && !local_ip)
 		return;		/* If we don't know yet our local IP, we can't reply */
 
+	/*
+	 * Mark pong if we are an ultra node: the amount of kbytes scanned must
+	 * be an exact power of two, and at minimum 8.
+	 */
+
+	if (current_peermode == NODE_P_ULTRA) {
+		if (kbytes_scanned <= 8)
+			kbytes = 8;
+		else
+			kbytes = next_pow2(kbytes_scanned);
+	} else
+		kbytes = kbytes_scanned | 0x1;		/* Ensure not a power of two */
+
 	/*
 	 * Pongs are sent with a TTL just large enough to reach the pinging host,
 	 * up to a maximum of max_ttl.	Note that we rely on the hop count being
@@ -201,7 +219,7 @@ static void send_personal_info(struct gnutella_node *n, gboolean control)
 	 */
 
 	send_pong(n, control, 0, MIN(n->header.hops + 1, max_ttl), n->header.muid,
-		listen_ip(), listen_port, files_scanned, kbytes_scanned);
+		listen_ip(), listen_port, files_scanned, kbytes);
 }
 
 /*
@@ -245,7 +263,8 @@ static void send_neighbouring_info(struct gnutella_node *n)
 		 * it in our reserve, so do it from here.
 		 */
 
-		host_add(cn->gnet_ip, cn->gnet_port, FALSE);
+		if (!NODE_IS_LEAF(n))
+			host_add(cn->gnet_ip, cn->gnet_port, FALSE);
 
 		/*
 		 * Node can be removed should its send queue saturate.
@@ -289,15 +308,19 @@ struct cache_line {			/* A cache line for a given hop value */
 	GSList *cursor;			/* Cursor within list: last item traversed */
 };
 
-static gint pcache_recent_pong_count = 0;	/* # of pongs in recent list */
-static GHashTable *ht_recent_pongs;
+struct recent {
+	GHashTable *ht_recent_pongs;	/* Recent pongs we know about */
+	GList *recent_pongs;			/* Recent pongs we got */
+	GList *last_returned_pong;		/* Last returned from list */
+	gint recent_pong_count;			/* # of pongs in recent list */
+};
 
 #define PONG_CACHE_SIZE		(MAX_CACHE_HOPS+1)
 
 static struct cache_line pong_cache[PONG_CACHE_SIZE];
+static struct recent recent_pongs[HCACHE_MAX];
 
 #define CACHE_LIFESPAN		5		/* seconds */
-#define PING_THROTTLE		3		/* seconds */
 #define MAX_PONGS			10		/* Max pongs returned per ping */
 #define OLD_PING_PERIOD		45		/* Pinging period for "old" clients */
 #define OLD_CACHE_RATIO		20		/* % of pongs from "old" clients we cache */
@@ -312,14 +335,14 @@ static struct cache_line pong_cache[PONG_CACHE_SIZE];
 
 static guint cached_pong_hash(gconstpointer key)
 {
-	struct cached_pong *cp = (struct cached_pong *) key;
+	const struct cached_pong *cp = (const struct cached_pong *) key;
 
 	return (guint) (cp->ip ^ ((cp->port << 16) | cp->port));
 }
 static gint cached_pong_eq(gconstpointer v1, gconstpointer v2)
 {
-	struct cached_pong *h1 = (struct cached_pong *) v1;
-	struct cached_pong *h2 = (struct cached_pong *) v2;
+	const struct cached_pong *h1 = (const struct cached_pong *) v1;
+	const struct cached_pong *h2 = (const struct cached_pong *) v2;
 
 	return h1->ip == h2->ip && h1->port == h2->port;
 }
@@ -332,11 +355,16 @@ void pcache_init(void)
 	gint h;
 
 	memset(pong_cache, 0, sizeof(pong_cache));
+	memset(recent_pongs, 0, sizeof(recent_pongs));
 
 	for (h = 0; h < PONG_CACHE_SIZE; h++)
 		pong_cache[h].hops = h;
 
-	ht_recent_pongs = g_hash_table_new(cached_pong_hash, cached_pong_eq);
+	recent_pongs[HCACHE_ANY].ht_recent_pongs =
+		g_hash_table_new(cached_pong_hash, cached_pong_eq);
+
+	recent_pongs[HCACHE_ULTRA].ht_recent_pongs =
+		g_hash_table_new(cached_pong_hash, cached_pong_eq);
 }
 
 /*
@@ -351,7 +379,7 @@ static void free_cached_pong(struct cached_pong *cp)
 	if (--(cp->refcount) != 0)
 		return;
 
-	g_free(cp);
+	wfree(cp, sizeof(*cp));
 }
 
 
@@ -364,14 +392,19 @@ static void free_cached_pong(struct cached_pong *cp)
  * Fills `ip' and `port' with the pong value and return TRUE if we
  * got a pong.  Otherwise return FALSE.
  */
-gboolean pcache_get_recent(guint32 *ip, guint16 *port)
+gboolean pcache_get_recent(hcache_type_t type, guint32 *ip, guint16 *port)
 {
 	static guint32 last_ip = 0;
 	static guint16 last_port = 0;
 	GList *l;
 	struct cached_pong *cp;
+	struct recent *rec;
 
-	if (!pcache_recent_pongs)		/* List empty */
+	g_assert(type >= 0 && type < HCACHE_MAX);
+
+	rec = &recent_pongs[type];
+
+	if (!rec->recent_pongs)		/* List empty */
 		return FALSE;
 
 	/*
@@ -383,8 +416,8 @@ gboolean pcache_get_recent(guint32 *ip, guint16 *port)
 	 * go back to the tail of the list.
 	 */
 
-	if (last_returned_pong == NULL) {
-		l = g_list_first(pcache_recent_pongs);
+	if (rec->last_returned_pong == NULL) {
+		l = g_list_first(rec->recent_pongs);
 		cp = (struct cached_pong *) l->data;
 
 		if (cp->ip != last_ip || cp->port != last_port)
@@ -394,7 +427,7 @@ gboolean pcache_get_recent(guint32 *ip, guint16 *port)
 			return FALSE;
 	} else {
 		/* Regular case */
-		for (l = last_returned_pong->prev; l; l = l->prev) {
+		for (l = rec->last_returned_pong->prev; l; l = l->prev) {
 			cp = (struct cached_pong *) l->data;
 			if (cp->ip != last_ip || cp->port != last_port)
 				goto found;
@@ -405,7 +438,7 @@ gboolean pcache_get_recent(guint32 *ip, guint16 *port)
 	 * Still none found, go back to the end of the list.
 	 */
 
-	for (l = g_list_last(pcache_recent_pongs); l; l = l->prev) {
+	for (l = g_list_last(rec->recent_pongs); l; l = l->prev) {
 		cp = (struct cached_pong *) l->data;
 		if (cp->ip != last_ip || cp->port != last_port)
 			goto found;
@@ -414,13 +447,13 @@ gboolean pcache_get_recent(guint32 *ip, guint16 *port)
 	return FALSE;
 
 found:
-	last_returned_pong = l;
+	rec->last_returned_pong = l;
 	*ip = last_ip = cp->ip;
 	*port = last_port = cp->port;
 
 	if (dbg > 8)
-		printf("returning recent PONG %s\n",
-			ip_port_to_gchar(cp->ip, cp->port));
+		printf("returning recent %s PONG %s\n",
+			hcache_type_to_gchar(type), ip_port_to_gchar(cp->ip, cp->port));
 
 	return TRUE;
 }
@@ -431,51 +464,81 @@ found:
  * Add recent pong to the list, handled as a FIFO cache, if not already
  * present.
  */
-static void add_recent_pong(struct cached_pong *cp)
+static void add_recent_pong(hcache_type_t type, struct cached_pong *cp)
 {
-	if (g_hash_table_lookup(ht_recent_pongs, (gconstpointer) cp))
+	struct recent *rec;
+
+	g_assert(type >= 0 && type < HCACHE_MAX);
+
+	rec = &recent_pongs[type];
+
+	if (g_hash_table_lookup(rec->ht_recent_pongs, (gconstpointer) cp))
 		return;
 
-	if (pcache_recent_pong_count == RECENT_PING_SIZE) {		/* Full */
-		GList *link = g_list_last(pcache_recent_pongs);
-		struct cached_pong *cp = (struct cached_pong *) link->data;
+	if (rec->recent_pong_count == RECENT_PING_SIZE) {		/* Full */
+		GList *lnk = g_list_last(rec->recent_pongs);
+		struct cached_pong *cp = (struct cached_pong *) lnk->data;
 
-		pcache_recent_pongs = g_list_remove_link(pcache_recent_pongs, link);
-		g_hash_table_remove(ht_recent_pongs, cp);
+		rec->recent_pongs = g_list_remove_link(rec->recent_pongs, lnk);
+		g_hash_table_remove(rec->ht_recent_pongs, cp);
 
-		if (link == last_returned_pong)
-			last_returned_pong = last_returned_pong->prev;
+		if (lnk == rec->last_returned_pong)
+			rec->last_returned_pong = rec->last_returned_pong->prev;
 
 		free_cached_pong(cp);
-		g_list_free_1(link);
+		g_list_free_1(lnk);
 	} else
-		pcache_recent_pong_count++;
+		rec->recent_pong_count++;
 	
-	pcache_recent_pongs = g_list_prepend(pcache_recent_pongs, cp);
-	g_hash_table_insert(ht_recent_pongs, cp, (gpointer) 1);
+	rec->recent_pongs = g_list_prepend(rec->recent_pongs, cp);
+	g_hash_table_insert(rec->ht_recent_pongs, cp, (gpointer) 1);
 	cp->refcount++;		/* We don't refcount insertion in the hash table */
 }
 
+/*
+ * pong_type
+ *
+ * Determine the pong type (any, or of the ultra kind).
+ */
+static hcache_type_t pong_type(struct gnutella_init_response *pong)
+{
+	guint32 kbytes;
+
+	READ_GUINT32_LE(pong->kbytes_count, kbytes);
+
+	/*
+	 * Ultra pongs are marked by having their kbytes count be an
+	 * exact power of two.
+	 */
+
+	return is_pow2(kbytes) ? HCACHE_ULTRA : HCACHE_ANY;
+}
+
 /*
  * pcache_clear_recent
  *
  * Clear the whole recent pong list.
  */
-void pcache_clear_recent(void)
+void pcache_clear_recent(hcache_type_t type)
 {
 	GList *l;
+	struct recent *rec;
 
-	for (l = pcache_recent_pongs; l; l = l->next) {
+	g_assert(type >= 0 && type < HCACHE_MAX);
+
+	rec = &recent_pongs[type];
+
+	for (l = rec->recent_pongs; l; l = l->next) {
 		struct cached_pong *cp = (struct cached_pong *) l->data;
 
-		g_hash_table_remove(ht_recent_pongs, cp);
+		g_hash_table_remove(rec->ht_recent_pongs, cp);
 		free_cached_pong(cp);
 	}
 
-	g_list_free(pcache_recent_pongs);
-	pcache_recent_pongs = NULL;
-	last_returned_pong = NULL;
-	pcache_recent_pong_count = 0;
+	g_list_free(rec->recent_pongs);
+	rec->recent_pongs = NULL;
+	rec->last_returned_pong = NULL;
+	rec->recent_pong_count = 0;
 }
 
 /*
@@ -491,7 +554,7 @@ void pcache_outgoing_connection(struct gnutella_node *n)
 {
 	g_assert(NODE_IS_CONNECTED(n));
 
-	if (connected_nodes() < up_connections || host_cache_is_empty())
+	if (connected_nodes() < up_connections || hcache_is_low(HCACHE_ANY))
 		send_ping(n, my_ttl);		/* Regular ping, get fresh pongs */
 	else
 		send_ping(n, 1);			/* Handshaking ping */
@@ -523,7 +586,7 @@ static void pcache_expire(void)
 
 	if (dbg > 4)
 		printf("Pong CACHE expired (%d entr%s, %d in reserve)\n",
-			entries, entries == 1 ? "y" : "ies", host_cache_size());
+			entries, entries == 1 ? "y" : "ies", hcache_size(HCACHE_ANY));
 }
 
 /*
@@ -533,8 +596,17 @@ static void pcache_expire(void)
  */
 void pcache_close(void)
 {
+	static hcache_type_t types[] = { HCACHE_ANY, HCACHE_ULTRA };
+	gint i;
+
 	pcache_expire();
-	g_hash_table_destroy(ht_recent_pongs);
+
+	for (i = 0; i < sizeof(types) / sizeof(types[0]); i++) {
+		hcache_type_t type = types[i];
+
+		pcache_clear_recent(type);
+		g_hash_table_destroy(recent_pongs[type].ht_recent_pongs);
+	}
 }
 
 /*
@@ -551,7 +623,7 @@ static void ping_all_neighbours(time_t now)
 	for (l = sl_nodes; l; l = l->next) {
 		struct gnutella_node *n = (struct gnutella_node *) l->data;
 
-		if (!NODE_IS_WRITABLE(n))
+		if (!NODE_IS_WRITABLE(n) || NODE_IS_LEAF(n))
 			continue;
 
 		if (n->attrs & NODE_A_PONG_CACHING)
@@ -754,7 +826,8 @@ static void send_cached_pongs(struct gnutella_node *n,
  * they need one at this hop count.
  */
 static void pong_all_neighbours_but_one(
-	struct gnutella_node *n, struct cached_pong *cp, guint8 hops, guint8 ttl)
+	struct gnutella_node *n, struct cached_pong *cp, hcache_type_t ptype,
+	guint8 hops, guint8 ttl)
 {
 	GSList *l;
 
@@ -779,6 +852,13 @@ static void pong_all_neighbours_but_one(
 		if (!cn->pong_needed[hops])
 			continue;
 
+		/*
+		 * If node is a leaf node, we can only send it Ultra pongs.
+		 */
+
+		if (NODE_IS_LEAF(cn) && ptype != HCACHE_ULTRA)
+			continue;
+
 		cn->pong_missing--;
 		cn->pong_needed[hops]--;
 
@@ -801,13 +881,71 @@ static void pong_all_neighbours_but_one(
 	}
 }
 
+/*
+ * pong_random_leaf
+ *
+ * We received an ultra pong.
+ * Send it to one randomly selected leaf, which is not already missing pongs.
+ */
+static void pong_random_leaf(struct cached_pong *cp, guint8 hops, guint8 ttl)
+{
+	GSList *l;
+	gint leaves;
+	struct gnutella_node *leaf = NULL;
+
+	g_assert(current_peermode == NODE_P_ULTRA);
+
+	for (l = sl_nodes, leaves = 0; l; l = l->next) {
+		struct gnutella_node *cn = (struct gnutella_node *) l->data;
+		gint threshold;
+
+		if (cn->pong_missing)	/* A job for pong_all_neighbours_but_one() */
+			continue;
+
+		if (!NODE_IS_LEAF(cn))
+			continue;
+
+		/*
+		 * Randomly select one leaf.
+		 *
+		 * As we go along, the probability that we retain the current leaf
+		 * decreases.  It is 1 for the first leaf, 1/2 for the second leaf,
+		 * 1/3 for the third leaf, etc...
+		 */
+
+		leaves++;
+		threshold = (gint) (1000.0 / leaves);
+
+		if (random_value(999) < threshold)
+			leaf = cn;
+	}
+
+	/*
+	 * Send the pong to the selected leaf, if any.
+	 *
+	 * NB: If the leaf never sent a ping before, leaf->ping_guid will
+	 * be a zero GUID.  That's OK.
+	 */
+
+	if (leaf != NULL) {
+		send_pong(leaf, FALSE, hops + 1, ttl, leaf->ping_guid,
+			cp->ip, cp->port, cp->files_count, cp->kbytes_count);
+
+		if (dbg > 7)
+			printf("pong_random_leaf: sent pong %s (hops=%d, TTL=%d) to %s\n",
+				ip_port_to_gchar(cp->ip, cp->port), hops, ttl, node_ip(leaf));
+	}
+}
+
 /*
  * record_fresh_pong
  *
  * Add pong from node `n' to our cache of recent pongs.
  * Returns the cached pong object.
  */
-static struct cached_pong *record_fresh_pong(struct gnutella_node *n,
+static struct cached_pong *record_fresh_pong(
+	hcache_type_t type,
+	struct gnutella_node *n,
 	guint8 hops, guint32 ip, guint16 port,
 	guint32 files_count, guint32 kbytes_count)
 {
@@ -815,7 +953,9 @@ static struct cached_pong *record_fresh_pong(struct gnutella_node *n,
 	struct cached_pong *cp;
 	guint8 hop;
 
-	cp = (struct cached_pong *) g_malloc(sizeof(struct cached_pong));
+	g_assert(type >= 0 && type < HCACHE_MAX);
+
+	cp = (struct cached_pong *) walloc(sizeof(struct cached_pong));
 
 	cp->refcount = 1;
 	cp->node_id = n->id;
@@ -828,49 +968,18 @@ static struct cached_pong *record_fresh_pong(struct gnutella_node *n,
 	hop = CACHE_HOP_IDX(hops);		/* Trim high values to MAX_CACHE_HOPS */
 	cl = &pong_cache[hop];
 	cl->pongs = g_slist_append(cl->pongs, cp);
-	add_recent_pong(cp);
+	add_recent_pong(type, cp);
 
 	return cp;
 }
 
-/*
- * can_answer_ping
- *
- * Check whether we can answer a ping with a pong.
- *
- * Normally, when we're firewalled, we don't answer. However, if we have
- * a non-private IP and are within the first FW_STATUS_GRACE seconds of
- * startup time, act as if we were not: we can only know we're not
- * firewalled when we get an incoming connection.
- */
-static gboolean can_answer_ping(void)
-{
-	guint32 ip;
-
-	if (!is_firewalled)
-		return TRUE;
-
-	ip = listen_ip();
-
-	if (!ip)
-		return FALSE;		/* We don't know our local IP, we can't reply */
-
-	if (is_private_ip(ip))
-		return FALSE;
-
-	if (time(NULL) - start_time < FW_STATUS_GRACE)
-		return TRUE;
-
-	return FALSE;
-}
-
 /*
  * pcache_ping_received
  *
  * Called when a ping is received from a node.
  *
  * + If current time is less than what `ping_accept' says, drop the ping.
- *   Otherwise, accept the ping and increment `ping_accept' by PING_THROTTLE.
+ *   Otherwise, accept the ping and increment `ping_accept' by n->ping_throttle.
  * + If cache expired, call pcache_expire() and broadcast a new ping to all
  *   the "new" clients (i.e. those flagged NODE_A_PONG_CACHING).  For "old"
  *   clients, do so only if "next_ping" time was reached.
@@ -898,9 +1007,10 @@ void pcache_ping_received(struct gnutella_node *n)
 		n->n_ping_special++;
 		if (n->header.ttl == 1)
 			send_personal_info(n, TRUE);	/* Control message, prioritary */
-		else if (n->header.ttl == 2)
-			send_neighbouring_info(n);
-		else
+		else if (n->header.ttl == 2) {
+			if (current_peermode != NODE_P_LEAF)
+				send_neighbouring_info(n);
+		} else
 			node_sent_ttl0(n);
 		return;
 	}
@@ -920,7 +1030,7 @@ void pcache_ping_received(struct gnutella_node *n)
 		if (dbg)
 			g_warning("node %s (%s) [%d.%d] claimed ping reduction, "
 				"got ping with hops=%d", node_ip(n),
-				n->vendor ? n->vendor : "????",
+				node_vendor(n),
 				n->proto_major, n->proto_minor, n->header.hops);
 		n->attrs |= NODE_A_PONG_ALIEN;		/* Warn only once */
 	}
@@ -932,11 +1042,11 @@ void pcache_ping_received(struct gnutella_node *n)
 	if (now < n->ping_accept) {
 		n->n_ping_throttle++;		/* Drop the ping */
 		n->rx_dropped++;
-		dropped_messages++;
+        gnet_stats_count_dropped(n, MSG_DROP_PING_THROTTLE);
 		return;
 	} else {
 		n->n_ping_accepted++;
-		n->ping_accept = now + PING_THROTTLE;	/* Drop more ones until then */
+		n->ping_accept = now + n->ping_throttle;	/* Drop all until then */
 	}
 
 	/*
@@ -952,20 +1062,38 @@ void pcache_ping_received(struct gnutella_node *n)
 	 * If TTL = 0, only us can reply, and we'll do that below in any case..
 	 * We call setup_pong_demultiplexing() anyway to reset the pong_needed[]
 	 * array.
+	 *
+	 * A leaf node will not demultiplex pongs, so don't bother.
 	 */
 
-	setup_pong_demultiplexing(n, n->header.muid, n->header.ttl);
+	if (current_peermode != NODE_P_LEAF)
+		setup_pong_demultiplexing(n, n->header.muid, n->header.ttl);
 
 	/*
 	 * If we can accept an incoming connection, send a reply.
+	 *
+	 * If we are firewalled, we nonetheless send a ping
+	 * when inet_can_answer_ping() tells us we can, irrespective
+	 * of whether we can accept a new node connection: the aim is
+	 * to trigger an incoming connection that will prove us we're
+	 * not firewalled.
 	 */
 
-	if (node_count() < max_connections && can_answer_ping()) {
+	if (
+		(is_firewalled || node_missing() > 0) && inet_can_answer_ping()
+	) {
 		send_personal_info(n, FALSE);
 		if (!NODE_IS_CONNECTED(n))	/* Can be removed if send queue is full */
 			return;
 	}
 
+	if (current_peermode == NODE_P_LEAF)
+		return;
+
+	/*
+	 * We continue here only for non-leaf nodes.
+	 */
+
 	/*
 	 * Return cached pongs if we have some and they are needed.
 	 * We first try to send pongs on a per-hop basis, based on pong_needed[].
@@ -1019,6 +1147,7 @@ void pcache_pong_received(struct gnutella_node *n)
 	guint32 files_count;
 	guint32 kbytes_count;
 	struct cached_pong *cp;
+	hcache_type_t ptype;
 
 	n->n_pong_received++;
 
@@ -1043,7 +1172,7 @@ void pcache_pong_received(struct gnutella_node *n)
 			} else if (!(n->flags & NODE_F_ALIEN_IP)) {
 				if (dbg) g_warning(
 					"node %s (%s) sent us a pong for itself with alien IP %s",
-					node_ip(n), n->vendor ? n->vendor : "", ip_to_gchar(ip));
+					node_ip(n), node_vendor(n), ip_to_gchar(ip));
 				n->flags |= NODE_F_ALIEN_IP;	/* Probably firewalled */
 			}
 		}
@@ -1060,7 +1189,7 @@ void pcache_pong_received(struct gnutella_node *n)
 		if (n->gnet_pong_ip && ip != n->gnet_pong_ip) {
 			if (dbg && n->n_ping_sent > 2) g_warning(
 				"node %s (%s) sent us a pong for new IP %s (used %s before)",
-				node_ip(n), n->vendor ? n->vendor : "",
+				node_ip(n), node_vendor(n),
 				ip_port_to_gchar(ip, port), ip_to_gchar(n->gnet_pong_ip));
 		}
 
@@ -1078,8 +1207,19 @@ void pcache_pong_received(struct gnutella_node *n)
 	 * If it's not a connectible pong, discard it.
 	 */
 
-	if (!check_valid_host(ip, port))
+	if (!host_is_valid(ip, port)) {
+		gnet_stats_count_dropped(n, MSG_DROP_PONG_UNUSABLE);
 		return;
+	}
+
+	/*
+	 * If pong points to an hostile IP address, discard it.
+	 */
+
+	if (hostiles_check(ip)) {
+		gnet_stats_count_dropped(n, MSG_DROP_HOSTILE_IP);
+		return;
+	}
 
 	/*
 	 * If pong points to us, maybe we explicitly connected to ourselves
@@ -1116,11 +1256,18 @@ void pcache_pong_received(struct gnutella_node *n)
 	 * Insert pong within our cache.
 	 */
 
-	cp = record_fresh_pong(n, n->header.hops, ip, port,
+	ptype = pong_type((struct gnutella_init_response *) n->data);
+
+	cp = record_fresh_pong(HCACHE_ANY, n, n->header.hops, ip, port,
 		files_count, kbytes_count);
 
+	if (ptype == HCACHE_ULTRA)
+		(void) record_fresh_pong(HCACHE_ULTRA, n, n->header.hops, ip, port,
+			files_count, kbytes_count);
+
 	if (dbg > 6)
-		printf("CACHED pong %s (hops=%d, TTL=%d) from %s %s\n",
+		printf("CACHED %s pong %s (hops=%d, TTL=%d) from %s %s\n",
+			ptype == HCACHE_ULTRA ? "ultra" : "normal",
 			ip_port_to_gchar(ip, port), n->header.hops, n->header.ttl,
 			(n->attrs & NODE_A_PONG_CACHING) ? "NEW" : "OLD", node_ip(n));
 
@@ -1129,8 +1276,21 @@ void pcache_pong_received(struct gnutella_node *n)
 	 * received it from, provided they need more pongs of this hop count.
 	 */
 
-	pong_all_neighbours_but_one(n,
-		cp, CACHE_HOP_IDX(n->header.hops), MAX(1, n->header.ttl));
+	if (current_peermode != NODE_P_LEAF)
+		pong_all_neighbours_but_one(n,
+			cp, ptype, CACHE_HOP_IDX(n->header.hops), MAX(1, n->header.ttl));
+
+	/*
+	 * If we're in ultra mode, send 33% of all the ultra pongs we get
+	 * to one random leaf.
+	 */
+
+	if (
+		current_peermode == NODE_P_ULTRA &&
+		ptype == HCACHE_ULTRA && random_value(99) < 33
+	)
+		pong_random_leaf(
+			cp, CACHE_HOP_IDX(n->header.hops), MAX(1, n->header.ttl));
 }
 
 /*
@@ -1146,14 +1306,11 @@ void pcache_pong_received(struct gnutella_node *n)
  */
 void pcache_pong_fake(struct gnutella_node *n, guint32 ip, guint16 port)
 {
-	if (!check_valid_host(ip, port))
+	if (!host_is_valid(ip, port))
 		return;
 
 	host_add(ip, port, FALSE);
-	(void) record_fresh_pong(n, 1, ip, port, 0, 0);
-
-	n->gnet_ip = ip;
-	n->gnet_port = port;
+	(void) record_fresh_pong(HCACHE_ANY, n, 1, ip, port, 0, 0);
 }
 
 /* vi: set ts=4: */
