@@ -1,4 +1,6 @@
 /*
+ * $Id: routing.c,v 1.55 2003/05/08 21:45:09 rmanfredi Exp $
+ *
  * Copyright (c) 2001-2002, Raphael Manfredi
  *
  * Gnutella Network Messages routing.
@@ -24,22 +26,18 @@
  */
 
 #include "gnutella.h"
+
 #include "search.h" /* For search_passive. */
 #include "routing.h"
 #include "hosts.h"
-#include "misc.h"
 #include "gmsg.h"
-#include "atoms.h"
+#include "nodes.h"
+#include "gnet_stats.h"
+#include "hostiles.h"
 
 #include <stdarg.h>
-#include <assert.h>
 
-/*
- * Flags for GUID[15] tagging.
- */
-
-#define GUID_PONG_CACHING	0x01
-#define GUID_PERSISTENT		0x02
+RCSID("$Id: routing.c,v 1.55 2003/05/08 21:45:09 rmanfredi Exp $");
 
 struct gnutella_node *fake_node;		/* Our fake node */
 
@@ -67,7 +65,7 @@ struct route_data {
 
 struct route_data fake_route;		/* Our fake route_data */
 
-gchar *debug_msg[256];
+static const gchar *debug_msg[256];
 
 #define MAX_STORED_MESSAGES 65536	/* Max messages we can remember */
 
@@ -89,7 +87,8 @@ struct message message_array[MAX_STORED_MESSAGES];
 GHashTable *messages_hashed; 	/* we hash the last MAX_STORED_MESSAGES */
 guint next_message_index;		/* next slot to use in message_array */
 
-static gboolean find_message(guchar *muid, guint8 function, struct message **m);
+static gboolean find_message(
+	const guchar *muid, guint8 function, struct message **m);
 
 /*
  * "banned" GUIDs for push routing.
@@ -97,7 +96,7 @@ static gboolean find_message(guchar *muid, guint8 function, struct message **m);
  * The following GUIDs are so common that it does not make sense to
  * route pushes to them (i.e. they are are NOT unique on the network!).
  */
-static guchar *banned_push[] = {
+static const guchar *banned_push[] = {
 	"20d262ff0e6fd6119734004005a207b1",		/* Morpheus, 29/06/2002 */
 };
 GHashTable *ht_banned_push = NULL;
@@ -108,17 +107,16 @@ GHashTable *ht_banned_push = NULL;
  * Log function
  */
 
-void routing_log(gchar * fmt, ...)
+static void routing_log(gchar * fmt, ...)
 {
-	static gchar t[4096];
 	va_list va;
 
-	va_start(va, fmt);
-	g_vsnprintf(t, sizeof(t), fmt, va);
-	va_end(va);
+	if (dbg <= 8)
+		return;
 
-	if (dbg > 8)
-		printf("%s", t);
+	va_start(va, fmt);
+	vprintf(fmt, va);
+	va_end(va);
 }
 
 static void free_route_list(struct message *m);
@@ -131,7 +129,7 @@ static struct route_data * get_routing_data(struct gnutella_node *n)
 
 /* if a node doesn't currently have routing data attached, this
    creates and attaches some */
-void init_routing_data(struct gnutella_node *node)
+static void init_routing_data(struct gnutella_node *node)
 {
 	struct route_data *route;
 
@@ -142,7 +140,7 @@ void init_routing_data(struct gnutella_node *node)
 	 * Allocate and link some routing data to it
 	 */
 
-	route = (struct route_data *) g_malloc(sizeof(struct route_data));
+	route = (struct route_data *) walloc(sizeof(struct route_data));
 
 	route->node = node;
 	route->saved_messages = 0;
@@ -176,59 +174,40 @@ static gboolean node_sent_message(struct gnutella_node *n, struct message *m)
 }
 
 /* compares two message structures */
-gint message_compare_func(gconstpointer a, gconstpointer b)
+static gint message_compare_func(gconstpointer a, gconstpointer b)
 {
 	return
-		0 == memcmp(((struct message *) a)->muid,
-			((struct message *) b)->muid, 16)
-		&& ((struct message *) a)->function ==
-			((struct message *) b)->function;
+		0 == memcmp(((const struct message *) a)->muid,
+			((const struct message *) b)->muid, 16)
+		&& ((const struct message *) a)->function ==
+			((const struct message *) b)->function;
 }
 
 /* hashes message structures for storage in a hash table */
-guint message_hash_func(gconstpointer key)
+static guint message_hash_func(gconstpointer key)
 {
 	int count;
 	guint hash = 0;
 	
 	for (count = 0; count <= 12; count += 4) {
 		guint hashadd =
-			( (struct message *) key)->muid[count]            |
-			(((struct message *) key)->muid[count + 1] << 8)  |
-			(((struct message *) key)->muid[count + 2] << 16) |
-			(((struct message *) key)->muid[count + 3] << 24);
+			( (const struct message *) key)->muid[count]            |
+			(((const struct message *) key)->muid[count + 1] << 8)  |
+			(((const struct message *) key)->muid[count + 2] << 16) |
+			(((const struct message *) key)->muid[count + 3] << 24);
 
 		hash ^= hashadd;
 	}
 
-	hash ^= (guint) ((struct message *) key)->function;
+	hash ^= (guint) ((const struct message *) key)->function;
 
 	return hash;
 }
 
-/*
- * patch_muid_for_modern_node
- *
- * Make sure the MUID we use in initial handshaking pings are marked
- * specially to indicate we're modern nodes.
- */
-static void patch_muid_for_modern_node(guchar *muid)
-{
-	/*
-	 * We're a "modern" client, meaning we're not Gnutella 0.56.
-	 * Therefore we must set our ninth byte, muid[8] to 0xff, and
-	 * put the protocol version number in muid[15].	For 0.4, this
-	 * means 0.
-	 *				--RAM, 15/09/2001
-	 */
-
-	muid[8] = 0xff;
-	muid[15] = GUID_PONG_CACHING | GUID_PERSISTENT;
-}
-
 /* Init function */
 void routing_init(void)
 {
+    guint8 guid_buf[16];
 	guint32 i;
 	gboolean need_guid = TRUE;
 	extern guint guid_hash(gconstpointer key);		/* from atoms.c */
@@ -242,8 +221,6 @@ void routing_init(void)
 	fake_route.saved_messages = 0;
 	fake_route.node = fake_node;
 
-	srand(time((time_t *) NULL) ^ getpid());
-
 	/*
 	 * Initialize the banned GUID hash.
 	 */
@@ -252,7 +229,7 @@ void routing_init(void)
 
 	for (i = 0; i < BANNED_PUSH_COUNT; i++) {
 		guchar g[16];
-		guchar *hex = banned_push[i];
+		const guchar *hex = banned_push[i];
 
 		g_assert(strlen(hex) == 2*sizeof(g));
 
@@ -268,38 +245,33 @@ void routing_init(void)
 	 *		--RAM, 08/03/2002
 	 */
 
+    gnet_prop_get_storage(PROP_GUID, guid_buf, sizeof(guid_buf));
+
 	for (i = 0; i < 15; i++) {
-		if (guid[i]) {
+		if (guid_buf[i]) {
 			need_guid = FALSE;
 			break;
 		}
 	}
 
+	if (!guid_is_gtkg(guid_buf, NULL, NULL, NULL))
+		need_guid = TRUE;
+
 retry:
-	if (need_guid) {
-		for (i = 0; i < 15; i++)
-			guid[i] = random_value(0xff);
-	}
-
-	/*
-	 * We *always* patch the GUID, even a persistent one.  This means the
-	 * last byte can change when new features are supported.  Since it's not
-	 * something that happens on a regular basis, we'll achieve a fairly
-	 * long-lived GUID anyway.
-	 *		--RAM, 08/03/2002
-	 */
-
-	patch_muid_for_modern_node(guid);
+	if (need_guid)
+		guid_ping_muid(guid_buf);		/* We want a "modern" GUID */
 
 	/*
 	 * If by extraordinary, we have generated a banned GUID, retry.
 	 */
 
-	if (g_hash_table_lookup(ht_banned_push, guid)) {
+	if (g_hash_table_lookup(ht_banned_push, guid_buf)) {
 		need_guid = TRUE;
 		goto retry;
 	}
 
+    gnet_prop_set_storage(PROP_GUID, guid_buf, sizeof(guid_buf));
+
 	/*
 	 * Initialize message type array for routing logs.
 	 */
@@ -313,6 +285,8 @@ retry:
 	debug_msg[GTA_MSG_SEARCH_RESULTS] = "Q-Hit";
 	debug_msg[GTA_MSG_PUSH_REQUEST]   = "Push ";
 	debug_msg[GTA_MSG_VENDOR]         = "Vndor";
+	debug_msg[GTA_MSG_STANDARD]       = "V-Std";
+	debug_msg[GTA_MSG_QRP]            = "QRP  ";
 
 	/*
 	 * Should be around for life of program, so should *never*
@@ -325,48 +299,27 @@ retry:
 	memset(message_array, 0, sizeof(message_array));
 }
 
-/*
- * generate_new_muid
- *
- * Generate a new random message ID.  We will never send the same MUID twice
- * due to the use of a counter.
- *
- * If `modern' is true, then patch it to indicate we're a "modern node".
- * If `modern' is false, it probably does not matter.
- */
-void generate_new_muid(guchar *muid, gboolean modern)
-{
-	static guint32 muid_cnt = 0;		/* Ensure messages we send are unique */
-	gint i;
-
-	/*
-	 * We place our unique counter at offset 4, and it is 4 bytes long
-	 * hence the tweaking with `k', and the upper bound for `i' of 12.
-	 */
-
-	for (i = 0; i < 12; i += 2) {
-		gint k = (i < 4) ? i : (i + 4);
-		(*((guint16 *) (muid + k))) = (guint16) random_value(0xffff);
-	}
-
-	*((guint32 *) (muid + 4)) = muid_cnt++;
-
-	if (modern)
-		patch_muid_for_modern_node(muid);
-	else {
-		if (muid[8] == 0xff)	/* To not confuse it with special GUID... */
-			muid[8] = 0x00;		/* the 9th byte cannot be 0xff */
-	}
-}
-
 /*
  * message_set_muid
  *
  * Generate a new muid and put it in a message header.
  */
-void message_set_muid(struct gnutella_header *header, gboolean modern)
+void message_set_muid(struct gnutella_header *header, guint8 function)
 {
-	generate_new_muid(header->muid, modern);
+	switch (function) {
+	case GTA_MSG_PUSH_REQUEST:
+	case GTA_MSG_BYE:
+	case GTA_MSG_QRP:
+	case GTA_MSG_VENDOR:
+	case GTA_MSG_STANDARD:
+		guid_random_muid(header->muid);
+		break;
+	case GTA_MSG_INIT:
+		guid_ping_muid(header->muid);
+		break;
+	default:
+		g_error("unexpected message type %d", function);
+	}
 }
 
 /*
@@ -393,7 +346,7 @@ static void remove_one_message_reference(struct route_data *rd)
 		 */
 
 		if (rd->node == NULL && rd->saved_messages == 0)
-			g_free(rd);
+			wfree(rd, sizeof(*rd));
 	} else
 		g_assert(rd == &fake_route);
 }
@@ -440,12 +393,12 @@ void routing_node_remove(struct gnutella_node *node)
 	 */
 
 	if (route->saved_messages == 0)
-		g_free(route);
+		wfree(route, sizeof(*route));
 }
 
 /* Adds a new message in the routing tables */
 
-void message_add(guchar * muid, guint8 function,
+void message_add(const guchar * muid, guint8 function,
 				 struct gnutella_node *node)
 {
 	static time_t last_rotation = 0;
@@ -566,7 +519,8 @@ static void purge_dangling_references(struct message *m)
  * If none of the nodes that sent us the message are still present, then
  * m->routes will be NULL.
  */
-static gboolean find_message(guchar *muid, guint8 function, struct message **m)
+static gboolean find_message(
+	const guchar *muid, guint8 function, struct message **m)
 {
 	/* Returns TRUE if the message is found */
 	/* Set *node to node if there is a connected node associated
@@ -624,7 +578,7 @@ static gboolean forward_message(struct gnutella_node **node,
 
 		sender->n_hard_ttl++;
 		sender->rx_dropped++;
-		dropped_messages++;
+        gnet_stats_count_dropped(sender, MSG_DROP_HARD_TTL_LIMIT);
 
 		if (sender->header.hops <= max_high_ttl_radius &&
 			sender->n_hard_ttl > max_high_ttl_msg
@@ -640,24 +594,24 @@ static gboolean forward_message(struct gnutella_node **node,
 	/*
 	 * If node propagates messages with TTL=0, it's a danger to
 	 * the network, kick him out.
-	 *				-- RAM, 15/09/2001
+	 *		-- RAM, 15/09/2001
+	 *
+	 * If we're a leaf node, it's OK though, as we don't have to route
+	 * the message.  Some broken Ultrapeers out there send such messages!
+	 * We'll even handle the message.
+	 *		-- RAM, 12/01/2003
 	 */
 
 	if (sender->header.ttl == 0) {
 		routing_log("[ ] [NEW] TTL was 0\n");
 		if (node_sent_ttl0(sender))
 			*node = NULL;
-		return FALSE;	/* Don't handle, shouldn't have seen it */
+		/* Don't handle unless we're a leaf: shouldn't have seen it */
+		return current_peermode == NODE_P_LEAF;
 	}
 
 	routing_log("[H] [NEW] ");
 
-	if (sender->header.ttl > max_ttl) {	/* TTL too large */
-		sender->header.ttl = max_ttl;	/* Trim down */
-		routing_log("(TTL trimmed down to %d) ", max_ttl);
-	}
-
-
 	message_add(sender->header.muid, sender->header.function, sender);
 
 	sender->header.hops++;	/* Going to handle it, must be accurate */
@@ -665,7 +619,7 @@ static gboolean forward_message(struct gnutella_node **node,
 	if (!--sender->header.ttl) {
 		/* TTL expired, message stops here */
 		routing_log("(TTL expired)\n");
-		dropped_messages++;
+        gnet_stats_count_expired(sender);
 		/* don't increase rx_dropped, we'll handle this message */
 	} else {			/* Forward it to all others nodes */
 		if (target) {
@@ -673,6 +627,16 @@ static gboolean forward_message(struct gnutella_node **node,
 			dest->type = ROUTE_ONE;
 			dest->node = target;
 		} else {
+			/*
+			 * This message is broadcasted, ensure its TTL is "reasonable".
+			 * Trim down if excessively large.
+			 */
+
+			if (sender->header.ttl > max_ttl) {	/* TTL too large */
+				sender->header.ttl = max_ttl;	/* Trim down */
+				routing_log("(TTL trimmed down to %d) ", max_ttl);
+			}
+
 			routing_log("-> sendto_all_but_one()\n");
 			dest->type = ROUTE_ALL_BUT_ONE;
 			dest->node = sender;
@@ -777,7 +741,7 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 
 		if (sender->header.hops == 255) {
 			routing_log("(max hop count reached)\n");
-			dropped_messages++;
+            gnet_stats_count_dropped(sender, MSG_DROP_MAX_HOP_COUNT);
 			sender->rx_dropped++;
 			sender->n_bad++;
 			if (dbg)
@@ -788,14 +752,20 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 		/*
 		 * If node propagates messages with TTL=0, it's a danger to
 		 * the network, kick him out.
-		 *				-- RAM, 15/09/2001
+		 *		-- RAM, 15/09/2001
+		 *
+		 * If we're a leaf node, it's OK though, as we don't have to route
+		 * the message.  Some broken Ultrapeers out there send such messages!
+		 * We'll even handle the message.
+	 	 *		-- RAM, 12/01/2003
 		 */
 
 		if (sender->header.ttl == 0) {
 			routing_log("(TTL was 0)\n");
 			if (node_sent_ttl0(sender))
 				*node = NULL;
-			return FALSE;	/* Don't handle, shouldn't have seen it */
+			/* Don't handle unless we're a leaf: shouldn't have seen it */
+			return current_peermode == NODE_P_LEAF;
 		}
 
 		if (!find_message
@@ -805,7 +775,7 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 			routing_log("[ ] no request matching the reply !\n");
 
 			sender->rx_dropped++;
-			dropped_messages++;
+            gnet_stats_count_dropped(sender, MSG_DROP_UNREQUESTED_REPLY);
 			sender->n_bad++;	/* Node shouldn't have forwarded this message */
 
 			if (dbg > 2)
@@ -830,9 +800,8 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 		if (m->routes == NULL) {
 			routing_log("[%c] route to target lost\n", handle_it ? 'H' : ' ');
 
-			routing_errors++;
-			sender->rx_dropped++;
-			dropped_messages++;
+            sender->rx_dropped++;
+            gnet_stats_count_dropped(sender, MSG_DROP_ROUTE_LOST);
 
 			if (handle_it) {
 				sender->header.hops++;	/* Must be accurate if we handle it */
@@ -875,16 +844,27 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 
 		sender->header.hops++;
 
-		if (!--sender->header.ttl) {
-			/* TTL expired, message stops here */
-			routing_log("(TTL expired)\n");
-			dropped_messages++;
-			sender->rx_dropped++;
-			return handle_it;
-		}
+		/*
+		 * If the TTL expired, drop the message, unless the target is a
+		 * leaf node, in which case we'll forward it the reply.
+		 */
 
 		found = ((struct route_data *) m->routes->data)->node;
 
+		if (!--sender->header.ttl) {
+			if (NODE_IS_LEAF(found)) {
+				/* TTL expired, but target is a leaf node */
+				routing_log("(expired TTL bumped)\n");
+				sender->header.ttl = 1;
+			} else {
+				/* TTL expired, message stops here */
+				routing_log("(TTL expired)\n");
+				gnet_stats_count_expired(sender);
+				sender->rx_dropped++;
+				return handle_it;
+			}
+		}
+
 		routing_log("-> sendto_one(%s)\n", node_ip(found));
 
 		dest->type = ROUTE_ONE;
@@ -901,15 +881,13 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 			 * This is a duplicated message, which we're going to drop.
 			 */
 
-			dropped_messages++;
+            gnet_stats_count_dropped(sender, MSG_DROP_DUPLICATE);
 			sender->rx_dropped++;
 
 			if (m->routes && node_sent_message(sender, m)) {
 				/* The same node has sent us a message twice ! */
 				routing_log("[ ] dup message (from the same node!)\n");
 
-				routing_errors++;
-
 				/*
 				 * That is a really good reason to kick the offender
 				 * But do so only if killing this node would not bring
@@ -924,8 +902,8 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 
 				if (++(sender->n_dups) > min_dup_msg &&
 					connected_nodes() > MAX(2, up_connections) &&
-					sender->n_dups >
-						(guint16) (min_dup_ratio / 100.0 * sender->received)
+					sender->n_dups > (guint16) 
+                        (((float)min_dup_ratio) / 10000.0 * sender->received)
 				) {
 					node_bye(sender, 401, "Sent %d dups (%.1f%% of RX)",
 						sender->n_dups, sender->received ?
@@ -964,6 +942,8 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 		 */
 
 		if (sender->header.function == GTA_MSG_PUSH_REQUEST) {
+			guint32 ip;
+
 			g_assert(sender->size > 16);	/* Must be a valid push */
 
 			/*
@@ -976,7 +956,24 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 						"from %s, banned GUID %s",
 						node_ip(sender), guid_hex_str(sender->data));
 
-				dropped_messages++;
+                gnet_stats_count_dropped(sender, MSG_DROP_BANNED);
+				sender->rx_dropped++;
+
+				return FALSE;
+			}
+
+			/*
+			 * If IP address is among the hostile set, drop.
+			 */
+
+			READ_GUINT32_BE(sender->data + 20, ip);
+
+			if (hostiles_check(ip)) {
+				if (dbg > 3)
+					gmsg_log_dropped(&sender->header,
+						"from %s, hostile IP address", node_ip(sender));
+
+                gnet_stats_count_dropped(sender, MSG_DROP_HOSTILE_IP);
 				sender->rx_dropped++;
 
 				return FALSE;
@@ -996,15 +993,16 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 					return TRUE;
 				}
 
-				if (m && m->routes == NULL)
+				if (m && m->routes == NULL) {
 					routing_log("[ ] route to target GUID %s gone\n",
 						guid_hex_str(sender->data));
-				else
+                    gnet_stats_count_dropped(sender, MSG_DROP_ROUTE_LOST);
+                } else {
 					routing_log("[ ] no route to target GUID %s\n",
 						guid_hex_str(sender->data));
+                    gnet_stats_count_dropped(sender, MSG_DROP_NO_ROUTE);
+                }
 
-				routing_errors++;
-				dropped_messages++;
 				sender->rx_dropped++;
 
 				return FALSE;
@@ -1020,7 +1018,7 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 			 */
 
 			if (!NODE_IS_READABLE(sender)) {		/* Being shutdown */
-				dropped_messages++;
+                gnet_stats_count_dropped(sender, MSG_DROP_SHUTDOWN);
 				sender->rx_dropped++;
 				return FALSE;
 			}
@@ -1046,7 +1044,7 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
 					gmsg_log_dropped(&sender->header,
 						"from %s, in TX flow-control", node_ip(sender));
 
-				dropped_messages++;
+                gnet_stats_count_dropped(sender, MSG_DROP_FLOW_CONTROL);
 				sender->rx_dropped++;
 
 				return FALSE;
@@ -1070,7 +1068,7 @@ gboolean route_message(struct gnutella_node **node, struct route_dest *dest)
  * Returns NULL if we have no such route, or the node to which we should
  * send the packet otherwise.
  */
-struct gnutella_node *route_towards_guid(guchar *guid)
+struct gnutella_node *route_towards_guid(const guchar *guid)
 {
 	struct message *m;
 
