@@ -1,4 +1,6 @@
 /*
+ * $Id: rx_inflate.h,v 1.4 2003/01/01 00:14:36 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  * Network RX -- multiplexed decompressing stage.
@@ -23,14 +25,14 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __rx_inflate_h__
-#define __rx_inflate_h__
+#ifndef _rx_inflate_h_
+#define _rx_inflate_h_
 
 #include "rx.h"
 
 struct rxdrv_ops rx_inflate_ops;
 
-#endif	/* __rx_inflate_h__ */
+#endif	/* _rx_inflate_h_ */
 
 /* vi: set ts=4: */
 
