@@ -1,4 +1,6 @@
 /*
+ * $Id: rx_link.h,v 1.4 2003/01/01 00:14:36 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  * Network driver -- link layer.
@@ -23,14 +25,14 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __rx_link_h__
-#define __rx_link_h__
+#ifndef _rx_link_h_
+#define _rx_link_h_
 
 #include "rx.h"
 
 struct rxdrv_ops rx_link_ops;
 
-#endif	/* __rx_link_h__ */
+#endif	/* _rx_link_h_ */
 
 /* vi: set ts=4: */
 
