@@ -1,4 +1,6 @@
 /*
+ * $Id: rxbuf.h,v 1.4 2003/01/01 00:14:36 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  * Network RX buffer allocator.
@@ -23,8 +25,8 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __rxbuf_h__
-#define __rxbuf_h__
+#ifndef _rxbuf_h_
+#define _rxbuf_h_
 
 #include "pmsg.h"
 
@@ -38,7 +40,7 @@ void rxbuf_free(gpointer p, gpointer unused);
 void rxbuf_init(void);
 void rxbuf_close(void);
 
-#endif	/* __rxbuf_h__ */
+#endif	/* _rxbuf_h_ */
 
 /* vi: set ts=4: */
 
