@@ -1,4 +1,6 @@
 /*
+ * $Id: search_xml.c,v 1.35 2003/05/24 18:34:15 rmanfredi Exp $
+ *
  * Copyright (c) 2002, Richard Eckart
  *
  * Persistance for searches and filters in XML format.
@@ -23,23 +25,27 @@
  *----------------------------------------------------------------------
  */
 
+#include "gui.h"
+
 #include <libxml/tree.h>
 #include <libxml/parser.h>
 
-#include "gnutella.h"
-#include "gmsg.h"
-#include "misc.h"
 #include "filter.h"
-#include "search.h"
 #include "search_xml.h"
 
+#include "settings_gui.h"
+#include "search_gui.h"
+#include "version.h"
+
+RCSID("$Id: search_xml.c,v 1.35 2003/05/24 18:34:15 rmanfredi Exp $");
+
 #define GLOBAL_PRE 0
 #define GLOBAL_POST 1
 
 #define TO_BOOL(v) (v == 0 ? 0 : 1)
 
 typedef struct node_parser {
-    gchar * name;
+    const gchar * name;
     void (*parser_func)(xmlNodePtr, gpointer);
 } node_parser_t;
 
@@ -89,53 +95,57 @@ static const gchar NODE_RULE_SHA1[]   = "SHA1Rule";
 static const gchar NODE_RULE_FLAG[]   = "FlagRule";
 static const gchar NODE_RULE_STATE[]  = "StateRule";
 
-static const gchar PROP_BUILTIN_SHOW_UID[]       = "ShowUID";
-static const gchar PROP_BUILTIN_DROP_UID[]       = "DropUID";
-static const gchar PROP_BUILTIN_DOWNLOAD_UID[]   = "DownloadUID";
-static const gchar PROP_BUILTIN_NODOWNLOAD_UID[] = "NoDownloadUID";
-static const gchar PROP_BUILTIN_RETURN_UID[]     = "ReturnUID";
-static const gchar PROP_FILTER_NAME[]            = "Name";
-static const gchar PROP_FILTER_GLOBAL[]          = "Global";
-static const gchar PROP_FILTER_UID[]             = "UID";
-static const gchar PROP_FILTER_ACTIVE[]          = "Active";
-static const gchar PROP_SEARCH_QUERY[]           = "Query";
-static const gchar PROP_SEARCH_SPEED[]           = "Speed";
-static const gchar PROP_SEARCH_PASSIVE[]         = "Passive";
-static const gchar PROP_RULE_TEXT_CASE[]         = "Case";
-static const gchar PROP_RULE_TEXT_MATCH[]        = "Match";
-static const gchar PROP_RULE_TEXT_TYPE[]         = "Type";
-static const gchar PROP_RULE_IP_ADDR[]           = "Address";
-static const gchar PROP_RULE_IP_MASK[]           = "Netmask";
-static const gchar PROP_RULE_SIZE_LOWER[]        = "Lower";
-static const gchar PROP_RULE_SIZE_UPPER[]        = "Upper";
-static const gchar PROP_RULE_SHA1_HASH[]         = "Hash";
-static const gchar PROP_RULE_SHA1_FILENAME[]     = "OriginalFilename";
-static const gchar PROP_RULE_NEGATE[]            = "Negate";
-static const gchar PROP_RULE_ACTIVE[]            = "Active";
-static const gchar PROP_RULE_SOFT[]              = "Soft";
-static const gchar PROP_RULE_TARGET[]            = "Target";
-static const gchar PROP_RULE_FLAG_BUSY[]         = "Busy";
-static const gchar PROP_RULE_FLAG_PUSH[]         = "Push";
-static const gchar PROP_RULE_FLAG_STABLE[]       = "Stable";
-static const gchar PROP_RULE_STATE_DISPLAY[]     = "Display";
-static const gchar PROP_RULE_STATE_DOWNLOAD[]    = "Download";
+static const gchar TAG_BUILTIN_SHOW_UID[]       = "ShowUID";
+static const gchar TAG_BUILTIN_DROP_UID[]       = "DropUID";
+static const gchar TAG_BUILTIN_DOWNLOAD_UID[]   = "DownloadUID";
+static const gchar TAG_BUILTIN_NODOWNLOAD_UID[] = "NoDownloadUID";
+static const gchar TAG_BUILTIN_RETURN_UID[]     = "ReturnUID";
+static const gchar TAG_FILTER_NAME[]            = "Name";
+static const gchar TAG_FILTER_GLOBAL[]          = "Global";
+static const gchar TAG_FILTER_UID[]             = "UID";
+static const gchar TAG_FILTER_ACTIVE[]          = "Active";
+static const gchar TAG_SEARCH_QUERY[]           = "Query";
+static const gchar TAG_SEARCH_ENABLED[]         = "Enabled";
+static const gchar TAG_SEARCH_SPEED[]           = "Speed";
+static const gchar TAG_SEARCH_PASSIVE[]         = "Passive";
+static const gchar TAG_SEARCH_REISSUE_TIMEOUT[] = "ReissueTimeout";
+static const gchar TAG_SEARCH_SORT_COL[]        = "SortCol";
+static const gchar TAG_SEARCH_SORT_ORDER[]      = "SortOrder";
+static const gchar TAG_RULE_TEXT_CASE[]         = "Case";
+static const gchar TAG_RULE_TEXT_MATCH[]        = "Match";
+static const gchar TAG_RULE_TEXT_TYPE[]         = "Type";
+static const gchar TAG_RULE_IP_ADDR[]           = "Address";
+static const gchar TAG_RULE_IP_MASK[]           = "Netmask";
+static const gchar TAG_RULE_SIZE_LOWER[]        = "Lower";
+static const gchar TAG_RULE_SIZE_UPPER[]        = "Upper";
+static const gchar TAG_RULE_SHA1_HASH[]         = "Hash";
+static const gchar TAG_RULE_SHA1_FILENAME[]     = "OriginalFilename";
+static const gchar TAG_RULE_NEGATE[]            = "Negate";
+static const gchar TAG_RULE_ACTIVE[]            = "Active";
+static const gchar TAG_RULE_SOFT[]              = "Soft";
+static const gchar TAG_RULE_TARGET[]            = "Target";
+static const gchar TAG_RULE_FLAG_BUSY[]         = "Busy";
+static const gchar TAG_RULE_FLAG_PUSH[]         = "Push";
+static const gchar TAG_RULE_FLAG_STABLE[]       = "Stable";
+static const gchar TAG_RULE_STATE_DISPLAY[]     = "Display";
+static const gchar TAG_RULE_STATE_DOWNLOAD[]    = "Download";
 
 
 
 static gchar x_tmp[4096];
-static gchar *search_file_xml = "searches.xml";
+static const gchar *search_file_xml = "searches.xml";
 static GHashTable *id_map = NULL;
 static node_parser_t parser_map[] = {
-    {(gchar *)NODE_BUILTIN,     xml_to_builtin},
-    {(gchar *)NODE_SEARCH,      xml_to_search},
-    {(gchar *)NODE_FILTER,      xml_to_filter},
-    {(gchar *)NODE_RULE_TEXT,   xml_to_text_rule},
-    {(gchar *)NODE_RULE_IP,     xml_to_ip_rule},
-    {(gchar *)NODE_RULE_SIZE,   xml_to_size_rule},
-    {(gchar *)NODE_RULE_JUMP,   xml_to_jump_rule},
-    {(gchar *)NODE_RULE_SHA1,   xml_to_sha1_rule},
-    {(gchar *)NODE_RULE_FLAG,   xml_to_flag_rule},
-    {(gchar *)NODE_RULE_STATE,  xml_to_state_rule},
+    {NODE_BUILTIN,     xml_to_builtin},
+    {NODE_SEARCH,      xml_to_search},
+    {NODE_FILTER,      xml_to_filter},
+    {NODE_RULE_TEXT,   xml_to_text_rule},
+    {NODE_RULE_IP,     xml_to_ip_rule},
+    {NODE_RULE_SIZE,   xml_to_size_rule},
+    {NODE_RULE_JUMP,   xml_to_jump_rule},
+    {NODE_RULE_SHA1,   xml_to_sha1_rule},
+    {NODE_RULE_FLAG,   xml_to_flag_rule},
+    {NODE_RULE_STATE,  xml_to_state_rule},
     {NULL, NULL}
 };
 
@@ -151,7 +161,7 @@ void search_store_xml(void)
 	time_t now = time((time_t *) NULL);
     xmlDocPtr doc;
     xmlNodePtr root;
-	gchar filename[1024];
+	gchar *filename_new;
 
     /* 
      * Create new xml document with version 1.0 
@@ -165,8 +175,7 @@ void search_store_xml(void)
     xmlDocSetRootElement(doc, root);
     xmlSetProp(root,"Time", ctime(&now));
 
-	g_snprintf(x_tmp, sizeof(x_tmp), "%u.%u", 
-        GTA_VERSION, GTA_SUBVERSION);
+	gm_snprintf(x_tmp, sizeof(x_tmp), "%s", version_number);
     xmlSetProp(root,"Version", x_tmp);
 
     /*
@@ -194,23 +203,39 @@ void search_store_xml(void)
      */
 
     xmlKeepBlanksDefault(0);
-    g_snprintf(x_tmp, sizeof(x_tmp), "%s/%s.new", config_dir, search_file_xml);
+    filename_new = g_strdup_printf("%s/%s.new",
+						settings_gui_config_dir(), search_file_xml);
 
-    if(xmlSaveFormatFile(x_tmp, doc, TRUE) == -1) {
+    if (
+		NULL == filename_new ||
+		xmlSaveFormatFile(filename_new, doc, TRUE) == -1
+	) {
         g_warning("Unable to create %s to persist search: %s",
-			x_tmp, g_strerror(errno));
+			filename_new, g_strerror(errno));
     } else {
-        if (dbg >= 3)
+		gchar *filename;
+
+        if (gui_debug >= 3)
             printf("saved searches file: %s\n", x_tmp);
 
-		g_snprintf(filename, sizeof(filename), "%s/%s",
-			config_dir, search_file_xml);
+		filename = g_strdup_printf("%s/%s",
+						settings_gui_config_dir(), search_file_xml);
 
-		if (-1 == rename(x_tmp, filename))
+		if (
+			NULL == filename ||
+			NULL == filename_new ||
+			-1 == rename(filename_new, filename)
+		)
 			g_warning("could not rename %s as %s: %s",
-				x_tmp, filename, g_strerror(errno));
+				filename_new, filename, g_strerror(errno));
+
+		if (NULL != filename)
+			G_FREE_NULL(filename);
     }
 
+	if (NULL != filename_new)
+		G_FREE_NULL(filename_new);
+
 	xmlFreeDoc(doc);
 }
 
@@ -227,28 +252,34 @@ gboolean search_retrieve_xml(void)
     xmlNodePtr node;
     xmlNodePtr root;
     GList *f;
+	gchar *path;
     
-  	g_snprintf(x_tmp, sizeof(x_tmp), "%s/%s", config_dir, search_file_xml);
-
+  	path = g_strdup_printf("%s/%s", settings_gui_config_dir(),
+			search_file_xml);
+	g_return_val_if_fail(NULL != path, FALSE);
+	
 	/* 
      * if the file doesn't exist 
      */
-	if(!file_exists(x_tmp)) {
-        g_warning("Searches file does not exist: %s", x_tmp);
+	if (!file_exists(path)) {
+        g_warning("Searches file does not exist: %s", path);
+		G_FREE_NULL(path);
 		return FALSE;
     }
+	
 
 	/* 
      * parse the new file and put the result into newdoc 
      */
-	doc = xmlParseFile(x_tmp);
+	doc = xmlParseFile(path);
     root = xmlDocGetRootElement(doc);
 
 	/* 
      * in case something went wrong 
      */
     if(!doc) {
-        g_warning("Error parsing searches file: %s", x_tmp);
+        g_warning("Error parsing searches file: %s", path);
+		G_FREE_NULL(path);
 		return FALSE;
     }
 
@@ -257,12 +288,14 @@ gboolean search_retrieve_xml(void)
 	    /* if it doesn't have a name */
 	    (root->name == NULL) ||
 	    /* if it isn't a Genealogy node */
-	    g_strcasecmp(root->name ,"Searches") != 0
+	    g_ascii_strcasecmp(root->name ,"Searches") != 0
     ) {
-        g_warning("Searches file has invalid format: %s", x_tmp);
+        g_warning("Searches file has invalid format: %s", path);
 		xmlFreeDoc(doc);
+		G_FREE_NULL(path);
 		return FALSE;
 	}
+	G_FREE_NULL(path);
 
     id_map = g_hash_table_new(NULL, NULL);
 
@@ -281,7 +314,7 @@ gboolean search_retrieve_xml(void)
      * shadows.
      */
 
-    if (dbg >= 6)
+    if (gui_debug >= 6)
         printf("resolving UIDs\n");
 
     for (f = filters; f != NULL; f = f->next) {
@@ -289,7 +322,7 @@ gboolean search_retrieve_xml(void)
         GList *r;
         gint n = 0;
 
-        if (dbg >= 6) {
+        if (gui_debug >= 6) {
             printf("\n\nresolving on filter:\n");
             dump_filter(filter);
         }
@@ -311,7 +344,7 @@ gboolean search_retrieve_xml(void)
                  * We circumwent the shadows, so we must do refcounting
                  * manually here.
                  */
-                if (dbg >= 7)
+                if (gui_debug >= 7)
                     printf("increasing refcount on \"%s\" to %d\n",
                         rule->target->name, rule->target->refcount+1);
                 rule->target->refcount ++;
@@ -319,7 +352,7 @@ gboolean search_retrieve_xml(void)
             }
         }
 
-        if (dbg >= 6) {
+        if (gui_debug >= 6) {
             printf("resolved filter:\n");
             dump_filter(filter);
         }
@@ -332,14 +365,14 @@ gboolean search_retrieve_xml(void)
         gboolean borked = FALSE;
         GList *s;
 
-        if (dbg >= 6)
+        if (gui_debug >= 6)
             printf("verifying bindings...\n");
 
         for (s = searches; s != NULL; s = s->next) {
             search_t * search = (search_t *)s->data;
 
             if (search->filter->search == search) {
-                if (dbg >= 6)
+                if (gui_debug >= 6)
                     printf("binding ok for: %s\n", search->query);
             } else {
                 g_warning("binding broken for: %s\n", search->query);
@@ -369,20 +402,20 @@ static void builtin_to_xml(xmlNodePtr parent)
 
     newxml = xmlNewChild(parent,NULL,NODE_BUILTIN, NULL);
     
-  	g_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_show_target());
-    xmlSetProp(newxml,PROP_BUILTIN_SHOW_UID, x_tmp);
+  	gm_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_show_target());
+    xmlSetProp(newxml,TAG_BUILTIN_SHOW_UID, x_tmp);
 
-  	g_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_drop_target());
-    xmlSetProp(newxml,PROP_BUILTIN_DROP_UID, x_tmp);
+  	gm_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_drop_target());
+    xmlSetProp(newxml,TAG_BUILTIN_DROP_UID, x_tmp);
 
-  	g_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_download_target());
-    xmlSetProp(newxml,PROP_BUILTIN_DOWNLOAD_UID, x_tmp);
+  	gm_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_download_target());
+    xmlSetProp(newxml,TAG_BUILTIN_DOWNLOAD_UID, x_tmp);
 
-    g_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_nodownload_target());
-    xmlSetProp(newxml,PROP_BUILTIN_NODOWNLOAD_UID, x_tmp);
+    gm_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_nodownload_target());
+    xmlSetProp(newxml,TAG_BUILTIN_NODOWNLOAD_UID, x_tmp);
 
-    g_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_return_target());
-    xmlSetProp(newxml,PROP_BUILTIN_RETURN_UID, x_tmp);
+    gm_snprintf(x_tmp, sizeof(x_tmp), "%p", filter_get_return_target());
+    xmlSetProp(newxml,TAG_BUILTIN_RETURN_UID, x_tmp);
 }
 
 static void search_to_xml(xmlNodePtr parent, search_t *s)
@@ -394,22 +427,36 @@ static void search_to_xml(xmlNodePtr parent, search_t *s)
     g_assert(s->query != NULL);
     g_assert(parent != NULL);
 
-    if (dbg >= 6) {
-        printf("saving search: %s\n", s->query);
+    if (gui_debug >= 6) {
+        printf("saving search: %s (%p enabled=%d)\n", s->query, s, s->enabled);
         printf("  -- filter is bound to: %p\n", s->filter->search);
         printf("  -- search is         : %p\n", s);
     }
 
     newxml = xmlNewChild(parent, NULL, NODE_SEARCH, NULL);
     
-    xmlSetProp(newxml, PROP_SEARCH_QUERY, s->query);
+    xmlSetProp(newxml, TAG_SEARCH_QUERY, s->query);
 
-  	g_snprintf(x_tmp, sizeof(x_tmp), "%u", s->speed);
-    xmlSetProp(newxml, PROP_SEARCH_SPEED, x_tmp);
+    gm_snprintf(x_tmp, sizeof(x_tmp), "%u", s->enabled);
+    xmlSetProp(newxml, TAG_SEARCH_ENABLED, x_tmp);
+
+  	gm_snprintf(x_tmp, sizeof(x_tmp), "%u", 
+        search_get_minimum_speed(s->search_handle));
+    xmlSetProp(newxml, TAG_SEARCH_SPEED, x_tmp);
+
+    gm_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(s->passive));
+    xmlSetProp(newxml, TAG_SEARCH_PASSIVE, x_tmp);
+
+  	gm_snprintf(x_tmp, sizeof(x_tmp), "%u", 
+        search_get_reissue_timeout(s->search_handle));
+    xmlSetProp(newxml, TAG_SEARCH_REISSUE_TIMEOUT, x_tmp);
+
+  	gm_snprintf(x_tmp, sizeof(x_tmp), "%i", s->sort_col);
+    xmlSetProp(newxml, TAG_SEARCH_SORT_COL, x_tmp);
+
+  	gm_snprintf(x_tmp, sizeof(x_tmp), "%i", s->sort_order);
+    xmlSetProp(newxml, TAG_SEARCH_SORT_ORDER, x_tmp);
 
-    g_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(s->passive));
-    xmlSetProp(newxml, PROP_SEARCH_PASSIVE, x_tmp);
-    
     for (l = s->filter->ruleset; l != NULL; l = l->next)
         rule_to_xml(newxml, (rule_t *)l->data);
 }
@@ -428,39 +475,39 @@ static void filter_to_xml(xmlNodePtr parent, filter_t *f)
      * Don't store the builtin targets or bound rulesets
      */
     if (filter_is_builtin(f) || filter_is_bound(f)) {
-        if (dbg >= 7)
+        if (gui_debug >= 7)
             printf("not saving bound/builtin: %s\n", f->name);
         return;
     }
 
-    if (dbg >= 6) {
+    if (gui_debug >= 6) {
         printf("saving filter: %s\n", f->name);
         printf("  -- bound   : %p\n", f->search);
     }
 
     newxml = xmlNewChild(parent, NULL, NODE_FILTER, NULL);
     
-    xmlSetProp(newxml, PROP_FILTER_NAME, f->name);
+    xmlSetProp(newxml, TAG_FILTER_NAME, f->name);
 
-    g_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(filter_is_active(f)));
-    xmlSetProp(newxml, PROP_FILTER_ACTIVE, x_tmp);
+    gm_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(filter_is_active(f)));
+    xmlSetProp(newxml, TAG_FILTER_ACTIVE, x_tmp);
 
     /*
      * We take the pointer as a unique id which
      * we use during read-in for setting the
      * destination of JUMP actions.
      */
-  	g_snprintf(x_tmp, sizeof(x_tmp), "%p", f);
-    xmlSetProp(newxml, PROP_FILTER_UID, x_tmp);
+  	gm_snprintf(x_tmp, sizeof(x_tmp), "%p", f);
+    xmlSetProp(newxml, TAG_FILTER_UID, x_tmp);
 
     if (filter_get_global_pre() == f) {
-    	g_snprintf(x_tmp, sizeof(x_tmp), "%u", GLOBAL_PRE);
-        xmlSetProp(newxml, PROP_FILTER_GLOBAL, x_tmp); 
+    	gm_snprintf(x_tmp, sizeof(x_tmp), "%u", GLOBAL_PRE);
+        xmlSetProp(newxml, TAG_FILTER_GLOBAL, x_tmp); 
     }
 
     if (filter_get_global_post() == f) {
-    	g_snprintf(x_tmp, sizeof(x_tmp), "%u", GLOBAL_POST);
-        xmlSetProp(newxml, PROP_FILTER_GLOBAL, x_tmp); 
+    	gm_snprintf(x_tmp, sizeof(x_tmp), "%u", GLOBAL_POST);
+        xmlSetProp(newxml, TAG_FILTER_GLOBAL, x_tmp); 
     }
 
     /* 
@@ -489,27 +536,27 @@ static void rule_to_xml(xmlNodePtr parent, rule_t *r)
     case RULE_TEXT:
         newxml = xmlNewChild(parent, NULL, NODE_RULE_TEXT, NULL);
 
-        xmlSetProp(newxml, PROP_RULE_TEXT_CASE, 
+        xmlSetProp(newxml, TAG_RULE_TEXT_CASE, 
             r->u.text.case_sensitive ? "1" : "0");
-        xmlSetProp(newxml, PROP_RULE_TEXT_MATCH, r->u.text.match);
+        xmlSetProp(newxml, TAG_RULE_TEXT_MATCH, r->u.text.match);
 
-        g_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.text.type);
-        xmlSetProp(newxml, PROP_RULE_TEXT_TYPE, x_tmp);
+        gm_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.text.type);
+        xmlSetProp(newxml, TAG_RULE_TEXT_TYPE, x_tmp);
         break;
     case RULE_IP:
         newxml = xmlNewChild(parent, NULL, NODE_RULE_IP, NULL);
 
-        xmlSetProp(newxml,PROP_RULE_IP_ADDR, ip_to_gchar(r->u.ip.addr));
-        xmlSetProp(newxml,PROP_RULE_IP_MASK, ip_to_gchar(r->u.ip.mask));
+        xmlSetProp(newxml,TAG_RULE_IP_ADDR, ip_to_gchar(r->u.ip.addr));
+        xmlSetProp(newxml,TAG_RULE_IP_MASK, ip_to_gchar(r->u.ip.mask));
         break;
     case RULE_SIZE:
         newxml = xmlNewChild(parent, NULL, NODE_RULE_SIZE, NULL);
         
-        g_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.size.lower);
-        xmlSetProp(newxml, PROP_RULE_SIZE_LOWER, x_tmp);
+        gm_snprintf(x_tmp, sizeof(x_tmp), "%lu", (gulong) r->u.size.lower);
+        xmlSetProp(newxml, TAG_RULE_SIZE_LOWER, x_tmp);
 
-        g_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.size.upper);
-        xmlSetProp(newxml, PROP_RULE_SIZE_UPPER, x_tmp);
+        gm_snprintf(x_tmp, sizeof(x_tmp), "%lu", (gulong) r->u.size.upper);
+        xmlSetProp(newxml, TAG_RULE_SIZE_UPPER, x_tmp);
         break;
     case RULE_JUMP:
         newxml = xmlNewChild(parent, NULL, NODE_RULE_JUMP, NULL);
@@ -523,9 +570,9 @@ static void rule_to_xml(xmlNodePtr parent, rule_t *r)
 
         if (r->u.sha1.hash != NULL)
             xmlSetProp
-                (newxml,PROP_RULE_SHA1_HASH, sha1_base32(r->u.sha1.hash));
+                (newxml,TAG_RULE_SHA1_HASH, sha1_base32(r->u.sha1.hash));
 
-        xmlSetProp(newxml, PROP_RULE_SHA1_FILENAME, r->u.sha1.filename);
+        xmlSetProp(newxml, TAG_RULE_SHA1_FILENAME, r->u.sha1.filename);
         
         /*
          * r->u.sha1.hash is NULL, we just omit the hash.
@@ -534,39 +581,39 @@ static void rule_to_xml(xmlNodePtr parent, rule_t *r)
     case RULE_FLAG:
         newxml = xmlNewChild(parent, NULL, NODE_RULE_FLAG, NULL);
         
-        g_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.flag.stable);
-        xmlSetProp(newxml, PROP_RULE_FLAG_STABLE, x_tmp);
+        gm_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.flag.stable);
+        xmlSetProp(newxml, TAG_RULE_FLAG_STABLE, x_tmp);
 
-        g_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.flag.busy);
-        xmlSetProp(newxml, PROP_RULE_FLAG_BUSY, x_tmp);
+        gm_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.flag.busy);
+        xmlSetProp(newxml, TAG_RULE_FLAG_BUSY, x_tmp);
 
-        g_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.flag.push);
-        xmlSetProp(newxml, PROP_RULE_FLAG_PUSH, x_tmp);
+        gm_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.flag.push);
+        xmlSetProp(newxml, TAG_RULE_FLAG_PUSH, x_tmp);
         break;
     case RULE_STATE:
          newxml = xmlNewChild(parent, NULL, NODE_RULE_STATE, NULL);
         
-        g_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.state.display);
-        xmlSetProp(newxml, PROP_RULE_STATE_DISPLAY, x_tmp);
+        gm_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.state.display);
+        xmlSetProp(newxml, TAG_RULE_STATE_DISPLAY, x_tmp);
 
-        g_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.state.download);
-        xmlSetProp(newxml, PROP_RULE_STATE_DOWNLOAD, x_tmp);
+        gm_snprintf(x_tmp, sizeof(x_tmp), "%u", r->u.state.download);
+        xmlSetProp(newxml, TAG_RULE_STATE_DOWNLOAD, x_tmp);
         break;
     default:
         g_error("Unknown rule type: %d", r->type);
     }
 
-    g_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(RULE_IS_NEGATED(r)));
-    xmlSetProp(newxml, PROP_RULE_NEGATE, x_tmp);
+    gm_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(RULE_IS_NEGATED(r)));
+    xmlSetProp(newxml, TAG_RULE_NEGATE, x_tmp);
 
-    g_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(RULE_IS_ACTIVE(r)));
-    xmlSetProp(newxml, PROP_RULE_ACTIVE, x_tmp);
+    gm_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(RULE_IS_ACTIVE(r)));
+    xmlSetProp(newxml, TAG_RULE_ACTIVE, x_tmp);
 
-    g_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(RULE_IS_SOFT(r)));
-    xmlSetProp(newxml, PROP_RULE_SOFT, x_tmp);
+    gm_snprintf(x_tmp, sizeof(x_tmp), "%u", TO_BOOL(RULE_IS_SOFT(r)));
+    xmlSetProp(newxml, TAG_RULE_SOFT, x_tmp);
 
-    g_snprintf(x_tmp, sizeof(x_tmp), "%p", r->target);
-    xmlSetProp(newxml, PROP_RULE_TARGET, x_tmp);
+    gm_snprintf(x_tmp, sizeof(x_tmp), "%p", r->target);
+    xmlSetProp(newxml, TAG_RULE_TARGET, x_tmp);
 }
 
 static void parse_xml(xmlNodePtr xmlnode, gpointer user_data)
@@ -584,7 +631,7 @@ static void parse_xml(xmlNodePtr xmlnode, gpointer user_data)
     } 
 
     for (n = 0; parser_map[n].name != NULL; n ++) {
-        if (g_strcasecmp(xmlnode->name, parser_map[n].name) == 0) {
+        if (g_ascii_strcasecmp(xmlnode->name, parser_map[n].name) == 0) {
             parser_map[n].parser_func(xmlnode, user_data);
             return;
         }
@@ -600,12 +647,12 @@ static void xml_to_builtin(xmlNodePtr xmlnode, gpointer user_data)
 
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_BUILTIN) == 0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_BUILTIN) == 0);
     g_assert(filter_get_show_target() != NULL);
     g_assert(filter_get_drop_target() != NULL);
     g_assert(filter_get_download_target() != NULL);
 
-    buf = xmlGetProp(xmlnode, PROP_BUILTIN_SHOW_UID);
+    buf = xmlGetProp(xmlnode, TAG_BUILTIN_SHOW_UID);
     g_assert(buf != NULL);
     errno = 0;
     target = (gpointer) strtoul(buf, 0, 16);
@@ -614,7 +661,7 @@ static void xml_to_builtin(xmlNodePtr xmlnode, gpointer user_data)
     g_free(buf);
     g_hash_table_insert(id_map, target, filter_get_show_target());
 
-    buf = xmlGetProp(xmlnode, PROP_BUILTIN_DROP_UID);
+    buf = xmlGetProp(xmlnode, TAG_BUILTIN_DROP_UID);
     g_assert(buf != NULL);
     errno = 0;
     target = (gpointer) strtoul(buf, 0, 16);
@@ -623,7 +670,7 @@ static void xml_to_builtin(xmlNodePtr xmlnode, gpointer user_data)
     g_free(buf);
     g_hash_table_insert(id_map, target, filter_get_drop_target());
 
-    buf = xmlGetProp(xmlnode, PROP_BUILTIN_DOWNLOAD_UID);
+    buf = xmlGetProp(xmlnode, TAG_BUILTIN_DOWNLOAD_UID);
     if (buf != NULL) {
         errno = 0;
         target = (gpointer) strtoul(buf, 0, 16);
@@ -635,7 +682,7 @@ static void xml_to_builtin(xmlNodePtr xmlnode, gpointer user_data)
         g_warning("xml_to_builtin: no \"DOWNLOAD\" target");
     }
     
-    buf = xmlGetProp(xmlnode, PROP_BUILTIN_NODOWNLOAD_UID);
+    buf = xmlGetProp(xmlnode, TAG_BUILTIN_NODOWNLOAD_UID);
     if (buf != NULL) {
         errno = 0;
         target = (gpointer) strtoul(buf, 0, 16);
@@ -647,7 +694,7 @@ static void xml_to_builtin(xmlNodePtr xmlnode, gpointer user_data)
         g_warning("xml_to_builtin: no \"DON'T DOWNLOAD\" target");
     }
 
-    buf = xmlGetProp(xmlnode, PROP_BUILTIN_RETURN_UID);
+    buf = xmlGetProp(xmlnode, TAG_BUILTIN_RETURN_UID);
     if (buf != NULL) {
         errno = 0;
         target = (gpointer) strtoul(buf, 0, 16);
@@ -664,41 +711,74 @@ static void xml_to_search(xmlNodePtr xmlnode, gpointer user_data)
 {
     gchar *buf;
     gchar *query;
-    gint32 speed = 0;
+    gint sort_col = SORT_NO_COL, sort_order = SORT_NONE;
+    gint32 speed;
+    guint32 reissue_timeout;
     xmlNodePtr node;
     search_t * search;
-    gboolean passive = FALSE;
-    guint flags;
+    guint flags = 0;
 
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_SEARCH) == 0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_SEARCH) == 0);
 
-	buf = xmlGetProp(xmlnode, PROP_SEARCH_QUERY);
+    gui_prop_get_guint32_val(PROP_DEFAULT_MINIMUM_SPEED, &speed);
+    gnet_prop_get_guint32_val(PROP_SEARCH_REISSUE_TIMEOUT, &reissue_timeout);
+
+	buf = xmlGetProp(xmlnode, TAG_SEARCH_QUERY);
     if (!buf) {
         g_warning("Ignored search without query");
         return;
     }
     query = buf;
 
-    buf = xmlGetProp(xmlnode, PROP_SEARCH_SPEED);
+    buf = xmlGetProp(xmlnode, TAG_SEARCH_ENABLED);
+    if (buf) {
+        if (atoi(buf) == 1)
+			flags |= SEARCH_ENABLED;
+        g_free(buf);
+    } else
+		flags |= SEARCH_ENABLED;	 /* Compatibility: searches always began */
+
+    buf = xmlGetProp(xmlnode, TAG_SEARCH_SPEED);
     if (buf) {
         speed = atol(buf);
         g_free(buf);
     }
 
-    buf = xmlGetProp(xmlnode, PROP_SEARCH_PASSIVE);
-    if (buf != NULL) {
-        passive = atol(buf) == 1 ? TRUE : FALSE;
+    buf = xmlGetProp(xmlnode, TAG_SEARCH_REISSUE_TIMEOUT);
+    if (buf) {
+        reissue_timeout = atol(buf);
         g_free(buf);
     }
 
-    flags =
-        (passive ? SEARCH_PASSIVE : 0);
+    buf = xmlGetProp(xmlnode, TAG_SEARCH_PASSIVE);
+    if (buf) {
+        if (atol(buf) == 1)
+			flags |= SEARCH_PASSIVE;
+        g_free(buf);
+    }
 
-    if (dbg >= 4)
-        printf("adding new search: %s\n", query);
-    search = _new_search(speed, query, flags);
+    buf = xmlGetProp(xmlnode, TAG_SEARCH_SORT_COL);
+    if (buf) {
+        sort_col = atol(buf);
+        g_free(buf);
+    }
+
+	buf = xmlGetProp(xmlnode, TAG_SEARCH_SORT_ORDER);
+    if (buf) {
+        sort_order = atol(buf);
+        g_free(buf);
+    }
+
+    if (gui_debug >= 4)
+        printf("adding new %s %s search: %s\n",
+			(flags & SEARCH_ENABLED) ? "enabled" : "disabled",
+			(flags & SEARCH_PASSIVE) ? "passive" : "active",
+			query);
+
+	search_gui_new_search_full(query, speed, reissue_timeout,
+		sort_col, sort_order, flags, &search);
 
     g_free(query);
 
@@ -720,16 +800,16 @@ static void xml_to_filter(xmlNodePtr xmlnode, gpointer user_data)
 
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_FILTER) == 0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_FILTER) == 0);
 
-	buf = xmlGetProp(xmlnode, PROP_FILTER_NAME);
+	buf = xmlGetProp(xmlnode, TAG_FILTER_NAME);
     if (!buf) {
         g_warning("Ignored unnamed filter");
         return;
     }
     name = buf;
 
-    buf = xmlGetProp(xmlnode, PROP_FILTER_GLOBAL);
+    buf = xmlGetProp(xmlnode, TAG_FILTER_GLOBAL);
     if (buf) {
         gint t;
         errno = 0;
@@ -747,14 +827,15 @@ static void xml_to_filter(xmlNodePtr xmlnode, gpointer user_data)
             filter = NULL;
             g_assert_not_reached();
         };
+        g_free(buf);
     } else {
-        if (dbg >= 4)
+        if (gui_debug >= 4)
             printf("adding new filter: %s\n", name);
         filter = filter_new(name);
         filters = g_list_append(filters, filter);
     }
 
-    buf = xmlGetProp(xmlnode, PROP_FILTER_ACTIVE);
+    buf = xmlGetProp(xmlnode, TAG_FILTER_ACTIVE);
     if (buf != NULL) {
         active = atol(buf) == 1 ? TRUE : FALSE;
         g_free(buf);
@@ -764,7 +845,7 @@ static void xml_to_filter(xmlNodePtr xmlnode, gpointer user_data)
     else
         clear_flags(filter->flags, FILTER_FLAG_ACTIVE);
 
-    buf = xmlGetProp(xmlnode, PROP_FILTER_UID);
+    buf = xmlGetProp(xmlnode, TAG_FILTER_UID);
     g_assert(buf);
     errno = 0;
     dest = (gpointer) strtoul(buf, 0, 16);
@@ -795,21 +876,21 @@ static void xml_to_text_rule(xmlNodePtr xmlnode, gpointer filter)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_RULE_TEXT) ==0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_RULE_TEXT) ==0);
 
-    match = xmlGetProp(xmlnode, PROP_RULE_TEXT_MATCH);
+    match = xmlGetProp(xmlnode, TAG_RULE_TEXT_MATCH);
     if (match == NULL)
         g_error("xml_to_text_rule: rule without match string");
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_TEXT_CASE);
+    buf = xmlGetProp(xmlnode, TAG_RULE_TEXT_CASE);
     case_sensitive = atol(buf) == 1 ? TRUE : FALSE;
     g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_TEXT_TYPE);
+    buf = xmlGetProp(xmlnode, TAG_RULE_TEXT_TYPE);
     type = (enum rule_text_type) atol(buf);
     g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_TARGET);
+    buf = xmlGetProp(xmlnode, TAG_RULE_TARGET);
     errno = 0;
     target = (gpointer) strtoul(buf, 0, 16);
     if (errno != 0)
@@ -821,7 +902,7 @@ static void xml_to_text_rule(xmlNodePtr xmlnode, gpointer filter)
         (match, type, case_sensitive, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (dbg >= 4)
+    if (gui_debug >= 4)
         printf( "added to filter \"%s\" rule with target %p\n",
             ((filter_t *)filter)->name, rule->target);
 
@@ -843,21 +924,21 @@ static void xml_to_ip_rule(xmlNodePtr xmlnode, gpointer filter)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_RULE_IP) ==0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_RULE_IP) ==0);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_IP_ADDR);
+    buf = xmlGetProp(xmlnode, TAG_RULE_IP_ADDR);
     if (buf == NULL)
         g_error("xml_to_ip_rule: rule without ip address");
     addr = gchar_to_ip(buf);
     g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_IP_MASK);
+    buf = xmlGetProp(xmlnode, TAG_RULE_IP_MASK);
     if (buf == NULL)
         g_error("xml_to_ip_rule: rule without netmask");
     mask = gchar_to_ip(buf);
     g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_TARGET);
+    buf = xmlGetProp(xmlnode, TAG_RULE_TARGET);
     errno = 0;
     target = (gpointer) strtoul(buf, 0, 16);
     if (errno != 0)
@@ -868,7 +949,7 @@ static void xml_to_ip_rule(xmlNodePtr xmlnode, gpointer filter)
     rule = filter_new_ip_rule(addr, mask, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (dbg >= 4)
+    if (gui_debug >= 4)
         printf( "added to filter \"%s\" rule with target %p\n",
             ((filter_t *)filter)->name, rule->target);
 
@@ -888,21 +969,21 @@ static void xml_to_size_rule(xmlNodePtr xmlnode, gpointer filter)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_RULE_SIZE) ==0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_RULE_SIZE) ==0);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_SIZE_LOWER);
+    buf = xmlGetProp(xmlnode, TAG_RULE_SIZE_LOWER);
     if (buf == NULL)
         g_error("xml_to_size_rule: rule without lower bound");
     lower = atol(buf);
     g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_SIZE_UPPER);
+    buf = xmlGetProp(xmlnode, TAG_RULE_SIZE_UPPER);
     if (buf == NULL)
         g_error("xml_to_size_rule: rule without upper bound");
     upper = atol(buf);
     g_free(buf);
  
-    buf = xmlGetProp(xmlnode, PROP_RULE_TARGET);
+    buf = xmlGetProp(xmlnode, TAG_RULE_TARGET);
     errno = 0;
     target = (gpointer) strtoul(buf, 0, 16);
     if (errno != 0)
@@ -913,7 +994,7 @@ static void xml_to_size_rule(xmlNodePtr xmlnode, gpointer filter)
     rule = filter_new_size_rule(lower, upper, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (dbg >= 4)
+    if (gui_debug >= 4)
         printf( "added to filter \"%s\" rule with target %p\n",
             ((filter_t *)filter)->name, rule->target);
 
@@ -931,9 +1012,10 @@ static void xml_to_jump_rule(xmlNodePtr xmlnode, gpointer filter)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_RULE_JUMP) ==0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_RULE_JUMP) ==0);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_TARGET);
+    buf = xmlGetProp(xmlnode, TAG_RULE_TARGET);
+    g_assert(buf != NULL);
     errno = 0;
     target = (gpointer) strtoul(buf, 0, 16);
     if (errno != 0)
@@ -944,7 +1026,7 @@ static void xml_to_jump_rule(xmlNodePtr xmlnode, gpointer filter)
     rule = filter_new_jump_rule(target,flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (dbg >= 4)
+    if (gui_debug >= 4)
         printf( "added to filter \"%s\" rule with target %p\n",
             ((filter_t *)filter)->name, rule->target);
 
@@ -964,20 +1046,18 @@ static void xml_to_sha1_rule(xmlNodePtr xmlnode, gpointer filter)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_RULE_SHA1) ==0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_RULE_SHA1) ==0);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_SHA1_FILENAME);
-    if (buf != NULL)
-        filename = buf;
-    else
-        filename = g_strdup("[Unknown]");
+    buf = xmlGetProp(xmlnode, TAG_RULE_SHA1_FILENAME);
+    filename = (buf != NULL) ? buf : g_strdup("[Unknown]");
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_SHA1_HASH);
+    buf = xmlGetProp(xmlnode, TAG_RULE_SHA1_HASH);
     if ((buf != NULL) && (strlen(buf) == SHA1_BASE32_SIZE))
         hash = base32_sha1(buf);
     g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_TARGET);
+    buf = xmlGetProp(xmlnode, TAG_RULE_TARGET);
+    g_assert(buf != NULL);
     errno = 0;
     target = (gpointer) strtoul(buf, 0, 16);
     if (errno != 0)
@@ -988,7 +1068,9 @@ static void xml_to_sha1_rule(xmlNodePtr xmlnode, gpointer filter)
     rule = filter_new_sha1_rule(hash, filename, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (dbg >= 4)
+    g_free(filename);
+
+    if (gui_debug >= 4)
         printf( "added to filter \"%s\" rule with target %p\n",
             ((filter_t *)filter)->name, rule->target);
 
@@ -1009,33 +1091,34 @@ static void xml_to_flag_rule(xmlNodePtr xmlnode, gpointer filter)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_RULE_FLAG) ==0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_RULE_FLAG) ==0);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_FLAG_STABLE);
+    buf = xmlGetProp(xmlnode, TAG_RULE_FLAG_STABLE);
     if (buf != NULL) {
         gint val = atol(buf);
         if ((val == RULE_FLAG_SET) || (val == RULE_FLAG_UNSET))
             stable = val;
+        g_free(buf);
     }
-    g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_FLAG_BUSY);
+    buf = xmlGetProp(xmlnode, TAG_RULE_FLAG_BUSY);
     if (buf != NULL) {
         gint val = atol(buf);
         if ((val == RULE_FLAG_SET) || (val == RULE_FLAG_UNSET))
             busy = val;
+        g_free(buf);
     }
-    g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_FLAG_PUSH);
+    buf = xmlGetProp(xmlnode, TAG_RULE_FLAG_PUSH);
     if (buf != NULL) {
         gint val = atol(buf);
         if ((val == RULE_FLAG_SET) || (val == RULE_FLAG_UNSET))
             push = val;
+        g_free(buf);
     }
-    g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_TARGET);
+    buf = xmlGetProp(xmlnode, TAG_RULE_TARGET);
+    g_assert(buf != NULL);
     errno = 0;
     target = (gpointer) strtoul(buf, 0, 16);
     if (errno != 0)
@@ -1046,7 +1129,7 @@ static void xml_to_flag_rule(xmlNodePtr xmlnode, gpointer filter)
     rule = filter_new_flag_rule(stable, busy, push, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (dbg >= 4)
+    if (gui_debug >= 4)
         printf( "added to filter \"%s\" rule with target %p\n",
             ((filter_t *)filter)->name, rule->target);
 
@@ -1066,27 +1149,28 @@ static void xml_to_state_rule(xmlNodePtr xmlnode, gpointer filter)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     g_assert(filter != NULL);
-    g_assert(g_strcasecmp(xmlnode->name, NODE_RULE_STATE) ==0);
+    g_assert(g_ascii_strcasecmp(xmlnode->name, NODE_RULE_STATE) ==0);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_STATE_DISPLAY);
+    buf = xmlGetProp(xmlnode, TAG_RULE_STATE_DISPLAY);
     if (buf != NULL) {
         gint val = atol(buf);
         if (((val >= 0) && (val <= MAX_FILTER_PROP_STATE)) ||
             (val == FILTER_PROP_STATE_IGNORE))
             display = val;
+        g_free(buf);
     }
-    g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_STATE_DOWNLOAD);
+    buf = xmlGetProp(xmlnode, TAG_RULE_STATE_DOWNLOAD);
     if (buf != NULL) {
         gint val = atol(buf);
         if (((val >= 0) && (val <= MAX_FILTER_PROP_STATE)) ||
             (val == FILTER_PROP_STATE_IGNORE))
             download = val;
+        g_free(buf);
     }
-    g_free(buf);
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_TARGET);
+    buf = xmlGetProp(xmlnode, TAG_RULE_TARGET);
+    g_assert(buf != NULL);
     errno = 0;
     target = (gpointer) strtoul(buf, 0, 16);
     if (errno != 0)
@@ -1097,7 +1181,7 @@ static void xml_to_state_rule(xmlNodePtr xmlnode, gpointer filter)
     rule = filter_new_state_rule(display, download, target, flags);
     clear_flags(rule->flags, RULE_FLAG_VALID);
 
-    if (dbg >= 4)
+    if (gui_debug >= 4)
         printf( "added to filter \"%s\" rule with target %p\n",
             ((filter_t *)filter)->name, rule->target);
 
@@ -1116,19 +1200,19 @@ static guint16 get_rule_flags_from_xml(xmlNodePtr xmlnode)
     g_assert(xmlnode != NULL);
     g_assert(xmlnode->name != NULL);
     
-    buf = xmlGetProp(xmlnode, PROP_RULE_NEGATE);
+    buf = xmlGetProp(xmlnode, TAG_RULE_NEGATE);
     if (buf != NULL) {
         negate = atol(buf) == 1 ? TRUE : FALSE;
         g_free(buf);
     }
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_ACTIVE);
+    buf = xmlGetProp(xmlnode, TAG_RULE_ACTIVE);
     if (buf != NULL) {
         active = atol(buf) == 1 ? TRUE : FALSE;
         g_free(buf);
     }
 
-    buf = xmlGetProp(xmlnode, PROP_RULE_SOFT);
+    buf = xmlGetProp(xmlnode, TAG_RULE_SOFT);
     if (buf != NULL) {
         soft = atol(buf) == 1 ? TRUE : FALSE;
         g_free(buf);
