@@ -1,4 +1,6 @@
 /*
+ * $Id: search_xml.h,v 1.3 2003/01/01 00:14:37 cbiere Exp $
+ *
  * Copyright (c) 2002, Richard Eckart
  *
  * Persistance for searches and filters in XML format.
@@ -23,10 +25,10 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __search_xml_h__
-#define __search_xml_h__
+#ifndef _search_xml_h_
+#define _search_xml_h_
 
 void search_store_xml(void);
 gboolean search_retrieve_xml(void);
 
-#endif	/* __search_xml_h__ */
+#endif	/* _search_xml_h_ */
