@@ -1,5 +1,8 @@
 /*
+ * $Id: sockets.c,v 1.90 2003/05/17 19:24:09 rmanfredi Exp $
+ *
  * Copyright (c) 2001-2002, Raphael Manfredi
+ * Copyright (c) 2000 Daniel Walker (dwalker@cats.ucsc.edu)
  *
  * Socket management.
  *
@@ -36,49 +39,201 @@
 #include <arpa/inet.h>
 #include <netinet/tcp.h>
 
-#include "appconfig.h"		/* For local_ip */
+#if defined(HAVE_IP_TOS) && defined(NEED_NETINET_IP_H_FOR_TOS)
+#include <netinet/in_systm.h>
+#include <netinet/ip.h>
+#endif
+
 #include "sockets.h"
 #include "downloads.h"
 #include "uploads.h"
+#include "parq.h"
 #include "nodes.h"
-#include "misc.h"
 #include "header.h"
-#include "getline.h"
 #include "bsched.h"
-#include "gui.h"			/* For gui_update_config_port() */
 #include "ban.h"
 #include "http.h"
+#include "settings.h"
+#include "inet.h"
+#include "walloc.h"
+#include "adns.h"
+#include "hostiles.h"
 
-#if !defined(SOL_TCP) && defined(IPPROTO_TCP)
-#define SOL_TCP IPPROTO_TCP
+#ifdef USE_REMOTE_SHELL
+#include "shell.h"
 #endif
 
+RCSID("$Id: sockets.c,v 1.90 2003/05/17 19:24:09 rmanfredi Exp $");
+
 #ifndef SHUT_WR
 #define SHUT_WR 1		/* Shutdown TX side */
 #endif
 
 #define RQST_LINE_LENGTH	256		/* Reasonable estimate for request line */
 
-gboolean is_firewalled = TRUE;		/* Assume the worst --RAM, 20/12/2001 */
+#define SOCK_ADNS_PENDING	1	/* Prevents free()ing the socket too early */
+#define SOCK_ADNS_FAILED	2	/* Indicates an error in the ADNS callback */
+
 static gboolean ip_computed = FALSE;
 
-static GSList *sl_incoming = (GSList *) NULL;	/* Track incoming sockets */
+static GSList *sl_incoming = (GSList *) NULL;	/* To spot inactive sockets */
 
 static void guess_local_ip(int sd);
+static void socket_destroy(struct gnutella_socket *s, gchar *reason);
 
-void socket_monitor_incoming(void)
+/* 
+ * SOL_TCP and SOL_IP aren't standards. Some platforms define them, on
+ * some it's safe to assume they're the same as IPPROTO_*, but the
+ * only way to be portably safe is to use protoent functions.
+ *
+ * If the user changes /etc/protocols while running gtkg, things may
+ * go badly.
+ */
+static gboolean sol_got = FALSE;
+static gint sol_tcp_cached = -1;
+static gint sol_ip_cached = -1;
+
+static void get_sol(void)
 {
-	/* Did an incoming connection timout? */
+	struct protoent *pent;
 
+	pent = getprotobyname("tcp");
+	if (NULL != pent)
+		sol_tcp_cached = pent->p_proto;
+	pent = getprotobyname("ip");
+	if (NULL != pent)
+		sol_ip_cached = pent->p_proto;
+	sol_got = TRUE;
+}
+
+static gint sol_tcp(void)
+{
+	if (!sol_got) /* FIXME: This should be done in socket_init() */
+		get_sol();
+	return sol_tcp_cached;
+}
+
+#ifdef HAVE_IP_TOS
+  
+static gint sol_ip(void)
+{
+	if (!sol_got) /* FIXME: This should be done in socket_init() */
+		get_sol();
+	return sol_ip_cached;
+}
+
+/*
+ * socket_tos
+ *
+ * Set the TOS on the socket.  Routers can use this information to
+ * better route the IP datagrams.
+ */
+static void socket_tos(struct gnutella_socket *s, gint tos)
+{
+	if (!use_ip_tos)
+		return;
+	if (
+		-1 == setsockopt(s->file_desc, sol_ip(),
+				IP_TOS, (gpointer) &tos, sizeof(tos))
+	) {
+		const gchar *tosname = "default";
+
+		switch (tos) {
+			case 0: break;
+			case IPTOS_LOWDELAY: tosname = "low delay"; break;
+			case IPTOS_THROUGHPUT: tosname = "throughput"; break;
+			default:
+				g_assert_not_reached();
+		}
+		g_warning("unable to set IP_TOS to %s (%d) on fd#%d: %s",
+			tosname, tos, s->file_desc, g_strerror(errno));
+	}
+}
+
+/*
+ * socket_tos_normal
+ * 
+ * Set the Type of Service (TOS) field to "normal."
+ */
+static void socket_tos_normal(struct gnutella_socket *s)
+{
+	socket_tos(s, 0);
+}
+
+/*
+ * socket_tos_lowdelay
+ *
+ * Set the Type of Service (TOS) field to "lowdelay." This may cause
+ * your host and/or any routers along the path to put its packets in
+ * a higher-priority queue, and/or to route them along the lowest-
+ * latency path without regard for bandwidth.
+ */
+static void socket_tos_lowdelay(struct gnutella_socket *s)
+{
+	socket_tos(s, IPTOS_LOWDELAY);
+}
+
+/*
+ * socket_tos_throughput
+ *
+ * Set the Type of Service (TOS) field to "throughput." This may cause
+ * your host and/or any routers along the path to put its packets in
+ * a lower-priority queue, and/or to route them along the highest-
+ * bandwidth path without regard for latency.
+ */
+static void socket_tos_throughput(struct gnutella_socket *s)
+{
+	socket_tos(s, IPTOS_THROUGHPUT);
+}
+
+/*
+ * socket_tos_default
+ *
+ * Pick an appropriate default TOS for packets on the socket, based
+ * on the socket's type.
+ */
+void socket_tos_default(struct gnutella_socket *s)
+{
+	switch (s->type) {
+	case SOCK_TYPE_CONTROL:
+	case SOCK_TYPE_DOWNLOAD: /* ACKs w/ low latency => higher transfer rates */
+	case SOCK_TYPE_HTTP:
+		socket_tos_lowdelay(s);
+		break;
+	case SOCK_TYPE_UPLOAD:
+		socket_tos_throughput(s);
+		break;
+	default:
+		socket_tos_normal(s);
+	}
+}
+#else
+void socket_tos_default(struct gnutella_socket *s)
+{
+	/* Empty */
+}
+#endif /* HAVE_IP_TOS */
+
+
+/*
+ * socket_timer
+ *
+ * Called by main timer.
+ * Expires inactive sockets.
+ */
+void socket_timer(time_t now)
+{
 	GSList *l;
-	time_t now = time((time_t *) 0);
+	GSList *to_remove = NULL;
 
-  retry:
-	for (l = sl_incoming; l; l = l->next) {
+	for (l = sl_incoming; l; l = g_slist_next(l)) {
 		struct gnutella_socket *s = (struct gnutella_socket *) l->data;
 		g_assert(s->last_update);
-		/* We reuse the `node_connecting_timeout' parameter, need a new one? */
-		if (now - s->last_update > node_connecting_timeout) {
+		/*
+		 * Last_update can be in the feature due to parq. This is needed
+		 * to avoid dropping the connection
+		 */
+		if (now - s->last_update > (gint32) incoming_connecting_timeout) {
 			if (dbg) {
 				g_warning("connection from %s timed out (%d bytes read)",
 						  ip_to_gchar(s->ip), s->pos);
@@ -86,65 +241,114 @@ void socket_monitor_incoming(void)
 					dump_hex(stderr, "Connection Header",
 						s->buffer, MIN(s->pos, 80));
 			}
-			socket_destroy(s);
-			goto retry;			/* Don't know the internals of lists, retry */
+			to_remove = g_slist_prepend(to_remove, s);
 		}
 	}
+
+	for (l = to_remove; l; l = g_slist_next(l)) {
+		struct gnutella_socket *s = (struct gnutella_socket *) l->data;
+		socket_destroy(s, "Connection timeout");
+	}
+
+	g_slist_free(to_remove);
 }
 
 void socket_shutdown(void)
 {
 	while (sl_incoming)
-		socket_destroy((struct gnutella_socket *) sl_incoming->data);
+		socket_destroy((struct gnutella_socket *) sl_incoming->data, NULL);
 }
 
 /* ----------------------------------------- */
 
-/* Destroy a socket, and free its resource if needed */
-
-void socket_destroy(struct gnutella_socket *s)
+/*
+ * socket_destroy
+ *
+ * Destroy a socket.
+ *
+ * If there is an attached resource, call the resource's termination routine
+ * with the supplied reason.
+ */
+static void socket_destroy(struct gnutella_socket *s, gchar *reason)
 {
 	g_assert(s);
 
-	if (s->type == GTA_TYPE_CONTROL && s->resource.node) {
-		node_remove(s->resource.node, NULL);
+	/*
+	 * If there is an attached resource, its removal routine is responsible
+	 * for calling back socket_free().
+	 */
+
+	if (s->type == SOCK_TYPE_CONTROL && s->resource.node) {
+		node_remove(s->resource.node, reason);
 		return;
-	} else if (s->type == GTA_TYPE_DOWNLOAD && s->resource.download) {
-		download_stop(s->resource.download, GTA_DL_ERROR, NULL);
+	} else if (s->type == SOCK_TYPE_DOWNLOAD && s->resource.download) {
+		download_stop(s->resource.download, GTA_DL_ERROR, reason);
 		return;
 
-	} else if (s->type == GTA_TYPE_UPLOAD && s->resource.upload) {
-		upload_remove(s->resource.upload, NULL);
+	} else if (s->type == SOCK_TYPE_UPLOAD && s->resource.upload) {
+		upload_remove(s->resource.upload, reason);
+		return;
+	} else if (s->type == SOCK_TYPE_HTTP && s->resource.handle) {
+		http_async_error(s->resource.handle, HTTP_ASYNC_IO_ERROR);
 		return;
 	}
 
+	/*
+	 * No attached resource, we can simply free this socket then.
+	 */
+
 	socket_free(s);
 }
 
+/*
+ * socket_free
+ *
+ * Dispose of socket, closing connection, removing input callback, and
+ * reclaiming attached getline buffer.
+ */
 void socket_free(struct gnutella_socket *s)
 {
 	g_assert(s);
+
 	if (s->last_update) {
 		g_assert(sl_incoming);
 		sl_incoming = g_slist_remove(sl_incoming, s);
+		s->last_update = 0;
 	}
-	if (s->gdk_tag)
-		gdk_input_remove(s->gdk_tag);
-	if (s->getline)
+	if (s->gdk_tag) {
+		g_source_remove(s->gdk_tag);
+		s->gdk_tag = 0;
+	}
+	if (s->adns & SOCK_ADNS_PENDING) {
+		s->type = SOCK_TYPE_DESTROYING;
+		return;
+	}
+	if (s->getline) {
 		getline_free(s->getline);
+		s->getline = NULL;
+	}
 	if (s->file_desc != -1) {
 		if (s->corked)
 			sock_cork(s, FALSE);
 		close(s->file_desc);
+		s->file_desc = -1;
 	}
-	g_free(s);
+	wfree(s, sizeof(*s));
 }
 
 /* ----------------------------------------- */
 
-/* Read bytes on an unknown incoming socket */
+/* 
+ * socket_read:
+ *
+ * Used for incoming connections, for outgoing too??
+ * Read bytes on an unknown incoming socket. When the first line
+ * has been read it's decided on what type cof connection this is.
+ * If the first line is not complete on the first call, this function
+ * will be called as often as necessary to fetch a full line.
+ */
 
-static void socket_read(gpointer data, gint source, GdkInputCondition cond)
+static void socket_read(gpointer data, gint source, inputevt_cond_t cond)
 {
 	gint r;
 	struct gnutella_socket *s = (struct gnutella_socket *) data;
@@ -152,21 +356,21 @@ static void socket_read(gpointer data, gint source, GdkInputCondition cond)
 	guint parsed;
 	guchar *first;
 
-	//s->type = 0;
+	/* s->type = 0; */
 
-	if (cond & GDK_INPUT_EXCEPTION) {
-		socket_destroy(s);
+	if (cond & INPUT_EVENT_EXCEPTION) {
+		socket_destroy(s, "Input exception");
 		return;
 	}
 
 	g_assert(s->pos == 0);		/* We read a line, then leave this callback */
-
+	
 	count = sizeof(s->buffer) - s->pos - 1;		/* -1 to allow trailing NUL */
 	if (count <= 0) {
 		g_warning("socket_read(): incoming buffer full, disconnecting from %s",
 			 ip_to_gchar(s->ip));
 		dump_hex(stderr, "Leading Data", s->buffer, MIN(s->pos, 256));
-		socket_destroy(s);
+		socket_destroy(s, "Incoming buffer full");
 		return;
 	}
 
@@ -182,12 +386,12 @@ static void socket_read(gpointer data, gint source, GdkInputCondition cond)
 	r = bws_read(bws.in, s->file_desc, s->buffer + s->pos, count);
 
 	if (r == 0) {
-		socket_destroy(s);
+		socket_destroy(s, "Got EOF");
 		return;
 	} else if (r < 0 && errno == EAGAIN)
 		return;
 	else if (r < 0) {
-		socket_destroy(s);
+		socket_destroy(s, "Read error");
 		return;
 	}
 
@@ -208,8 +412,8 @@ static void socket_read(gpointer data, gint source, GdkInputCondition cond)
 			0 == strncmp(s->buffer, "GET ", 4) ||
 			0 == strncmp(s->buffer, "HEAD ", 5)
 		)
-			http_send_status(s, 414, NULL, 0, "Requested URL Too Large");
-		socket_destroy(s);
+			http_send_status(s, 414, FALSE, NULL, 0, "Requested URL Too Large");
+		socket_destroy(s, "Requested URL too large");
 		return;
 	case READ_DONE:
 		if (s->pos != parsed)
@@ -229,25 +433,31 @@ static void socket_read(gpointer data, gint source, GdkInputCondition cond)
 	 * Whatever happens now, we're not going to use the existing read
 	 * callback, and we'll no longer monitor the socket via the `sl_incoming'
 	 * list: if it's a node connection, we'll monitor the node, if it's
-	 * an upload, we'll monitor the uplaod.
+	 * an upload, we'll monitor the upload.
 	 */
 
-	gdk_input_remove(s->gdk_tag);
+	g_source_remove(s->gdk_tag);
 	s->gdk_tag = 0;
 	sl_incoming = g_slist_remove(sl_incoming, s);
 	s->last_update = 0;
 
-	/*
-	 * Always authorize replies for our PUSH requests!
-	 */
-
 	first = getline_str(s->getline);
 
+	/*
+	 * Always authorize replies for our PUSH requests.
+	 * Likewise for PARQ download resuming.
+	 */
+
 	if (0 == strncmp(first, "GIV ", 4)) {
 		download_push_ack(s);
 		return;
 	}
 
+	if (0 == strncmp(first, "QUEUE ", 6)) {
+		parq_download_queue_ack(s);
+		return;
+	}
+
 	/*
 	 * Check for banning.
 	 */
@@ -267,12 +477,12 @@ static void socket_read(gpointer data, gint source, GdkInputCondition cond)
 			gchar msg[80];
 			http_extra_desc_t hev;
 
-			g_snprintf(msg, sizeof(msg)-1, "Retry-After: %d\r\n", delay);
+			gm_snprintf(msg, sizeof(msg)-1, "Retry-After: %d\r\n", delay);
 
 			hev.he_type = HTTP_EXTRA_LINE;
 			hev.he_msg = msg;
 
-			http_send_status(s, 550, &hev, 1, "Banned for %s",
+			http_send_status(s, 550, FALSE, &hev, 1, "Banned for %s",
 				short_time(delay));
 		}
 		goto cleanup;
@@ -281,119 +491,150 @@ static void socket_read(gpointer data, gint source, GdkInputCondition cond)
 	}
 
 	/*
-	 * Dispatch request.
+	 * Deny connections from hostile IP addresses.
+	 *
+	 * We do this after banning checks so that if they hammer us, they
+	 * get banned silently.
 	 */
 
+	if (hostiles_check(s->ip)) {
+		gchar *msg = "Hostile IP address banned";
+
+		g_warning("denying connection from hostile %s: \"%s\"",
+			ip_to_gchar(s->ip), first);
+		if (0 == strncmp(first, gnutella_hello, gnutella_hello_length))
+			send_node_error(s, 550, msg);
+		else
+			http_send_status(s, 550, FALSE, NULL, 0, msg);
+		goto cleanup;
+	}
+
+	/*
+	 * Dispatch request. Here we decide what kind of connection this is.
+	 */
 	if (0 == strncmp(first, gnutella_hello, gnutella_hello_length))
-		node_add(s, s->ip, s->port);	/* Incoming control connection */
+		node_add_socket(s, s->ip, s->port);	/* Incoming control connection */
 	else if (0 == strncmp(first, "GET ", 4))
 		upload_add(s);
 	else if (0 == strncmp(first, "HEAD ", 5))
 		upload_add(s);
-	else {
-		gint len = getline_length(s->getline);
-		if (dbg) {
-			g_warning("socket_read(): Got an unknown incoming connection, "
-				"dropping it.");
-			dump_hex(stderr, "First Line", first, MIN(len, 160));
-		}
-		goto cleanup;
-	}
+#ifdef USE_REMOTE_SHELL
+	else if (0 == strncmp(first, "HELO ", 5))
+        shell_add(s);
+#endif
+    else
+		goto unknown;
+
+	/* Socket might be free'ed now */
 
 	return;
 
+unknown:
+	if (dbg) {
+		gint len = getline_length(s->getline);
+		g_warning("socket_read(): got unknown incoming connection from %s, "
+			"dropping!", ip_to_gchar(s->ip));
+		if (len > 0)
+			dump_hex(stderr, "First Line", first, MIN(len, 160));
+	}
+	if (strstr(first, "HTTP"))
+		http_send_status(s, 501, FALSE, NULL, 0, "Method Not Implemented");
+	/* FALL THROUGH */
+
 cleanup:
-	socket_destroy(s);
+	socket_destroy(s, NULL);
 }
 
 /*
- * Sockets connection
+ * socket_connected:
+ *
+ * Used for outgoing connections!
+ * Called when a socket is connected. Checks type of connection and hands
+ * control over the connetion over to more specialized handlers. If no
+ * handler was found the connection is terminated.
+ * This is the place to hook up handlers for new communication types.
+ * So far there are CONTROL, UPLOAD, DOWNLOAD and HTTP handlers.
  */
-
-void socket_connected(gpointer data, gint source, GdkInputCondition cond)
+static void socket_connected(gpointer data, gint source, inputevt_cond_t cond)
 {
 	/* We are connected to somebody */
 
 	struct gnutella_socket *s = (struct gnutella_socket *) data;
 
-	if (cond & GDK_INPUT_EXCEPTION) {	/* Error while connecting */
-		if (s->type == GTA_TYPE_CONTROL && s->resource.node)
-			node_remove(s->resource.node, "Connection failed");
-		else if (s->type == GTA_TYPE_DOWNLOAD && s->resource.download)
+	if (cond & INPUT_EVENT_EXCEPTION) {	/* Error while connecting */
+		if (s->type == SOCK_TYPE_DOWNLOAD && s->resource.download)
 			download_fallback_to_push(s->resource.download, FALSE, FALSE);
-		else if (s->type == GTA_TYPE_UPLOAD && s->resource.upload)
-			upload_remove(s->resource.upload, "Connection failed");
 		else
-			socket_destroy(s);
+			socket_destroy(s, "Connection failed");
 		return;
 	}
 
-	if (cond & GDK_INPUT_READ) {
+	if (cond & INPUT_EVENT_READ) {
 		if (
-			proxy_connections
-			&& s->direction == GTA_CONNECTION_PROXY_OUTGOING
+			proxy_protocol != PROXY_NONE
+			&& s->direction == SOCK_CONN_PROXY_OUTGOING
 		) {
-			gdk_input_remove(s->gdk_tag);
+			g_source_remove(s->gdk_tag);
 			s->gdk_tag = 0;
 
-			if (proxy_protocol == 4) {
+			if (proxy_protocol == PROXY_SOCKSV4) {
 				if (recv_socks(s) != 0) {
-					socket_destroy(s);
+					socket_destroy(s, "Error receiving from SOCKS 4 proxy");
 					return;
 				}
 
-				s->direction = GTA_CONNECTION_OUTGOING;
+				s->direction = SOCK_CONN_OUTGOING;
 
 				s->gdk_tag =
-					gdk_input_add(s->file_desc,
-								  GDK_INPUT_READ | GDK_INPUT_WRITE |
-								  GDK_INPUT_EXCEPTION, socket_connected,
+					inputevt_add(s->file_desc,
+								  INPUT_EVENT_READ | INPUT_EVENT_WRITE |
+								  INPUT_EVENT_EXCEPTION, socket_connected,
 								  (gpointer) s);
 				return;
-			} else if (proxy_protocol == 5) {
+			} else if (proxy_protocol == PROXY_SOCKSV5) {
 				if (connect_socksv5(s) != 0) {
-					socket_destroy(s);
+					socket_destroy(s, "Error conneting to SOCKS 5 proxy");
 					return;
 				}
 
 				if (s->pos > 5) {
-					s->direction = GTA_CONNECTION_OUTGOING;
+					s->direction = SOCK_CONN_OUTGOING;
 
 					s->gdk_tag =
-						gdk_input_add(s->file_desc,
-									  GDK_INPUT_READ | GDK_INPUT_WRITE |
-									  GDK_INPUT_EXCEPTION,
+						inputevt_add(s->file_desc,
+									  INPUT_EVENT_READ | INPUT_EVENT_WRITE |
+									  INPUT_EVENT_EXCEPTION,
 									  socket_connected, (gpointer) s);
 
 					return;
 				} else
 					s->gdk_tag =
-						gdk_input_add(s->file_desc,
-									  GDK_INPUT_WRITE |
-									  GDK_INPUT_EXCEPTION,
+						inputevt_add(s->file_desc,
+									  INPUT_EVENT_WRITE |
+									  INPUT_EVENT_EXCEPTION,
 									  socket_connected, (gpointer) s);
 
 				return;
 
-			} else if (proxy_protocol == 1) {
+			} else if (proxy_protocol == PROXY_HTTP) {
 				if (connect_http(s) != 0) {
-					socket_destroy(s);
+					socket_destroy(s, "Unable to connect to HTTP proxy");
 					return;
 				}
 
 				if (s->pos > 2) {
-					s->direction = GTA_CONNECTION_OUTGOING;
+					s->direction = SOCK_CONN_OUTGOING;
 
 					s->gdk_tag =
-						gdk_input_add(s->file_desc,
-									  GDK_INPUT_READ | GDK_INPUT_WRITE |
-									  GDK_INPUT_EXCEPTION,
+						inputevt_add(s->file_desc,
+									  INPUT_EVENT_READ | INPUT_EVENT_WRITE |
+									  INPUT_EVENT_EXCEPTION,
 									  socket_connected, (gpointer) s);
 					return;
 				} else {
 					s->gdk_tag =
-						gdk_input_add(s->file_desc,
-									  GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
+						inputevt_add(s->file_desc,
+									  INPUT_EVENT_READ | INPUT_EVENT_EXCEPTION,
 									  socket_connected, (gpointer) s);
 					return;
 				}
@@ -401,11 +642,11 @@ void socket_connected(gpointer data, gint source, GdkInputCondition cond)
 		}
 	}
 
-	if (cond & GDK_INPUT_WRITE) {
+	if (cond & INPUT_EVENT_WRITE) {
 		/* We are just connected to our partner */
 		gint res, option, size = sizeof(gint);
 
-		gdk_input_remove(s->gdk_tag);
+		g_source_remove(s->gdk_tag);
 		s->gdk_tag = 0;
 
 		/* Check wether the socket is really connected */
@@ -414,43 +655,43 @@ void socket_connected(gpointer data, gint source, GdkInputCondition cond)
 					   (void *) &option, &size);
 
 		if (res == -1 || option) {
-			if (s->type == GTA_TYPE_CONTROL && s->resource.node)
-				node_remove(s->resource.node, "Connection failed");
-			else if (s->type == GTA_TYPE_DOWNLOAD && s->resource.download)
+			if (s->type == SOCK_TYPE_DOWNLOAD && s->resource.download)
 				download_fallback_to_push(s->resource.download, FALSE, FALSE);
 			else
-				socket_destroy(s);
+				socket_destroy(s, "Connection failed");
 			return;
 		}
 
-		if (proxy_connections
-			&& s->direction == GTA_CONNECTION_PROXY_OUTGOING) {
-			if (proxy_protocol == 4) {
+		if (proxy_protocol != PROXY_NONE
+			&& s->direction == SOCK_CONN_PROXY_OUTGOING) {
+			if (proxy_protocol == PROXY_SOCKSV4) {
 
 				if (send_socks(s) != 0) {
-					socket_destroy(s);
+					socket_destroy(s, "Error sending to SOCKS 4 proxy");
 					return;
 				}
-			} else if (proxy_protocol == 5) {
+			} else if (proxy_protocol == PROXY_SOCKSV5) {
 				if (connect_socksv5(s) != 0) {
-					socket_destroy(s);
+					socket_destroy(s, "Error connecting to SOCKS 5 proxy");
 					return;
 				}
 
-			} else if (proxy_protocol == 1) {
+			} else if (proxy_protocol == PROXY_HTTP) {
 				if (connect_http(s) != 0) {
-					socket_destroy(s);
+					socket_destroy(s, "Error connecting to HTTP proxy");
 					return;
 				}
 			}
 
 			s->gdk_tag =
-				gdk_input_add(s->file_desc,
-							  GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
+				inputevt_add(s->file_desc,
+							  INPUT_EVENT_READ | INPUT_EVENT_EXCEPTION,
 							  socket_connected, (gpointer) s);
 			return;
 		}
 
+		inet_connection_succeeded(s->ip);
+
 		s->pos = 0;
 		memset(s->buffer, 0, sizeof(s->buffer));
 
@@ -465,7 +706,7 @@ void socket_connected(gpointer data, gint source, GdkInputCondition cond)
 		guess_local_ip(s->file_desc);
 
 		switch (s->type) {
-		case GTA_TYPE_CONTROL:
+		case SOCK_TYPE_CONTROL:
 			{
 				struct gnutella_node *n = s->resource.node;
 
@@ -474,7 +715,7 @@ void socket_connected(gpointer data, gint source, GdkInputCondition cond)
 			}
 			break;
 
-		case GTA_TYPE_DOWNLOAD:
+		case SOCK_TYPE_DOWNLOAD:
 			{
 				struct download *d = s->resource.download;
 
@@ -483,21 +724,36 @@ void socket_connected(gpointer data, gint source, GdkInputCondition cond)
 			}
 			break;
 
-		case GTA_TYPE_UPLOAD:
+		case SOCK_TYPE_UPLOAD:
 			{
 				struct upload *u = s->resource.upload;
 
 				g_assert(u->socket == s);
-				upload_push_conf(u);
+				upload_connect_conf(u);
 			}
 			break;
 
+		case SOCK_TYPE_HTTP:
+			http_async_connected(s->resource.handle);
+			break;
+
+		case SOCK_TYPE_CONNBACK:
+			node_connected_back(s);
+			break;
+
+#ifdef USE_REMOTE_SHELL
+        case SOCK_TYPE_SHELL:
+            g_assert_not_reached(); /* FIXME: add code here? */
+            break;
+#endif
+
 		default:
 			g_warning("socket_connected(): Unknown socket type %d !", s->type);
-			socket_destroy(s);		/* ? */
+			socket_destroy(s, NULL);		/* ? */
 			break;
 		}
 	}
+
 }
 
 static void guess_local_ip(int sd)
@@ -508,7 +764,7 @@ static void guess_local_ip(int sd)
 
 	if (-1 != getsockname(sd, (struct sockaddr *) &addr, &len)) {
 		gboolean can_supersede;
-		ip = g_ntohl(addr.sin_addr.s_addr);
+		ip = ntohl(addr.sin_addr.s_addr);
 
 		/*
 		 * If local IP was unknown, keep what we got here, even if it's a
@@ -521,12 +777,10 @@ static void guess_local_ip(int sd)
 
 		if (!ip_computed) {
 			if (!local_ip || can_supersede)
-				local_ip = ip;
+				gnet_prop_set_guint32_val(PROP_LOCAL_IP, ip);
 			ip_computed = TRUE;
 		} else if (can_supersede)
-			local_ip = ip;
-
-		gui_update_config_port(FALSE);
+			gnet_prop_set_guint32_val(PROP_LOCAL_IP, ip);
 	}
 }
 
@@ -543,20 +797,23 @@ static int socket_local_port(struct gnutella_socket *s)
 	if (getsockname(s->file_desc, (struct sockaddr *) &addr, &len) == -1)
 		return -1;
 
-	return g_ntohs(addr.sin_port);
+	return ntohs(addr.sin_port);
 }
 
+/*
+ * socket_accept
+ *
+ * Someone is connecting to us.
+ */
 static void socket_accept(gpointer data, gint source,
-						  GdkInputCondition cond)
+						  inputevt_cond_t cond)
 {
-	/* Someone is connecting to us */
-
 	struct sockaddr_in addr;
 	gint sd, len = sizeof(struct sockaddr_in);
 	struct gnutella_socket *s = (struct gnutella_socket *) data;
 	struct gnutella_socket *t = NULL;
 
-	if (cond & GDK_INPUT_EXCEPTION) {
+	if (cond & INPUT_EVENT_EXCEPTION) {
 		g_warning("Input Exception for listening socket #%d !!!!\n",
 				  s->file_desc);
 		gtk_gnutella_exit(2);
@@ -564,12 +821,12 @@ static void socket_accept(gpointer data, gint source,
 	}
 
 	switch (s->type) {
-	case GTA_TYPE_CONTROL:
+	case SOCK_TYPE_CONTROL:
 		break;
 	default:
-		g_warning("socket_accept(): Unknown listning socket type %d !\n",
+		g_warning("socket_accept(): Unknown listening socket type %d !\n",
 				  s->type);
-		socket_destroy(s);
+		socket_destroy(s, NULL);
 		return;
 	}
 
@@ -587,20 +844,20 @@ static void socket_accept(gpointer data, gint source,
 
 	fcntl(sd, F_SETFL, O_NONBLOCK);	/* Set the file descriptor non blocking */
 
-	t = (struct gnutella_socket *) g_malloc0(sizeof(struct gnutella_socket));
+	t = (struct gnutella_socket *) walloc0(sizeof(struct gnutella_socket));
 
 	t->file_desc = sd;
-	t->ip = g_ntohl(addr.sin_addr.s_addr);
-	t->port = g_ntohs(addr.sin_port);
-	t->direction = GTA_CONNECTION_INCOMING;
+	t->ip = ntohl(addr.sin_addr.s_addr);
+	t->port = ntohs(addr.sin_port);
+	t->direction = SOCK_CONN_INCOMING;
 	t->type = s->type;
 	t->local_port = s->local_port;
-	t->getline = getline_make();
+	t->getline = getline_make(MAX_LINE_SIZE);
 
 	switch (s->type) {
-	case GTA_TYPE_CONTROL:
+	case SOCK_TYPE_CONTROL:
 		t->gdk_tag =
-			gdk_input_add(sd, GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
+			inputevt_add(sd, INPUT_EVENT_READ | INPUT_EVENT_EXCEPTION,
 						  socket_read, t);
 		/*
 		 * Whilst the socket is attached to that callback, it has been
@@ -628,40 +885,24 @@ static void socket_accept(gpointer data, gint source,
 		break;
 	}
 
-	/*
-	 * Since someone connected to us, we're not completely firewalled.
-	 * The Gnutella port has at least been opened.
-	 *		--RAM, 20/12/2001
-	 */
-
-	if (is_firewalled) {
-		/*
-		 * Make sure we're not connecting locally.
-		 */
-
-		if (
-			t->ip != listen_ip() &&					/* Not ourselves */
-			(t->ip & 0xff000000) != 0x7f000000 &&	/* Not loopback 127.xxx */
-			t->ip != host_to_ip(host_name())		/* Not ourselves */
-		) {
-			is_firewalled = FALSE;
-			gui_update_is_firewalled();
-			if (dbg) printf("Got evidence that we're not fully firewalled\n");
-		}
-	}
+	inet_got_incoming(t->ip);	/* Signal we got an incoming connection */
 }
 
 /*
  * Sockets creation
  */
 
-struct gnutella_socket *socket_connect(guint32 ip, guint16 port, gint type)
-{
-	/* Create a socket and try to connect it to ip:port */
-
-	gint sd, option = 1, res = 0;
-	struct sockaddr_in addr, lcladdr;
+/*
+ * socket_connect_prepare
+ *
+ * Called to prepare the creation of the socket connection.
+ * Returns NULL in case of failure.
+ */
+static struct gnutella_socket *socket_connect_prepare(
+	guint16 port, enum socket_type type)
+{	
 	struct gnutella_socket *s;
+	gint sd, option = 1;
 
 	sd = socket(AF_INET, SOCK_STREAM, 0);
 
@@ -670,84 +911,199 @@ struct gnutella_socket *socket_connect(guint32 ip, guint16 port, gint type)
 		return NULL;
 	}
 
-	s = (struct gnutella_socket *) g_malloc0(sizeof(struct gnutella_socket));
+	s = (struct gnutella_socket *) walloc0(sizeof(struct gnutella_socket));
 
 	s->type = type;
-	s->direction = GTA_CONNECTION_OUTGOING;
+	s->direction = SOCK_CONN_OUTGOING;
 	s->file_desc = sd;
-	s->ip = ip;
 	s->port = port;
 
-	setsockopt(sd, SOL_SOCKET, SO_KEEPALIVE, (void *) &option,
+	setsockopt(s->file_desc, SOL_SOCKET, SO_KEEPALIVE, (void *) &option,
 			   sizeof(option));
-	setsockopt(sd, SOL_SOCKET, SO_REUSEADDR, (void *) &option,
+	setsockopt(s->file_desc, SOL_SOCKET, SO_REUSEADDR, (void *) &option,
 			   sizeof(option));
 
-	fcntl(sd, F_SETFL, O_NONBLOCK);	/* Set the file descriptor non blocking */
+	/* Set the file descriptor non blocking */
+	fcntl(s->file_desc, F_SETFL, O_NONBLOCK);
 
+	socket_tos_default(s);
+	return s;
+}
+
+/*
+ * socket_connect_finalize
+ *
+ * Called to finalize the creation of the socket connection, which is done
+ * in two steps since DNS resolving is asynchronous.
+ */
+static struct gnutella_socket *socket_connect_finalize(
+	struct gnutella_socket *s, guint32 ip_addr)
+{
+	gint res = 0;
+	struct sockaddr_in addr;
+
+	g_assert(NULL != s);
+
+	s->ip = ip_addr;
+	memset(&addr, 0, sizeof(addr));
 	addr.sin_family = AF_INET;
-	addr.sin_addr.s_addr = g_htonl(ip);
-	addr.sin_port = g_htons(port);
+	addr.sin_addr.s_addr = htonl(s->ip);
+	addr.sin_port = htons(s->port);
+
+	inet_connection_attempted(s->ip);
 
 	/*
 	 * Now we check if we're forcing a local IP, and make it happen if so.
 	 *   --JSL
 	 */
 	if (force_local_ip) {
+		struct sockaddr_in lcladdr;
+
+		memset(&lcladdr, 0, sizeof(lcladdr));
 		lcladdr.sin_family = AF_INET;
-		lcladdr.sin_addr.s_addr = g_htonl(forced_local_ip);
-		lcladdr.sin_port = g_htons(0);
+		lcladdr.sin_addr.s_addr = htonl(forced_local_ip);
+		lcladdr.sin_port = htons(0);
 
 		/*
 		 * Note: we ignore failures: it will be automatic at connect()
 		 * It's useful only for people forcing the IP without being
 		 * behind a masquerading firewall --RAM.
 		 */
-		(void) bind(sd, (struct sockaddr *) &lcladdr,
+		(void) bind(s->file_desc, (struct sockaddr *) &lcladdr,
 			sizeof(struct sockaddr_in));
 	}
 
-	if (proxy_connections) {
+	if (proxy_protocol != PROXY_NONE) {
+		struct sockaddr_in lcladdr;
+
+		memset(&lcladdr, 0, sizeof(lcladdr));
 		lcladdr.sin_family = AF_INET;
 		lcladdr.sin_port = INADDR_ANY;
 
-		(void) bind(sd, (struct sockaddr *) &lcladdr,
+		(void) bind(s->file_desc, (struct sockaddr *) &lcladdr,
 			sizeof(struct sockaddr_in));
 
-		res = proxy_connect(sd, (struct sockaddr *) &addr,
+		s->direction = SOCK_CONN_PROXY_OUTGOING;
+		res = proxy_connect(s->file_desc, (struct sockaddr *) &addr,
 			sizeof(struct sockaddr_in));
-
-		s->direction = GTA_CONNECTION_PROXY_OUTGOING;
 	} else
-		res = connect(sd, (struct sockaddr *) &addr,
+		res = connect(s->file_desc, (struct sockaddr *) &addr,
 			sizeof(struct sockaddr_in));
 
 	if (res == -1 && errno != EINPROGRESS) {
 		g_warning("Unable to connect to %s: (%s)",
 			ip_port_to_gchar(s->ip, s->port), g_strerror(errno));
-		socket_destroy(s);
+	
+		if (s->adns & SOCK_ADNS_PENDING)
+			s->adns_msg = "Connection failed";
+		else
+			socket_destroy(s, "Connection failed");
 		return NULL;
 	}
 
 	s->local_port = socket_local_port(s);
 
-	fcntl(sd, F_SETFL, O_NONBLOCK);	/* Set the file descriptor non blocking */
+	/* Set the file descriptor non blocking */
+	fcntl(s->file_desc, F_SETFL, O_NONBLOCK);
 
-	g_assert(s->gdk_tag == 0);
+	g_assert(0 == s->gdk_tag);
 
-	if (proxy_connections)
-		s->gdk_tag = gdk_input_add(sd,
-			GDK_INPUT_READ | GDK_INPUT_WRITE | GDK_INPUT_EXCEPTION,
+	if (proxy_protocol != PROXY_NONE)
+		s->gdk_tag = inputevt_add(s->file_desc,
+			INPUT_EVENT_READ | INPUT_EVENT_WRITE | INPUT_EVENT_EXCEPTION,
 			socket_connected, s);
 	else
-		s->gdk_tag = gdk_input_add(sd,
-			GDK_INPUT_WRITE | GDK_INPUT_EXCEPTION,
+		s->gdk_tag = inputevt_add(s->file_desc,
+			INPUT_EVENT_WRITE | INPUT_EVENT_EXCEPTION,
 			socket_connected, s);
 
 	return s;
 }
 
-struct gnutella_socket *socket_listen(guint32 ip, guint16 port, gint type)
+/*
+ * socket_connect
+ *
+ * Creates a connected socket with an attached resource of `type'.
+ *
+ * Connection happens in the background, the connection callback being
+ * determined by the resource type.
+ */
+struct gnutella_socket *socket_connect(
+	guint32 ip_addr, guint16 port, enum socket_type type)
+{
+	/* Create a socket and try to connect it to ip:port */
+
+	struct gnutella_socket *s;
+	
+	s = socket_connect_prepare(port, type);
+	g_return_val_if_fail(NULL != s, NULL);
+	return socket_connect_finalize(s, ip_addr);
+}
+
+/*
+ * socket_connect_by_name_helper
+ *
+ * Called when we got a reply from the ADNS process.
+ */
+static void socket_connect_by_name_helper(guint32 ip_addr, gpointer user_data)
+{
+	struct gnutella_socket *s = user_data;
+
+	g_assert(NULL != s);
+
+	if (0 == ip_addr || s->type == SOCK_TYPE_DESTROYING) {
+		s->adns &= ~SOCK_ADNS_PENDING;
+		s->adns |= SOCK_ADNS_FAILED;
+		s->adns_msg = "Could not resolve address";
+		return; 
+	}
+	if (NULL == socket_connect_finalize(s, ip_addr)) {
+		s->adns &= ~SOCK_ADNS_PENDING;
+		s->adns |= SOCK_ADNS_FAILED;
+		return;
+	}
+	s->adns &= ~SOCK_ADNS_PENDING;
+}
+
+/*
+ * socket_connect_by_name
+ *
+ * Like socket_connect() but the remote address is not known and must be
+ * resolved through async DNS calls.
+ */
+struct gnutella_socket *socket_connect_by_name(
+	const gchar *host, guint16 port, enum socket_type type)
+{
+	/* Create a socket and try to connect it to host:port */
+
+	struct gnutella_socket *s;
+
+	g_assert(NULL != host);
+	s = socket_connect_prepare(port, type);
+	g_return_val_if_fail(NULL != s, NULL);
+	s->adns |= SOCK_ADNS_PENDING;
+	if (
+		!adns_resolve(host, &socket_connect_by_name_helper, s)
+		&& (s->adns & SOCK_ADNS_FAILED)
+	) {
+		/*	socket_connect_by_name_helper() was already invoked! */
+		if (dbg > 0)
+			g_warning("socket_connect_by_name: "
+				"adns_resolve() failed in synchronous mode");
+		socket_destroy(s, s->adns_msg);
+		return NULL;
+	}
+
+	return s;
+}
+
+/*
+ * socket_listen
+ *
+ * Creates a non-blocking listening socket with an attached resource of `type'.
+ */
+struct gnutella_socket *socket_listen(
+	guint32 ip, guint16 port, enum socket_type type)
 {
 	/* Create a socket, then bind() and listen() it */
 
@@ -763,10 +1119,10 @@ struct gnutella_socket *socket_listen(guint32 ip, guint16 port, gint type)
 		return NULL;
 	}
 
-	s = (struct gnutella_socket *) g_malloc0(sizeof(struct gnutella_socket));
+	s = (struct gnutella_socket *) walloc0(sizeof(struct gnutella_socket));
 
 	s->type = type;
-	s->direction = GTA_CONNECTION_LISTENING;
+	s->direction = SOCK_CONN_LISTENING;
 	s->file_desc = sd;
 	s->pos = 0;
 
@@ -778,15 +1134,15 @@ struct gnutella_socket *socket_listen(guint32 ip, guint16 port, gint type)
 	fcntl(sd, F_SETFL, O_NONBLOCK);	/* Set the file descriptor non blocking */
 
 	addr.sin_family = AF_INET;
-	addr.sin_addr.s_addr = g_htonl((ip) ? ip : INADDR_ANY);
-	addr.sin_port = g_htons(port);
+	addr.sin_addr.s_addr = htonl((ip) ? ip : INADDR_ANY);
+	addr.sin_port = htons(port);
 
 	/* bind() the socket */
 
 	if (bind(sd, (struct sockaddr *) &addr, l) == -1) {
 		g_warning("Unable to bind() the socket on port %u (%s)",
 				  port, g_strerror(errno));
-		socket_destroy(s);
+		socket_destroy(s, "Unable to bind socket");
 		return NULL;
 	}
 
@@ -794,7 +1150,7 @@ struct gnutella_socket *socket_listen(guint32 ip, guint16 port, gint type)
 
 	if (listen(sd, 5) == -1) {
 		g_warning("Unable to listen() the socket (%s)", g_strerror(errno));
-		socket_destroy(s);
+		socket_destroy(s, "Unable to listen on socket");
 		return NULL;
 	}
 
@@ -806,16 +1162,16 @@ struct gnutella_socket *socket_listen(guint32 ip, guint16 port, gint type)
 		if (getsockname(sd, (struct sockaddr *) &addr, &option) == -1) {
 			g_warning("Unable to get the port of the socket: "
 				"getsockname() failed (%s)", g_strerror(errno));
-			socket_destroy(s);
+			socket_destroy(s, "Can't probe socket for port");
 			return NULL;
 		}
 
-		s->local_port = g_ntohs(addr.sin_port);
+		s->local_port = ntohs(addr.sin_port);
 	} else
 		s->local_port = port;
 
 	s->gdk_tag =
-		gdk_input_add(sd, GDK_INPUT_READ | GDK_INPUT_EXCEPTION,
+		inputevt_add(sd, INPUT_EVENT_READ | INPUT_EVENT_EXCEPTION,
 					  socket_accept, s);
 
 	return s;
@@ -839,7 +1195,7 @@ void sock_cork(struct gnutella_socket *s, gboolean on)
 #ifdef TCP_CORK
 	gint arg = on ? 1 : 0;
 
-	if (-1 == setsockopt(s->file_desc, SOL_TCP, TCP_CORK, &arg, sizeof(arg)))
+	if (-1 == setsockopt(s->file_desc, sol_tcp(), TCP_CORK, &arg, sizeof(arg)))
 		g_warning("unable to %s TCP_CORK on fd#%d: %s",
 			on ? "set" : "clear", s->file_desc, g_strerror(errno));
 	else
@@ -875,7 +1231,7 @@ static void _sock_set(gint fd, gint option, gint size,
 		g_warning("cannot read old %s buffer length on fd #%d: %s",
 			type, fd, g_strerror(errno));
 
-// XXX needs to add metaconfig test
+/* XXX needs to add metaconfig test */
 #if linux
 	old_len >>= 1;		/* Linux returns twice the real amount */
 #endif
@@ -937,9 +1293,13 @@ void sock_nodelay(struct gnutella_socket *s, gboolean on)
 {
 	gint arg = on ? 1 : 0;
 
-	if (-1 == setsockopt(s->file_desc, SOL_TCP, TCP_NODELAY, &arg, sizeof(arg)))
+	if (
+		-1 == setsockopt(s->file_desc, sol_tcp(), TCP_NODELAY,
+				&arg, sizeof(arg))
+	) {
 		g_warning("unable to %s TCP_NODELAY on fd#%d: %s",
 			on ? "set" : "clear", s->file_desc, g_strerror(errno));
+	}
 }
 
 /*
@@ -968,7 +1328,7 @@ static void show_error(char *fmt, ...)
  * It was modified to work with gtk_gnutella and non-blocking sockets. --DW
  */
 
-int proxy_connect(int __fd, const struct sockaddr *__addr, guint __len)
+int proxy_connect(int fd, const struct sockaddr *addr, guint len)
 {
 	struct sockaddr_in *connaddr;
 	void **kludge;
@@ -977,25 +1337,24 @@ int proxy_connect(int __fd, const struct sockaddr *__addr, guint __len)
 	int rc = 0;
 
 
-	if (!(inet_aton(proxy_ip, &server.sin_addr))) {
+	if (!inet_aton(ip_to_gchar(proxy_ip), &server.sin_addr)) {
 		show_error("The SOCKS server (%s) in configuration "
-				   "file is invalid\n", proxy_ip);
+				   "file is invalid\n", ip_to_gchar(proxy_ip));
 	} else {
 		/* Construct the addr for the socks server */
 		server.sin_family = AF_INET;	/* host byte order */
 		server.sin_port = htons(proxy_port);
 		/* zero the rest of the struct */
-		memset(&(server.sin_zero), 0, sizeof(server.sin_zero));
+		memset(&server.sin_zero, 0, sizeof(server.sin_zero));
 	}
 
 
 	/* Ok, so this method sucks, but it's all I can think of */
 
-	kludge = (void *) &__addr;
+	kludge = (void *) &addr;
 	connaddr = (struct sockaddr_in *) *kludge;
 
-	rc = connect(__fd, (struct sockaddr *) &server,
-				 sizeof(struct sockaddr));
+	rc = connect(fd, (struct sockaddr *) &server, sizeof(struct sockaddr));
 
 	return rc;
 
@@ -1008,18 +1367,18 @@ struct socksent {
 } __attribute__((__packed__));
 
 struct sockreq {
-	int8_t version;
-	int8_t command;
-	int16_t dstport;
-	int32_t dstip;
+	gint8 version;
+	gint8 command;
+	gint16 dstport;
+	gint32 dstip;
 	/* A null terminated username goes here */
 } __attribute__((__packed__));
 
 struct sockrep {
-	int8_t version;
-	int8_t result;
-	int16_t ignore1;
-	int32_t ignore2;
+	gint8 version;
+	gint8 result;
+	gint16 ignore1;
+	gint32 ignore2;
 } __attribute__((__packed__));
 
 int send_socks(struct gnutella_socket *s)
@@ -1119,7 +1478,7 @@ int connect_http(struct gnutella_socket *s)
 
 	switch (s->pos) {
 	case 0:
-		g_snprintf(s->buffer, sizeof(s->buffer),
+		gm_snprintf(s->buffer, sizeof(s->buffer),
 			"CONNECT %s HTTP/1.0\r\n\r\n",
 			ip_port_to_gchar(s->ip, s->port));
 		if (
@@ -1139,7 +1498,7 @@ int connect_http(struct gnutella_socket *s)
 				g_strerror(errno));
 			return -1;
 		}
-		s->getline = getline_make();
+		s->getline = getline_make(HEAD_MAX_SIZE);
 		switch (getline_read(s->getline, s->buffer, rc, &parsed)) {
 		case READ_OVERFLOW:
 			show_error("Reading buffer overflow\n");
@@ -1239,7 +1598,7 @@ int connect_socksv5(struct gnutella_socket *s)
 {
 	int rc = 0;
 	int offset = 0;
-	char *verstring = "\x05\x02\x02\x00";
+	const char *verstring = "\x05\x02\x02\x00";
 	char *uname, *upass;
 	struct passwd *nixuser;
 	char *buf;
@@ -1316,11 +1675,11 @@ int connect_socksv5(struct gnutella_socket *s)
 		offset = 0;
 		buf[offset] = '\x01';
 		offset++;
-		buf[offset] = (int8_t) strlen(uname);
+		buf[offset] = (gint8) strlen(uname);
 		offset++;
 		memcpy(&buf[offset], uname, strlen(uname));
 		offset = offset + strlen(uname);
-		buf[offset] = (int8_t) strlen(upass);
+		buf[offset] = (gint8) strlen(upass);
 		offset++;
 		memcpy(&buf[offset], upass, strlen(upass));
 		offset = offset + strlen(upass);
@@ -1394,7 +1753,7 @@ int connect_socksv5(struct gnutella_socket *s)
 		/* See the connection succeeded */
 		if (buf[1] != '\x00') {
 			show_error("SOCKS connect failed: ");
-			switch ((int8_t) buf[1]) {
+			switch ((gint8) buf[1]) {
 			case 1:
 				show_error("General SOCKS server failure\n");
 				return (ECONNABORTED);
