@@ -1,4 +1,6 @@
 /*
+ * $Id: tx_deflate.c,v 1.22 2003/03/18 00:28:31 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  * Network driver -- compressing level.
@@ -34,9 +36,8 @@
 #include "nodes.h"
 #include "tx.h"
 #include "tx_deflate.h"
-#include "misc.h"
-#include "zlib_util.h"
-#include "cq.h"
+
+RCSID("$Id: tx_deflate.c,v 1.22 2003/03/18 00:28:31 cbiere Exp $");
 
 /*
  * The driver manages two fixed-size buffers: one is being filled by the
@@ -51,7 +52,7 @@
  */
 
 #define BUFFER_COUNT	2
-#define BUFFER_SIZE		2048
+#define BUFFER_SIZE		1024
 #define BUFFER_FLUSH	4096	/* Try to flush every 4K */
 #define BUFFER_NAGLE	200		/* 200 ms */
 
@@ -119,8 +120,10 @@ static void deflate_send(txdrv_t *tx)
 	r = tx_write(attr->nd, b->rptr, len);
 
 	if (dbg > 9)
-		printf("deflate_send: (%s) wrote %d bytes (buffer #%d)\n",
-			node_ip(tx->node), r, attr->send_idx);
+		printf("deflate_send: (%s) wrote %d bytes (buffer #%d) [%c%c]\n",
+			node_ip(tx->node), r, attr->send_idx,
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
 
 	if (r < 0)
 		return;
@@ -214,8 +217,10 @@ static void deflate_rotate_and_send(txdrv_t *tx)
 		attr->fill_idx = 0;
 
 	if (dbg > 9)
-		printf("deflate_rotate_and_send: (%s) fill buffer now #%d\n",
-			node_ip(tx->node), attr->fill_idx);
+		printf("deflate_rotate_and_send: (%s) fill buffer now #%d [%c%c]\n",
+			node_ip(tx->node), attr->fill_idx,
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
 
 	deflate_send(tx);
 }
@@ -236,9 +241,12 @@ static void deflate_service(gpointer data)
 	g_assert(attr->send_idx < BUFFER_COUNT);
 
 	if (dbg > 9)
-		printf("deflate_service: (%s) (buffer #%d, %d bytes held)\n",
+		printf("deflate_service: (%s) (buffer #%d, %d bytes held) [%c%c]\n",
 			node_ip(tx->node), attr->send_idx,
-			attr->buf[attr->send_idx].wptr - attr->buf[attr->send_idx].rptr);
+			(gint) (attr->buf[attr->send_idx].wptr - 
+					attr->buf[attr->send_idx].rptr),
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
 
 	/*
 	 * First, attempt to transmit the whole send buffer.
@@ -263,7 +271,7 @@ static void deflate_service(gpointer data)
 	if (b->wptr >= b->end) {
 		if (dbg > 9)
 			printf("deflate_service: (%s) sending fill buffer #%d, %d bytes\n",
-				node_ip(tx->node), attr->fill_idx, b->wptr - b->rptr);
+				node_ip(tx->node), attr->fill_idx, (gint) (b->wptr - b->rptr));
 
 		deflate_rotate_and_send(tx);
 	}
@@ -297,6 +305,12 @@ static void deflate_service(gpointer data)
 			tx->srv_routine(tx->srv_arg);
 		}
 	}
+
+	if (dbg > 9)
+		printf("deflate_service: (%s) leaving [%c%c]\n",
+			node_ip(tx->node),
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
 }
 
 /*
@@ -317,8 +331,10 @@ retry:
 	b = &attr->buf[attr->fill_idx];	/* Buffer we fill */
 
 	if (dbg > 9)
-		printf("deflate_flush: (%s) flushing (buffer #%d)\n",
-			node_ip(tx->node), attr->fill_idx);
+		printf("deflate_flush: (%s) flushing (buffer #%d) [%c%c]\n",
+			node_ip(tx->node), attr->fill_idx,
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
 
 	/*
 	 * Prepare call to deflate().
@@ -395,8 +411,12 @@ static void deflate_nagle_timeout(cqueue_t *cq, gpointer arg)
 	if (attr->send_idx != -1) {		/* Send buffer still incompletely sent */
 
 		if (dbg > 9)
-			printf("deflate_nagle_timeout: (%s) buffer #%d unsent, exiting\n",
-				node_ip(tx->node), attr->send_idx);
+			printf("deflate_nagle_timeout: (%s) buffer #%d unsent,"
+				" exiting [%c%c]\n",
+				node_ip(tx->node), attr->send_idx,
+				(attr->flags & DF_FLOWC) ? 'C' : '-',
+				(attr->flags & DF_FLUSH) ? 'f' : '-');
+
 
 		attr->tm_ev =
 			cq_insert(attr->cq, BUFFER_NAGLE, deflate_nagle_timeout, tx);
@@ -407,8 +427,10 @@ static void deflate_nagle_timeout(cqueue_t *cq, gpointer arg)
 	attr->tm_ev = NULL;
 
 	if (dbg > 9) {
-		printf("deflate_nagle_timeout: (%s) flushing (buffer #%d)\n",
-			node_ip(tx->node), attr->fill_idx);
+		printf("deflate_nagle_timeout: (%s) flushing (buffer #%d) [%c%c]\n",
+			node_ip(tx->node), attr->fill_idx,
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
 		fflush(stdout);
 	}
 
@@ -441,9 +463,11 @@ static gint deflate_add(txdrv_t *tx, gpointer data, gint len)
 
 	if (dbg > 9) {
 		printf("deflate_add: (%s) given %d bytes (buffer #%d, nagle %s, "
-			"unflushed %d)\n",
+			"unflushed %d) [%c%c]\n",
 			node_ip(tx->node), len, attr->fill_idx,
-			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed);
+			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed,
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
 		fflush(stdout);
 	}
 
@@ -461,7 +485,7 @@ static gint deflate_add(txdrv_t *tx, gpointer data, gint len)
 		outz->next_out = b->wptr;
 		outz->avail_out = old_avail = b->end - b->wptr;
 
-		outz->next_in = data + added;
+		outz->next_in = (gchar *) data + added;
 		outz->avail_in = len - added;
 
 		g_assert(outz->avail_out > 0);
@@ -488,7 +512,7 @@ static gint deflate_add(txdrv_t *tx, gpointer data, gint len)
 		 */
 
 		b->wptr = outz->next_out;
-		added = (gpointer) outz->next_in - data;
+		added = (gchar *) outz->next_in - (gchar *) data;
 
 		g_assert(added >= old_added);
 
@@ -572,7 +596,7 @@ static gpointer tx_deflate_init(txdrv_t *tx, gpointer args)
 
 	g_assert(tx);
 
-	outz = g_malloc(sizeof(*outz));
+	outz = walloc(sizeof(*outz));
 
 	outz->zalloc = NULL;
 	outz->zfree = NULL;
@@ -581,13 +605,13 @@ static gpointer tx_deflate_init(txdrv_t *tx, gpointer args)
 	ret = deflateInit(outz, Z_DEFAULT_COMPRESSION);
 
 	if (ret != Z_OK) {
-		g_free(outz);
+		wfree(outz, sizeof(*outz));
 		g_warning("unable to initialize compressor for node %s: %s",
 			node_ip(tx->node), zlib_strerror(ret));
 		return NULL;
 	}
 
-	attr = g_malloc(sizeof(*attr));
+	attr = walloc(sizeof(*attr));
 
 	attr->nd = targs->nd;
 	attr->cq = targs->cq;
@@ -654,12 +678,12 @@ static void tx_deflate_destroy(txdrv_t *tx)
 		g_warning("while freeing compressor for node %s: %s",
 			node_ip(tx->node), zlib_strerror(ret));
 
-	g_free(attr->outz);
+	wfree(attr->outz, sizeof(*attr->outz));
 
 	if (attr->tm_ev)
 		cq_cancel(attr->cq, attr->tm_ev);
 
-	g_free(tx->opaque);
+	wfree(attr, sizeof(*attr));
 }
 
 /*
@@ -672,6 +696,14 @@ static gint tx_deflate_write(txdrv_t *tx, gpointer data, gint len)
 {
 	struct attr *attr = (struct attr *) tx->opaque;
 
+	if (dbg > 9)
+		printf("tx_deflate_write: (%s) (buffer #%d, nagle %s, "
+			"unflushed %d) [%c%c]\n",
+			node_ip(tx->node), attr->fill_idx,
+			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed,
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
+
 	/*
 	 * If we're flow controlled or shut down, don't accept anything.
 	 */
@@ -693,6 +725,14 @@ static gint tx_deflate_writev(txdrv_t *tx, struct iovec *iov, gint iovcnt)
 	struct attr *attr = (struct attr *) tx->opaque;
 	gint sent = 0;
 
+	if (dbg > 9)
+		printf("tx_deflate_writev: (%s) (buffer #%d, nagle %s, "
+			"unflushed %d) [%c%c]\n",
+			node_ip(tx->node), attr->fill_idx,
+			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed,
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
+
 	while (iovcnt--) {
 		gint ret;
 
@@ -714,6 +754,14 @@ static gint tx_deflate_writev(txdrv_t *tx, struct iovec *iov, gint iovcnt)
 		iov++;
 	}
 
+	if (dbg > 9)
+		printf("tx_deflate_writev: (%s) sent %d bytes (buffer #%d, nagle %s, "
+			"unflushed %d) [%c%c]\n",
+			node_ip(tx->node), sent, attr->fill_idx,
+			(attr->flags & DF_NAGLE) ? "on" : "off", attr->unflushed,
+			(attr->flags & DF_FLOWC) ? 'C' : '-',
+			(attr->flags & DF_FLUSH) ? 'f' : '-');
+
 	return sent;
 }
 
@@ -760,6 +808,18 @@ static gint tx_deflate_pending(txdrv_t *tx)
 	return pending + tx_pending(attr->nd);
 }
 
+/*
+ * tx_deflate_bio_source
+ *
+ * Fetch the I/O source of the network driver.
+ */
+static struct bio_source *tx_deflate_bio_source(txdrv_t *tx)
+{
+	struct attr *attr = (struct attr *) tx->opaque;
+
+	return tx_bio_source(attr->nd);
+}
+
 struct txdrv_ops tx_deflate_ops = {
 	tx_deflate_init,		/* init */
 	tx_deflate_destroy,		/* destroy */
@@ -768,5 +828,6 @@ struct txdrv_ops tx_deflate_ops = {
 	tx_deflate_enable,		/* enable */
 	tx_deflate_disable,		/* disable */
 	tx_deflate_pending,		/* pending */
+	tx_deflate_bio_source,	/* bio_source */
 };
 
