@@ -1,4 +1,6 @@
 /*
+ * $Id: tx_deflate.h,v 1.5 2003/01/01 00:14:37 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  * Network driver -- compressing layer.
@@ -23,11 +25,12 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __tx_deflate_h__
-#define __tx_deflate_h__
+#ifndef _tx_deflate_h_
+#define _tx_deflate_h_
+
+#include "gnutella.h"
 
 #include "tx.h"
-#include "cq.h"
 
 struct txdrv_ops tx_deflate_ops;
 
@@ -39,7 +42,7 @@ struct tx_deflate_args {
 	cqueue_t *cq;				/* Callout queue to use */
 };
 
-#endif	/* __tx_deflate_h__ */
+#endif	/* _tx_deflate_h_ */
 
 /* vi: set ts=4: */
 
