@@ -1,4 +1,6 @@
 /*
+ * $Id: tx_link.h,v 1.4 2003/01/01 00:14:37 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  * Network driver -- link layer.
@@ -23,14 +25,14 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __tx_link_h__
-#define __tx_link_h__
+#ifndef _tx_link_h_
+#define _tx_link_h_
 
 #include "tx.h"
 
 struct txdrv_ops tx_link_ops;
 
-#endif	/* __tx_link_h__ */
+#endif	/* _tx_link_h_ */
 
 /* vi: set ts=4: */
 
