@@ -1,4 +1,6 @@
 /*
+ * $Id: url.h,v 1.10 2003/04/27 20:49:18 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  *----------------------------------------------------------------------
@@ -21,21 +23,22 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __url_h__
-#define __url_h__
+#ifndef _url_h_
+#define _url_h_
 
 #include <glib.h>
 
 /*
- * Public interaface.
+ * Public interface.
  */
 
 guchar *url_escape(guchar *url);
-gint url_escape_into(guchar *url, guchar *target, gint len);
+guchar *url_escape_query(guchar *url);
+gint url_escape_into(const guchar *url, guchar *target, gint len);
 guchar *url_escape_cntrl(guchar *url);
 guchar *url_unescape(guchar *url, gboolean inplace);
 
-#endif	/* __url_h__ */
+#endif	/* _url_h_ */
 
 /* vi: set ts=4: */
 
