@@ -1,4 +1,6 @@
 /*
+ * $Id: utf8.h,v 1.7 2003/05/12 16:59:36 cbiere Exp $
+ *
  * Copyright (c) 2002, Raphael Manfredi
  *
  * Unicode Transformation Format 8 bits.
@@ -23,17 +25,26 @@
  *----------------------------------------------------------------------
  */
 
-#ifndef __utf8_h__
-#define __utf8_h__
+#ifndef _utf8_h_
+#define _utf8_h_
 
 #include <glib.h>
 
-gint utf8_is_valid_char(guchar *s);
-gint utf8_is_valid_string(guchar *s, gint len);
+gint utf8_is_valid_char(const guchar *s);
+gint utf8_is_valid_string(const guchar *s, gint len);
 guint32 utf8_decode_char(guchar *s, gint len, gint *retlen, gboolean warn);
 gint utf8_to_iso8859(guchar *s, gint len, gboolean space);
 
-#endif	/* __utf8_h__ */
+#ifdef USE_GTK2
+/* 
+ * Only necessary for GTK+ 2.x version because it expects almost any string
+ * to be encoded as UTF-8.
+ */
+gchar *convert_to_utf8(gchar *str, size_t len, const gchar *charset);
+gchar *locale_to_utf8(gchar *str, size_t len);
+#endif
+
+#endif	/* _utf8_h_ */
 
 /* vi: set ts=4: */
 
